@@ -16,6 +16,8 @@
  ***************************************************************************/
 
 #include "config.h"
+
+#include <errno.h>
 #include <math.h>
 #include <stdlib.h>
 
@@ -26,6 +28,7 @@
 #include <QHBoxLayout>
 #include <QResizeEvent>
 #include <QScrollBar>
+#include <QtGlobal>
 #include <QWheelEvent>
 
 #include <klocale.h>
@@ -125,12 +128,12 @@ MainWidget::MainWidget(QWidget *parent)
     Q_ASSERT(m_overview);
     if (!m_overview) return;
     topLayout->addWidget(m_overview, 1, 1);
-    connect(&m_signal_widget, SIGNAL(selectedTimeInfo(unsigned int,
-	unsigned int, double)),
-	m_overview, SLOT(setSelection(unsigned int, unsigned int, double)));
+    connect(&m_signal_widget, SIGNAL(selectedTimeInfo(sample_index_t,
+	sample_index_t, double)),
+	m_overview, SLOT(setSelection(sample_index_t, sample_index_t, double)));
     m_overview->labelsChanged(m_signal_widget.signalManager().labels());
-    connect(&(playbackController()), SIGNAL(sigPlaybackPos(unsigned int)),
-	m_overview, SLOT(playbackPositionChanged(unsigned int)));
+    connect(&(playbackController()), SIGNAL(sigPlaybackPos(sample_index_t)),
+	m_overview, SLOT(playbackPositionChanged(sample_index_t)));
     connect(&(playbackController()), SIGNAL(sigPlaybackStopped()),
 	m_overview, SLOT(playbackStopped()));
 
@@ -145,12 +148,14 @@ MainWidget::MainWidget(QWidget *parent)
     m_vertical_scrollbar->hide();
     m_vertical_scrollbar->setFixedWidth(0);
     signalLayout->addWidget(m_vertical_scrollbar, 0, Qt::AlignRight);
+    m_vertical_scrollbar->hide();
 
     // -- signal widget --
 
     m_signal_widget.setMinimumSize(100, MIN_PIXELS_PER_CHANNEL);
 
     // -- do all the geometry management stuff --
+    m_overview->hide();
 
     topLayout->addWidget(frmChannelsBack, 0, 0);
 
@@ -161,6 +166,7 @@ MainWidget::MainWidget(QWidget *parent)
     topLayout->setColumnStretch(1, 1);
     signalLayout->activate();
     topLayout->activate();
+    m_horizontal_scrollbar->hide();
 
     // -- connect all signals from/to the signal widget --
 
@@ -169,27 +175,27 @@ MainWidget::MainWidget(QWidget *parent)
     connect(m_horizontal_scrollbar, SIGNAL(valueChanged(int)),
 	    this, SLOT(horizontalScrollBarMoved(int)));
 
-    connect(m_overview, SIGNAL(valueChanged(unsigned int)),
-	    &m_signal_widget, SLOT(setOffset(unsigned int)));
+    connect(m_overview, SIGNAL(valueChanged(sample_index_t)),
+	    &m_signal_widget, SLOT(setOffset(sample_index_t)));
     connect(m_overview, SIGNAL(sigCommand(const QString &)),
             this, SLOT(forwardCommand(const QString &)));
-    connect(&m_signal_widget, SIGNAL(viewInfo(unsigned int,
-	    unsigned int, unsigned int)),
-	    m_overview, SLOT(setRange(unsigned int, unsigned int,
-	    unsigned int)));
-    connect(&m_signal_widget, SIGNAL(viewInfo(unsigned int,
-	    unsigned int, unsigned int)),
-	    this, SLOT(updateViewInfo(unsigned int, unsigned int,
-	    unsigned int)));
+    connect(&m_signal_widget, SIGNAL(viewInfo(sample_index_t,
+	    unsigned int, sample_index_t)),
+	    m_overview, SLOT(setRange(sample_index_t, unsigned int,
+	    sample_index_t)));
+    connect(&m_signal_widget, SIGNAL(viewInfo(sample_index_t,
+	    unsigned int, sample_index_t)),
+	    this, SLOT(updateViewInfo(sample_index_t, unsigned int,
+	    sample_index_t)));
 
     connect(&m_signal_widget, SIGNAL(sigZoomChanged(double)),
 	    this, SIGNAL(sigZoomChanged(double)));
     connect(&m_signal_widget, SIGNAL(sigCommand(const QString &)),
 	    this, SLOT(forwardCommand(const QString &)));
-    connect(&m_signal_widget, SIGNAL(selectedTimeInfo(unsigned int,
-            unsigned int, double)),
-	    this, SIGNAL(selectedTimeInfo(unsigned int,
-            unsigned int, double)));
+    connect(&m_signal_widget, SIGNAL(selectedTimeInfo(sample_index_t,
+            sample_index_t, double)),
+	    this, SIGNAL(selectedTimeInfo(sample_index_t,
+            sample_index_t, double)));
     connect(&m_signal_widget, SIGNAL(sigTrackInserted(unsigned int)),
 	    this, SLOT(slotTrackInserted(unsigned int)));
     connect(&m_signal_widget, SIGNAL(sigTrackDeleted(unsigned int)),
@@ -197,8 +203,8 @@ MainWidget::MainWidget(QWidget *parent)
     connect(&m_signal_widget, SIGNAL(sigTrackSelectionChanged()),
 	    this, SLOT(refreshChannelControls()));
 
-    connect(&m_signal_widget, SIGNAL(sigMouseChanged(int)),
-	    this, SIGNAL(sigMouseChanged(int)));
+    connect(&m_signal_widget, SIGNAL(sigMouseChanged(Kwave::MouseMark::Mode)),
+	    this, SIGNAL(sigMouseChanged(Kwave::MouseMark::Mode)));
 
     refreshChannelControls();
 
@@ -226,7 +232,6 @@ void MainWidget::resizeEvent(QResizeEvent *)
     refreshChannelControls();
 }
 
-
 //***************************************************************************
 void MainWidget::wheelEvent(QWheelEvent *event)
 {
@@ -313,7 +318,7 @@ void MainWidget::closeSignal()
 }
 
 //***************************************************************************
-void MainWidget::newSignal(unsigned int samples, double rate,
+void MainWidget::newSignal(sample_index_t samples, double rate,
                            unsigned int bits, unsigned int tracks)
 {
     m_signal_widget.newSignal(samples, rate, bits, tracks);
@@ -341,9 +346,9 @@ void MainWidget::forwardCommand(const QString &command)
 }
 
 //***************************************************************************
-bool MainWidget::executeCommand(const QString &command)
+int MainWidget::executeCommand(const QString &command)
 {
-    if (!command.length()) return false;
+    if (!command.length()) return -EINVAL;
     Parser parser(command);
 
     if (false) {
@@ -361,7 +366,7 @@ bool MainWidget::executeCommand(const QString &command)
 	return m_signal_widget.executeCommand(command);
     }
 
-    return true;
+    return 0;
 }
 
 //***************************************************************************
@@ -467,7 +472,7 @@ void MainWidget::refreshChannelControls()
 }
 
 //***************************************************************************
-void MainWidget::updateViewInfo(unsigned int, unsigned int, unsigned int)
+void MainWidget::updateViewInfo(sample_index_t, unsigned int, sample_index_t)
 {
     refreshHorizontalScrollBar();
 }
@@ -479,6 +484,16 @@ void MainWidget::refreshHorizontalScrollBar()
 
     m_horizontal_scrollbar->blockSignals(true);
 
+    // show/hide the overview widget
+    if (!signalManager().isEmpty() && m_overview && !m_overview->isVisible()) {
+	m_overview->show();
+	m_horizontal_scrollbar->show();
+    }
+    if (signalManager().isEmpty() && m_overview && m_overview->isVisible()) {
+	m_overview->hide();
+	m_horizontal_scrollbar->hide();
+    }
+
     // adjust the limits of the horizontal scrollbar
     if (signalManager().length() > 1) {
 	// get the view information in samples
@@ -534,13 +549,13 @@ void MainWidget::horizontalScrollBarMoved(int newval)
     }
 
     // convert the current position into samples
-    unsigned int length = signalManager().length();
-    unsigned int visible = m_signal_widget.pixels2samples(displayWidth());
+    sample_index_t length = signalManager().length();
+    sample_index_t visible = m_signal_widget.pixels2samples(displayWidth());
     if (visible > length) visible = length;
-    unsigned int range   = length - visible;
+    sample_index_t range   = length - visible;
     if (range) range--;
 
-    unsigned int pos = static_cast<int>(floor(static_cast<double>(range) *
+    sample_index_t pos = static_cast<sample_index_t>(floor(static_cast<double>(range) *
 	static_cast<double>(newval) / static_cast<double>(max)));
 //     qDebug("horizontalScrollBarMoved(%d) -> %u", newval, pos);
     m_signal_widget.setOffset(pos);
