@@ -30,6 +30,7 @@
 #include <QFile>
 #include <QFrame>
 #include <QLabel>
+#include <QMap>
 #include <QPixmap>
 #include <QSizePolicy>
 #include <QStringList>
@@ -60,7 +61,7 @@
 
 #include "libgui/MenuManager.h"
 #include "libgui/KwaveFileDialog.h"
-#include "libgui/SignalWidget.h" // for MouseMode
+#include "libgui/MouseMark.h"
 
 #include "KwaveApp.h"
 #include "KwaveSplash.h"
@@ -106,8 +107,8 @@ TopWidget::TopWidget(KwaveApp &main_app)
      m_lbl_status_size(0), m_lbl_status_mode(0), m_lbl_status_cursor(0)
 {
     KIconLoader icon_loader;
-
-    showInSplashSreen(i18n("loading main menu..."));
+    
+    showInSplashSreen(i18n("Loading main menu..."));
     KMenuBar *menubar = menuBar();
     Q_ASSERT(menubar);
     if (!menubar) return;
@@ -174,17 +175,17 @@ TopWidget::TopWidget(KwaveApp &main_app)
     // connect the main widget
     connect(m_main_widget, SIGNAL(sigCommand(const QString &)),
             this, SLOT(executeCommand(const QString &)));
-    connect(m_main_widget, SIGNAL(selectedTimeInfo(unsigned int,
-            unsigned int, double)),
-            this, SLOT(setSelectedTimeInfo(unsigned int, unsigned int,
+    connect(m_main_widget, SIGNAL(selectedTimeInfo(sample_index_t,
+            sample_index_t, double)),
+            this, SLOT(setSelectedTimeInfo(sample_index_t, sample_index_t,
             double)));
     connect(m_main_widget, SIGNAL(sigTrackCount(unsigned int)),
             this, SLOT(setTrackInfo(unsigned int)));
-    connect(m_main_widget, SIGNAL(sigMouseChanged(int)),
-            this, SLOT(mouseChanged(int)));
+    connect(m_main_widget, SIGNAL(sigMouseChanged(Kwave::MouseMark::Mode)),
+            this, SLOT(mouseChanged(Kwave::MouseMark::Mode)));
     connect(&m_main_widget->playbackController(),
-            SIGNAL(sigPlaybackPos(unsigned int)),
-            this, SLOT(updatePlaybackPos(unsigned int)));
+            SIGNAL(sigPlaybackPos(sample_index_t)),
+            this, SLOT(updatePlaybackPos(sample_index_t)));
 
     // connect the sigCommand signal to ourself, this is needed
     // for the plugins
@@ -193,7 +194,7 @@ TopWidget::TopWidget(KwaveApp &main_app)
 
     // --- set up the toolbar ---
 
-    showInSplashSreen(i18n("initializing toolbar..."));
+    showInSplashSreen(i18n("Initializing toolbar..."));
     KToolBar *toolbar_file = toolBar("MainWidget File");
     Q_ASSERT(toolbar_file);
     if (!toolbar_file) return;
@@ -202,17 +203,17 @@ TopWidget::TopWidget(KwaveApp &main_app)
 
     toolbar_file->addAction(
 	icon_loader.loadIcon("document-new", KIconLoader::Toolbar),
-	i18n("create a new empty file"),
+	i18n("Create a new empty file"),
 	this, SLOT(toolbarFileNew()));
 
     toolbar_file->addAction(
 	icon_loader.loadIcon("document-open", KIconLoader::Toolbar),
-	i18n("open an existing file"),
+	i18n("Open an existing file"),
 	this, SLOT(toolbarFileOpen()));
 
     toolbar_file->addAction(
 	icon_loader.loadIcon("document-save", KIconLoader::Toolbar),
-	i18n("save the current file"),
+	i18n("Save the current file"),
 	this, SLOT(toolbarFileSave()));
 
     // --- edit, cut&paste ---
@@ -233,17 +234,17 @@ TopWidget::TopWidget(KwaveApp &main_app)
 
     toolbar_edit->addAction(
 	icon_loader.loadIcon("edit-cut", KIconLoader::Toolbar),
-	i18n("cut the current selection and move it to the clipboard"),
+	i18n("Cut the current selection and move it to the clipboard"),
 	this, SLOT(toolbarEditCut()));
 
     toolbar_edit->addAction(
 	icon_loader.loadIcon("edit-copy", KIconLoader::Toolbar),
-	i18n("copy the current selection to the clipboard"),
+	i18n("Copy the current selection to the clipboard"),
 	this, SLOT(toolbarEditCopy()));
 
     QAction *btPaste = toolbar_edit->addAction(
 	icon_loader.loadIcon("edit-paste", KIconLoader::Toolbar),
-	i18n("insert the content of clipboard"),
+	i18n("Insert the content of clipboard"),
 	this, SLOT(toolbarEditPaste()));
     btPaste->setEnabled(!ClipBoard::instance().isEmpty());
     connect(&ClipBoard::instance(), SIGNAL(clipboardChanged(bool)),
@@ -251,12 +252,12 @@ TopWidget::TopWidget(KwaveApp &main_app)
 
     toolbar_edit->addAction(
 	icon_loader.loadIcon("draw-eraser", KIconLoader::Toolbar),
-	i18n("mute the current selection"),
+	i18n("Mute the current selection"),
 	this, SLOT(toolbarEditErase()));
 
     toolbar_edit->addAction(
 	icon_loader.loadIcon("edit-delete", KIconLoader::Toolbar),
-	i18n("delete the current selection"),
+	i18n("Delete the current selection"),
 	this, SLOT(toolbarEditDelete()));
 
 //                  Zoom
@@ -275,37 +276,37 @@ TopWidget::TopWidget(KwaveApp &main_app)
 
     m_action_play = toolbar_playback->addAction(
 	QPixmap(xpm_play),
-	i18n("start playback"),
+	i18n("Start playback"),
 	playback, SLOT(playbackStart()));
 
     m_action_loop = toolbar_playback->addAction(
 	QPixmap(xpm_loop),
-	i18n("start playback and loop"),
+	i18n("Start playback and loop"),
 	playback, SLOT(playbackLoop()));
 
     m_action_pause = toolbar_playback->addAction(
 	QPixmap(xpm_pause),
-	i18n("pause playback"),
+	i18n("Pause playback"),
 	this, SLOT(pausePressed()));
 
     m_action_stop = toolbar_playback->addAction(
 	QPixmap(xpm_stop),
-	i18n("stop playback or loop"),
+	i18n("Stop playback or loop"),
 	playback, SLOT(playbackStop()));
 
     // --- zoom controls ---
-    m_zoom_factors.append(ZoomFactor("1 ms",              1L));
-    m_zoom_factors.append(ZoomFactor("10 ms",            10L));
-    m_zoom_factors.append(ZoomFactor("100 ms",          100L));
-    m_zoom_factors.append(ZoomFactor("1 sec",          1000L));
-    m_zoom_factors.append(ZoomFactor("10 sec",     10L*1000L));
-    m_zoom_factors.append(ZoomFactor("30 sec",     30L*1000L));
-    m_zoom_factors.append(ZoomFactor("1 min",   1L*60L*1000L));
-    m_zoom_factors.append(ZoomFactor("3 min",   3L*60L*1000L));
-    m_zoom_factors.append(ZoomFactor("5 min",   5L*60L*1000L));
-    m_zoom_factors.append(ZoomFactor("10 min", 10L*60L*1000L));
-    m_zoom_factors.append(ZoomFactor("30 min", 30L*60L*1000L));
-    m_zoom_factors.append(ZoomFactor("60 min", 60L*60L*1000L));
+    m_zoom_factors.append(ZoomFactor(i18n("%1 ms",   1),            1L));
+    m_zoom_factors.append(ZoomFactor(i18n("%1 ms",  10),           10L));
+    m_zoom_factors.append(ZoomFactor(i18n("%1 ms", 100),          100L));
+    m_zoom_factors.append(ZoomFactor(i18n("%1 sec",  1),         1000L));
+    m_zoom_factors.append(ZoomFactor(i18n("%1 sec", 10),     10L*1000L));
+    m_zoom_factors.append(ZoomFactor(i18n("%1 sec", 30),     30L*1000L));
+    m_zoom_factors.append(ZoomFactor(i18n("%1 min",  1),  1L*60L*1000L));
+    m_zoom_factors.append(ZoomFactor(i18n("%1 min",  3),  3L*60L*1000L));
+    m_zoom_factors.append(ZoomFactor(i18n("%1 min",  5),  5L*60L*1000L));
+    m_zoom_factors.append(ZoomFactor(i18n("%1 min", 10), 10L*60L*1000L));
+    m_zoom_factors.append(ZoomFactor(i18n("%1 min", 30), 30L*60L*1000L));
+    m_zoom_factors.append(ZoomFactor(i18n("%1 min", 60), 60L*60L*1000L));
 
     KToolBar *toolbar_zoom = toolBar("MainWidget Zoom");
     Q_ASSERT(toolbar_zoom);
@@ -313,34 +314,34 @@ TopWidget::TopWidget(KwaveApp &main_app)
 
     m_action_zoomselection = toolbar_zoom->addAction(
 	QPixmap(xpm_zoomrange),
-	i18n("zoom to selection"),
+	i18n("Zoom to selection"),
 	m_main_widget, SLOT(zoomSelection()));
 
     m_action_zoomin = toolbar_zoom->addAction(
 	QPixmap(xpm_zoomin),
-	i18n("zoom in"),
+	i18n("Zoom in"),
 	m_main_widget, SLOT(zoomIn()));
 
     m_action_zoomout = toolbar_zoom->addAction(
 	QPixmap(xpm_zoomout),
-	i18n("zoom out"),
+	i18n("Zoom out"),
 	m_main_widget, SLOT(zoomOut()));
 
     m_action_zoomnormal = toolbar_zoom->addAction(
 	QPixmap(xpm_zoomnormal),
-	i18n("zoom to 100%%"),
+	i18n("Zoom to 100%"),
 	m_main_widget, SLOT(zoomNormal()));
 
     m_action_zoomall = toolbar_zoom->addAction(
 	QPixmap(xpm_zoomall),
-	i18n("zoom to all"),
+	i18n("Zoom to all"),
 	m_main_widget, SLOT(zoomAll()));
 
     // zoom selection combo box
     m_zoomselect = new KComboBox(this);
     Q_ASSERT(m_zoomselect);
     if (!m_zoomselect) return;
-    m_zoomselect->setToolTip(i18n("select zoom factor"));
+    m_zoomselect->setToolTip(i18n("Select zoom factor"));
     m_zoomselect->setInsertPolicy(QComboBox::InsertAtTop);
     m_zoomselect->setEditable(false);
     foreach (ZoomFactor zoom, m_zoom_factors)
@@ -367,9 +368,9 @@ TopWidget::TopWidget(KwaveApp &main_app)
 
     // connect the signal manager
     SignalManager *signal_manager = &(m_main_widget->signalManager());
-    connect(signal_manager, SIGNAL(sigStatusInfo(unsigned int, unsigned int,
+    connect(signal_manager, SIGNAL(sigStatusInfo(sample_index_t, unsigned int,
 	double, unsigned int)),
-	this, SLOT(setStatusInfo(unsigned int, unsigned int,
+	this, SLOT(setStatusInfo(sample_index_t, unsigned int,
 	double, unsigned int)));
     connect(signal_manager, SIGNAL(sigUndoRedoInfo(const QString&,
 	const QString&)),
@@ -389,7 +390,7 @@ TopWidget::TopWidget(KwaveApp &main_app)
     connect(m_plugin_manager, SIGNAL(sigProgress(const QString &)),
             this, SLOT(showInSplashSreen(const QString &)));
 
-    showInSplashSreen(i18n("scanning plugins..."));
+    showInSplashSreen(i18n("Scanning plugins..."));
     m_plugin_manager->findPlugins();
 
     // set the MainWidget as the main view
@@ -422,7 +423,7 @@ TopWidget::TopWidget(KwaveApp &main_app)
     w = qMax(w, m_main_widget->minimumSize().width());
     w = qMax(w, m_main_widget->sizeHint().width());
     w = qMax(w, width());
-    h = qMax(m_main_widget->sizeHint().height(), w*6/10);
+    h = qMax(m_main_widget->sizeHint().height(), (w * 6) / 10);
     h = qMax(h, height());
     resize(w, h);
 
@@ -437,7 +438,7 @@ TopWidget::TopWidget(KwaveApp &main_app)
     showInSplashSreen(i18n("Loading plugins..."));
     statusBar()->showMessage(i18n("Loading plugins..."));
     m_plugin_manager->loadAllPlugins();
-    statusBar()->showMessage(i18n("Ready."), 1000);
+    statusBar()->showMessage(i18n("Ready"), 1000);
 
     setTrackInfo(0);
     updateMenu();
@@ -471,6 +472,7 @@ TopWidget::TopWidget(KwaveApp &main_app)
 	}
 	cfg.writeEntry("toolbars", magic);
     }
+  
 }
 
 //***************************************************************************
@@ -498,10 +500,10 @@ TopWidget::~TopWidget()
     m_pause_timer = 0;
 
     if (m_main_widget) delete m_main_widget;
-    m_main_widget=0;
+    m_main_widget = 0;
 
     if (m_menu_manager) delete m_menu_manager;
-    m_menu_manager=0;
+    m_menu_manager = 0;
 
     m_app.closeWindow(this);
 }
@@ -513,7 +515,7 @@ int TopWidget::executeCommand(const QString &line)
     bool use_recorder = true;
     QString command = line;
 
-//    qDebug("TopWidget::executeCommand(%s)", command.toLocal8Bit().data()); // ###
+//    qDebug("TopWidget::executeCommand(%s)", command.toLocal8Bit().data());
     if (!command.length()) return 0; // empty line -> nothing to do
     if (command.trimmed().startsWith("#")) return 0; // only a comment
 
@@ -560,16 +562,16 @@ int TopWidget::executeCommand(const QString &line)
 
     // all others only if no plugin is currently running
     if (m_plugin_manager && m_plugin_manager->onePluginRunning()) {
-	qWarning("TopWidget::executeCommand(...) - currently not possible, "\
-		"a plugin is running :-(");
-	return false;
+	qWarning("TopWidget::executeCommand('%s') - currently not possible, "\
+		"a plugin is running :-(",
+		parser.command().toLocal8Bit().data());
+	return -1;
     }
 
     if (use_recorder) {
 	// append the command to the macro recorder
 	// @TODO macro recording...
-	qDebug("TopWidget::executeCommand() >>> %s <<<",
-	       command.toLocal8Bit().data()); // ###
+	qDebug("# %s ", command.toLocal8Bit().data());
     }
 
     if (m_app.executeCommand(command)) {
@@ -617,10 +619,10 @@ int TopWidget::executeCommand(const QString &line)
 	Q_ASSERT(m_menu_manager);
 	if (m_menu_manager) /*result = */m_menu_manager->executeCommand(command);
     CASE_COMMAND("newsignal")
-	unsigned int samples = parser.toUInt();
-	double       rate    = parser.toDouble();
-	unsigned int bits    = parser.toUInt();
-	unsigned int tracks  = parser.toUInt();
+	sample_index_t samples = parser.toUInt();
+	double         rate    = parser.toDouble();
+	unsigned int   bits    = parser.toUInt();
+	unsigned int   tracks  = parser.toUInt();
 	result = newSignal(samples, rate, bits, tracks);
     CASE_COMMAND("open")
 	QString filename = parser.nextParam();
@@ -642,31 +644,41 @@ int TopWidget::executeCommand(const QString &line)
     CASE_COMMAND("saveas")
 	result = saveFileAs(false);
     CASE_COMMAND("loadbatch")
-	result = loadBatch(command);
+	result = loadBatch(parser.nextParam());
     CASE_COMMAND("saveselect")
 	result = saveFileAs(true);
     CASE_COMMAND("quit")
-	result = close();
+	result = (close()) ? 0 : -1;
     } else {
+	// try to forward the command to the main widget
 	Q_ASSERT(m_main_widget);
-	result = (m_main_widget &&
-	    m_main_widget->executeCommand(command)) ? 0 : -1;
+	return (m_main_widget) ? m_main_widget->executeCommand(command) : -1;
     }
 
     return result;
 }
 
 //***************************************************************************
-int TopWidget::loadBatch(const QString &str)
+int TopWidget::loadBatch(const KUrl &url)
 {
-    Parser parser(str);
-    QFile file(parser.firstParam());
+    // open the URL, read-only mode is enough
+    QFile file(url.path());
+    if (!file.open(QIODevice::ReadOnly)) {
+	qWarning("unable to open source in read-only mode!");
+	return -EIO;
+    }
 
-    file.open(QIODevice::ReadOnly);
+    // use a text stream for parsing the commands
     QTextStream stream(&file);
     int result = parseCommands(stream);
+
     file.close();
 
+    // successful run -> add URL to recent files
+    m_app.addRecentFile(url.path());
+    updateMenu();
+    updateToolbar();
+
     return result;
 }
 
@@ -697,10 +709,80 @@ int TopWidget::executePlaybackCommand(const QString &command)
 int TopWidget::parseCommands(QTextStream &stream)
 {
     int result = 0;
+    QMap<QString, qint64> labels;
+
+    // set hourglass cursor
+    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
     while (!stream.atEnd() && !result) {
-	QString line = stream.readLine();
+	QString line = stream.readLine().simplified();
+	if (line.startsWith("#")) continue; // skip comments
+	if (!line.length()) continue;       // skip empty lines
+
+	// remove stuff after the "#'" (comments)
+	if (line.contains('#')) {
+	}
+
+	if (line.endsWith(':')) {
+	    // this line seems to be a "label"
+	    line = line.left(line.length() - 1).simplified();
+	    if (!labels.contains(line)) {
+		qDebug("new label '%s' at %u", line.toLocal8Bit().data(),
+		       static_cast<unsigned int>(stream.pos()));
+		labels[line] = stream.pos();
+	    }
+	    continue;
+	}
+
+	Parser parser(line);
+
+	// the "goto" command
+	if (line.split(' ').at(0) == "goto") {
+	    qDebug(">>> detected 'goto'");
+	    QString label = line.split(' ').at(1).simplified();
+	    if (labels.contains(label)) {
+		qDebug(">>> goto '%s' @ offset %u", label.toLocal8Bit().data(),
+		       static_cast<unsigned int>(labels[label]));
+		stream.seek(labels[label]);
+	    } else {
+		qWarning("label '%s' not found", label.toLocal8Bit().data());
+		break;
+	    }
+	    continue;
+	}
+
+	// synchronize before the command
+	if (m_plugin_manager) m_plugin_manager->sync();
+
+	// the "msgbox" command (useful for debugging)
+	if (parser.command() == "msgbox") {
+	    QApplication::restoreOverrideCursor();
+	    result = (Kwave::MessageBox::questionYesNo(this, 
+		parser.firstParam()) == KMessageBox::Yes) ? 0 : 1;
+	    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+	    continue;
+	}
+
+	// prevent this command from being re-added to the macro recorder
+	if (!line.startsWith("nomacro:", Qt::CaseInsensitive))
+	    line = "nomacro:" + line;
+
+	// emit the command
 	result = executeCommand(line);
+// 	qDebug(">>> '%s' - result=%d", line.toLocal8Bit().data(), result);
+
+	// synchronize after the command
+	if (m_plugin_manager) m_plugin_manager->sync();
+	
+	// special handling of the "quit" command
+	if (parser.command() == "quit") {
+	    break;
+	}
     }
+
+    // remove hourglass
+    QApplication::restoreOverrideCursor();
+
     return result;
 }
 
@@ -763,6 +845,13 @@ int TopWidget::loadFile(const KUrl &url)
     // abort if new file not valid and local
     if (!url.isLocalFile()) return -1;
 
+    // detect whether it is a macro (batch) file
+    QFileInfo file(url.fileName());
+    QString suffix = file.suffix();
+    if (suffix == "kwave") {
+	return loadBatch(url);
+    }
+
     // try to close the previous file
     if (!closeFile()) return -1;
 
@@ -796,7 +885,7 @@ int TopWidget::openRecent(const QString &str)
 int TopWidget::openFile()
 {
     QString filter = CodecManager::decodingFilter();
-    KwaveFileDialog dlg(":<kwave_open_dir>", filter, this, true);
+    KwaveFileDialog dlg("kfiledialog:///kwave_open_dir", filter, this, true);
     dlg.setMode(static_cast<KFile::Modes>(KFile::File | KFile::ExistingOnly));
     dlg.setOperationMode(KFileDialog::Opening);
     dlg.setCaption(i18n("Open"));
@@ -841,7 +930,7 @@ int TopWidget::saveFileAs(bool selection)
 
     KUrl current_url;
     current_url = signalName();
-    KwaveFileDialog dlg(":<kwave_save_as>", CodecManager::encodingFilter(),
+    KwaveFileDialog dlg("kfiledialog:///kwave_save_as", CodecManager::encodingFilter(),
         this, true, current_url.prettyUrl(), "*.wav");
     dlg.setOperationMode(KFileDialog::Saving);
     dlg.setCaption(i18n("Save As"));
@@ -929,7 +1018,7 @@ int TopWidget::saveFileAs(bool selection)
 }
 
 //***************************************************************************
-int TopWidget::newSignal(unsigned int samples, double rate,
+int TopWidget::newSignal(sample_index_t samples, double rate,
                          unsigned int bits, unsigned int tracks)
 {
     // abort if the user pressed cancel
@@ -967,8 +1056,8 @@ void TopWidget::selectZoom(int index)
 	index--;
     }
 
-    const double rate = signalManager().rate();
-    const double ms = m_zoom_factors[index].second;
+    const double rate  = signalManager().rate();
+    const double ms    = m_zoom_factors[index].second;
     unsigned int width = m_main_widget->displayWidth();
     Q_ASSERT(width > 1);
     if (width <= 1) width = 2;
@@ -1036,9 +1125,10 @@ void TopWidget::setZoomInfo(double zoom)
 }
 
 //***************************************************************************
-void TopWidget::setStatusInfo(unsigned int length, unsigned int /*tracks*/,
+void TopWidget::setStatusInfo(sample_index_t length, unsigned int tracks,
                               double rate, unsigned int bits)
 {
+    Q_UNUSED(tracks);
     Q_ASSERT(statusBar());
     Q_ASSERT(m_menu_manager);
     if (!statusBar() || !m_menu_manager) return;
@@ -1049,9 +1139,11 @@ void TopWidget::setStatusInfo(unsigned int length, unsigned int /*tracks*/,
     if (length) {
 	ms = (rate) ? (static_cast<double>(length) /
 	    static_cast<double>(rate) * 1E3) : 0;
-	txt = " " + i18n("Length: %1", Kwave::Plugin::ms2string(ms)) +
-	    " " + i18n("(%1 samples)", Kwave::Plugin::dottedNumber(length)) +
-	    " ";
+	txt = " " + i18nc(
+	    "Length, as in total duration of loaded song",
+	    "Length: %1 (%2 samples)",
+	    Kwave::Plugin::ms2string(ms), Kwave::Plugin::dottedNumber(length)
+	) + " ";
     } else txt = "";
     m_lbl_status_size->setText(txt);
 
@@ -1059,7 +1151,7 @@ void TopWidget::setStatusInfo(unsigned int length, unsigned int /*tracks*/,
     if (bits) {
 	QString khz = "%0.3f";
 	khz = khz.sprintf("%0.3f", static_cast<double>(rate) * 1E-3);
-	txt = " "+i18n("Mode: %1 kHz@%2 bit", khz, bits)+" ";
+	txt = " " + i18n("Mode: %1 kHz @ %2 Bit", khz, bits) + " ";
     } else txt = "";
     m_lbl_status_mode->setText(txt);
 
@@ -1084,13 +1176,19 @@ void TopWidget::setTrackInfo(unsigned int tracks)
     m_menu_manager->setItemEnabled("@SIGNAL", have_signal);
 
     // revert is not possible if no signal at all is present
-    if (!have_signal)
+    if (!have_signal) {
 	m_menu_manager->setItemEnabled("ID_FILE_REVERT", false);
+    }
 
+    // remove selection/position display on file close
+    if (!have_signal) setSelectedTimeInfo(0, 0, 0);
+
+    // update the toolbar as well
+    updateToolbar();
 }
 
 //***************************************************************************
-void TopWidget::setSelectedTimeInfo(unsigned int offset, unsigned int length,
+void TopWidget::setSelectedTimeInfo(sample_index_t offset, sample_index_t length,
                                     double rate)
 {
     Q_ASSERT(statusBar());
@@ -1098,26 +1196,34 @@ void TopWidget::setSelectedTimeInfo(unsigned int offset, unsigned int length,
 
     if (length > 1) {
 	// show offset and length
-	// Selected: 02:00...05:00 (3 min)
 	// Selected: 2000...3000 (1000 samples)
+	// Selected: 02:00...05:00 (3 min)
 	bool sample_mode = false;
 
-	unsigned int last = offset + ((length) ? length-1 : 0);
+	sample_index_t last = offset + ((length) ? length-1 : 0);
 	if (rate == 0) sample_mode = true; // force sample mode if rate==0
-	QString txt = " "+i18n("Selected")+": %1...%2 (%3)";
+	QString txt = " ";
 	if (sample_mode) {
-	    txt = txt.arg(
-	          Kwave::Plugin::dottedNumber(offset)).arg(
-	          Kwave::Plugin::dottedNumber(last)).arg(
-	          Kwave::Plugin::dottedNumber(length) + " " +
-	          i18n("samples"));
+	    txt += i18nc(
+	        "%1=first sample, %2=last sample, %3=number of samples, "\
+	        "example: 'Selected: 2000...3000 (1000 samples)'",
+	        "Selected: %1...%2 (%3 samples)",
+	        Kwave::Plugin::dottedNumber(offset),
+	        Kwave::Plugin::dottedNumber(last),
+	        Kwave::Plugin::dottedNumber(length)
+	    );
 	} else {
 	    double ms_first = static_cast<double>(offset)   * 1E3 / rate;
 	    double ms_last  = static_cast<double>(last + 1) * 1E3 / rate;
 	    double ms = (ms_last - ms_first);
-	    txt = txt.arg(Kwave::Plugin::ms2string(ms_first)).arg(
-		Kwave::Plugin::ms2string(ms_last)).arg(
-		Kwave::Plugin::ms2string(ms));
+	    txt += i18nc(
+	        "%1=start time, %2=end time, %3=time span, "\
+	        "example: 'Selected: 02:00...05:00 (3 min)'",
+	        "Selected: %1...%2 (%3)",
+	        Kwave::Plugin::ms2string(ms_first),
+	        Kwave::Plugin::ms2string(ms_last),
+	        Kwave::Plugin::ms2string(ms)
+	    );
 	}
 
 	m_lbl_status_cursor->setText("");
@@ -1129,12 +1235,12 @@ void TopWidget::setSelectedTimeInfo(unsigned int offset, unsigned int length,
 	bool sample_mode = false;
 
 	if (rate == 0) sample_mode = true; // force sample mode if rate==0
-	if (sample_mode) {
+	if (sample_mode || !signalManager().tracks()) {
 	    m_lbl_status_cursor->setText("");
 	} else {
-	    QString txt = i18n("Position")+": %1";
 	    double ms_first = static_cast<double>(offset) * 1E3 / rate;
-	    txt = txt.arg(Kwave::Plugin::ms2string(ms_first));
+	    QString txt = i18n("Position: %1",
+		Kwave::Plugin::ms2string(ms_first));
 	    m_lbl_status_cursor->setText(txt);
 	}
 
@@ -1143,22 +1249,21 @@ void TopWidget::setSelectedTimeInfo(unsigned int offset, unsigned int length,
 }
 
 //***************************************************************************
-void TopWidget::updatePlaybackPos(unsigned int offset)
+void TopWidget::updatePlaybackPos(sample_index_t offset)
 {
     if (!m_plugin_manager) return;
     if (!m_main_widget) return;
 
     bool playing = m_main_widget->playbackController().running();
     if (!playing) return;
-
-    QString txt = i18n("Playback")+": %1";
+    QString txt;
     double rate = m_plugin_manager->signalRate();
     if (rate > 0) {
 	double ms = static_cast<double>(offset) * 1E3 / rate;
-	txt = txt.arg(Kwave::Plugin::ms2string(ms));
+	txt = i18n("Playback: %1", Kwave::Plugin::ms2string(ms));
     } else {
-	txt = txt.arg(Kwave::Plugin::dottedNumber(offset) + " " +
-	    i18n("samples"));
+	txt = i18n("Playback: %1 samples",
+	            Kwave::Plugin::dottedNumber(offset));
     }
     statusBar()->showMessage(txt, 2000);
 }
@@ -1172,16 +1277,22 @@ void TopWidget::setUndoRedoInfo(const QString &undo, const QString &redo)
 
     // set the state and tooltip of the undo toolbar button
     if (m_action_undo) {
-	txt = i18n("Undo");
-	if (undo_enabled) txt += " (" + undo + ")";
+	txt = (undo_enabled) ?
+	    i18nc("tooltip of the undo toolbar button if undo enabled",
+		  "Undo (%1)", undo) :
+	    i18nc("tooltip of the undo toolbar button if undo disabled",
+		  "Undo");
 	m_action_undo->setToolTip(txt);
 	m_action_undo->setEnabled(undo_enabled);
     }
 
     // set the state and tooltip of the redo toolbar button
     if (m_action_redo) {
-	txt = i18n("Redo");
-	if (redo_enabled) txt += " (" + redo + ")";
+	txt = (redo_enabled) ?
+	    i18nc("tooltip of the redo toolbar button, redo enabled",
+		  "Redo (%1)", redo) :
+	    i18nc("tooltip of the redo toolbar button, redo disabled",
+		  "Redo");
 	m_action_redo->setToolTip(txt);
 	m_action_redo->setEnabled(redo_enabled);
     }
@@ -1191,23 +1302,25 @@ void TopWidget::setUndoRedoInfo(const QString &undo, const QString &redo)
 
     // set new enable and text of the undo menu entry
     m_menu_manager->setItemEnabled("ID_EDIT_UNDO", undo_enabled);
-    txt = i18n("U&ndo");
-    if (undo_enabled) txt += " (" + undo + ")";
+    txt = (undo_enabled) ?
+	i18nc("menu entry for undo if undo enabled",  "U&ndo (%1)", undo) :
+	i18nc("menu entry for undo if undo disabled", "U&ndo");
     m_menu_manager->setItemText("ID_EDIT_UNDO", txt);
 
     // set new enable and text of the undo menu entry
     m_menu_manager->setItemEnabled("ID_EDIT_REDO", redo_enabled);
-    txt = i18n("R&edo");
-    if (redo_enabled) txt += " (" + redo + ")";
+    txt = (redo_enabled) ?
+	i18nc("menu entry for redo if redo enabled",  "R&edo (%1)", redo) :
+	i18nc("menu entry for redo if redo disabled", "R&edo");
     m_menu_manager->setItemText("ID_EDIT_REDO", txt);
 }
 
 //***************************************************************************
-void TopWidget::mouseChanged(int mode)
+void TopWidget::mouseChanged(Kwave::MouseMark::Mode mode)
 {
-    switch (static_cast<SignalWidget::MouseMode>(mode)) {
-	case (SignalWidget::MouseAtSelectionBorder) :
-	case (SignalWidget::MouseInSelection) :
+    switch (mode) {
+	case (Kwave::MouseMark::MouseAtSelectionBorder) :
+	case (Kwave::MouseMark::MouseInSelection) :
 	{
 	    double rate         = signalManager().rate();
 	    unsigned int offset = signalManager().selection().offset();
@@ -1389,12 +1502,16 @@ void TopWidget::updateCaption()
 
     // shortcut if no file loaded
     if (signalName().length() == 0) {
-	setCaption(0);
+	setCaption(QString());
 	return;
     }
 
     if (modified)
-	setCaption("* "+signalName()+i18n(" (modified)"));
+	setCaption(i18nc(
+	    "%1 = Path to modified file",
+	    "* %1 (modified)",
+	    signalName())
+	);
     else
 	setCaption(signalName());
 }
