@@ -33,8 +33,8 @@
 #define CACHE_SIZE 8192           /* number of cache entries */
 
 //***************************************************************************
-OverViewCache::OverViewCache(SignalManager &signal, unsigned int src_offset,
-                             unsigned int src_length,
+OverViewCache::OverViewCache(SignalManager &signal, sample_index_t src_offset,
+                             sample_index_t src_length,
                              const QList<unsigned int> *src_tracks)
     :m_signal(signal), m_min(), m_max(), m_state(), m_count(0), m_scale(1),
      m_lock(QMutex::Recursive), m_src_offset(src_offset),
@@ -47,18 +47,18 @@ OverViewCache::OverViewCache(SignalManager &signal, unsigned int src_offset,
             this, SLOT(slotTrackInserted(unsigned int, Track *)));
     connect(sig, SIGNAL(sigTrackDeleted(unsigned int)),
             this, SLOT(slotTrackDeleted(unsigned int)));
-    connect(sig, SIGNAL(sigSamplesDeleted(unsigned int, unsigned int,
-	unsigned int)),
-	this, SLOT(slotSamplesDeleted(unsigned int, unsigned int,
-	unsigned int)));
-    connect(sig, SIGNAL(sigSamplesInserted(unsigned int, unsigned int,
-	unsigned int)),
-	this, SLOT(slotSamplesInserted(unsigned int, unsigned int,
-	unsigned int)));
-    connect(sig, SIGNAL(sigSamplesModified(unsigned int, unsigned int,
-	unsigned int)),
-	this, SLOT(slotSamplesModified(unsigned int, unsigned int,
-	unsigned int)));
+    connect(sig, SIGNAL(sigSamplesDeleted(unsigned int, sample_index_t,
+	sample_index_t)),
+	this, SLOT(slotSamplesDeleted(unsigned int, sample_index_t,
+	sample_index_t)));
+    connect(sig, SIGNAL(sigSamplesInserted(unsigned int, sample_index_t,
+	sample_index_t)),
+	this, SLOT(slotSamplesInserted(unsigned int, sample_index_t,
+	sample_index_t)));
+    connect(sig, SIGNAL(sigSamplesModified(unsigned int, sample_index_t,
+	sample_index_t)),
+	this, SLOT(slotSamplesModified(unsigned int, sample_index_t,
+	sample_index_t)));
 
     if (src_tracks && !src_tracks->isEmpty()) {
 	// already having a list of selected tracks
@@ -86,7 +86,7 @@ OverViewCache::~OverViewCache()
 }
 
 //***************************************************************************
-unsigned int OverViewCache::sourceLength()
+sample_index_t OverViewCache::sourceLength()
 {
     return (m_src_length) ? m_src_length : m_signal.length();
 }
@@ -100,7 +100,7 @@ void OverViewCache::scaleUp()
     if (!m_scale) return;
 
     // calculate the new scale
-    const unsigned int len = sourceLength();
+    const sample_index_t len = sourceLength();
     unsigned int shrink = len / (m_scale * CACHE_SIZE);
     Q_ASSERT(shrink);
     while (len > CACHE_SIZE * m_scale * shrink) {
@@ -162,15 +162,15 @@ void OverViewCache::scaleDown()
 {
     QMutexLocker lock(&m_lock);
 
-    const unsigned int len = sourceLength();
+    const sample_index_t len = sourceLength();
     unsigned int new_scale = static_cast<unsigned int>(
-	rint(ceil(len/CACHE_SIZE)));
+	rint(ceil(len / CACHE_SIZE)));
     if (!new_scale) new_scale = 1;
     if (m_scale == new_scale) return;
 
     m_scale = new_scale;
     for (int track = 0; track < m_state.count(); ++track) {
-	invalidateCache(track, 0, len / m_scale);
+	invalidateCache(track, 0, (len / m_scale) + 1);
     }
 }
 
@@ -200,7 +200,7 @@ void OverViewCache::invalidateCache(unsigned int track, unsigned int first,
 
     QVector<CacheState> &state = m_state[cache_track];
 
-    if (last >= CACHE_SIZE) last = CACHE_SIZE-1;
+    if (last >= CACHE_SIZE) last = CACHE_SIZE - 1;
     unsigned int pos;
     for (pos = first; pos <= last; ++pos) {
 	state[pos] = Invalid;
@@ -262,9 +262,9 @@ void OverViewCache::slotTrackInserted(unsigned int index, Track *)
 
     // mark the new cache content as invalid
     if (sourceLength()) {
-	invalidateCache(index, 0, (sourceLength() / m_scale) - 1);
+	invalidateCache(index, 0, (sourceLength() / m_scale) + 1);
     } else {
-	invalidateCache(index, 0, CACHE_SIZE-1);
+	invalidateCache(index, 0, CACHE_SIZE - 1);
     }
 
     emit changed();
@@ -307,7 +307,7 @@ void OverViewCache::slotTrackDeleted(unsigned int index)
 
 //***************************************************************************
 void OverViewCache::slotSamplesInserted(unsigned int track,
-    unsigned int offset, unsigned int length)
+    sample_index_t offset, sample_index_t length)
 {
     QMutexLocker lock(&m_lock);
 
@@ -318,7 +318,7 @@ void OverViewCache::slotSamplesInserted(unsigned int track,
     if (!m_src_tracks.isEmpty() && !(m_src_tracks.contains(track))) return;
 
     // right out of our range -> out of interest
-    const unsigned int len = sourceLength();
+    const sample_index_t len = sourceLength();
     if (offset >= (m_src_offset + ((len) ? (len - 1) : 1))) return;
 
     // left from us -> just move our own offset right
@@ -332,23 +332,15 @@ void OverViewCache::slotSamplesInserted(unsigned int track,
     if (m_src_length) m_src_length += length;
     if ((sourceLength() / m_scale) > CACHE_SIZE) scaleUp();
 
-    unsigned int first = (offset - m_src_offset) / m_scale;
-    unsigned int last  = sourceLength() / m_scale;
-    if (last != first) last--;
-
-//    static unsigned int last_length = 0;
-//    if (sourceLength() != last_length) {
-//	last_length = sourceLength();
-//	first = 0;
-//    }
-//
+    sample_index_t first = (offset - m_src_offset) / m_scale;
+    sample_index_t last  = (sourceLength() / m_scale) + 1;
     invalidateCache(track, first, last);
     emit changed();
 }
 
 //***************************************************************************
 void OverViewCache::slotSamplesDeleted(unsigned int track,
-    unsigned int offset, unsigned int length)
+    sample_index_t offset, sample_index_t length)
 {
     QMutexLocker lock(&m_lock);
 
@@ -364,15 +356,15 @@ void OverViewCache::slotSamplesDeleted(unsigned int track,
     if (offset > (m_src_offset + sourceLength())) return;
 
     // completely left from us -> just move our own offset left
-    if (offset+length <= m_src_offset) {
+    if (offset + length <= m_src_offset) {
 	m_src_offset -= length;
 	return;
     }
 
     // overlapping
     if (offset < m_src_offset) {
-	unsigned int overlap = (offset+length) - m_src_offset;
-	m_src_offset -= length-overlap;
+	sample_index_t overlap = (offset + length) - m_src_offset;
+	m_src_offset -= length - overlap;
 	length -= overlap;
 	offset = m_src_offset;
     }
@@ -385,16 +377,16 @@ void OverViewCache::slotSamplesDeleted(unsigned int track,
 	else
 	    m_src_length = 1;
     }
-    unsigned int first = (offset-m_src_offset) / m_scale;
-    unsigned int last  = sourceLength() / m_scale;
-    if (last != first) last--;
+
+    unsigned int first = (offset - m_src_offset) / m_scale;
+    unsigned int last  = (sourceLength() / m_scale) + 1;
     invalidateCache(track, first, last);
     emit changed();
 }
 
 //***************************************************************************
 void OverViewCache::slotSamplesModified(unsigned int track,
-    unsigned int offset, unsigned int length)
+    sample_index_t offset, sample_index_t length)
 {
     QMutexLocker lock(&m_lock);
 
@@ -407,18 +399,18 @@ void OverViewCache::slotSamplesModified(unsigned int track,
     if (offset > (m_src_offset + sourceLength())) return;
 
     // completely left from us -> out of interest
-    if (offset+length < m_src_offset) return;
+    if (offset + length < m_src_offset) return;
 
     // overlapping
-    unsigned int first = offset;
-    unsigned int last  = offset + length - 1;
+    sample_index_t first = offset;
+    sample_index_t last  = offset + length - 1;
     if (first < m_src_offset) first = m_src_offset;
-    if (last > m_src_offset+sourceLength()-1)
-        last = m_src_offset+sourceLength()-1;
+    if (last > m_src_offset + sourceLength() - 1)
+        last = m_src_offset + sourceLength() - 1;
     first -= m_src_offset;
     last  -= m_src_offset;
     first = static_cast<unsigned int>(floor(first / m_scale));
-    last  = static_cast<unsigned int>(ceil(last / m_scale));
+    last  = static_cast<unsigned int>(ceil(last   / m_scale));
     invalidateCache(track, first, last);
     emit changed();
 }
@@ -454,13 +446,13 @@ QImage OverViewCache::getOverView(int width, int height,
 
     MultiTrackReader src(Kwave::SinglePassForward,
 	m_signal, track_list, m_src_offset,
-	m_src_offset+length-1);
+	m_src_offset + length - 1);
     Q_ASSERT(m_min.count() == m_max.count());
     Q_ASSERT(m_min.count() == m_state.count());
 
     // abort if the track count has recently changed
     if (m_state.count() != static_cast<int>(src.tracks())) {
-	qWarning("OverViewCache::getOverView(): track count has changed");
+// 	qWarning("OverViewCache::getOverView(): track count has changed");
 	p.end();
 	return bitmap;
     }
@@ -489,7 +481,7 @@ QImage OverViewCache::getOverView(int width, int height,
 	}
     }
 
-    if ((width < 2) || (height < 2) || (length/m_scale < 2)) {
+    if ((width < 2) || (height < 2) || (length / m_scale < 2)) {
 	p.end();
 	return bitmap; // empty ?
     }
@@ -522,7 +514,11 @@ QImage OverViewCache::getOverView(int width, int height,
 		CacheState *state = m_state[t].data();
 		Q_ASSERT(state);
 		if (!state) continue;
-		if (state[index] != Valid) continue;
+		if (state[index] != Valid) {
+		    if (minimum > 0) minimum = 0;
+		    if (maximum < 0) maximum = 0;
+		    continue;
+		}
 
 		if (min[index] < minimum) minimum = min[index];
 		if (max[index] > maximum) maximum = max[index];
