@@ -43,19 +43,33 @@ CodecBase::~CodecBase()
 void CodecBase::addMimeType(const QString &name, const QString &description,
                             const QString &patterns)
 {
+    // check for duplicates
+    if (supports(name)) return;
+
     CodecBase::MimeType type;
     KMimeType::Ptr t = KMimeType::mimeType(name);
 
     if (!t || (t && t->isDefault())) {
-	qWarning("mime type '%s' not registered, using built-in!",
-	         name.toLocal8Bit().data());
+// 	qWarning("mime type '%s' not registered, using built-in!",
+// 	         name.toLocal8Bit().data());
 	type.name        = name;
 	type.description = description;
 	type.patterns    = patterns.split("; ", QString::SkipEmptyParts);
     } else {
-	type.name        = t->name();
 	type.description = t->comment();
 	type.patterns    = t->patterns();
+
+	if (t->name() != name) {
+	    // type has been translated (maybe un-alias'ed)
+	    // manually add the original name
+	    type.name    = name;
+	    m_supported_mime_types.append(type);
+	}
+
+	if  (!supports(t->name())) {
+	    // new type or new alias
+	    type.name        = t->name();
+	}
     }
     m_supported_mime_types.append(type);
 }
@@ -69,9 +83,7 @@ bool CodecBase::supports(const KMimeType &mimetype)
 /***************************************************************************/
 bool CodecBase::supports(const QString &mimetype_name)
 {
-    QListIterator<CodecBase::MimeType> it(m_supported_mime_types);
-    while (it.hasNext()) {
-	CodecBase::MimeType mime = it.next();
+    foreach (const CodecBase::MimeType &mime, m_supported_mime_types) {
 	if (mime.name == mimetype_name) return true;
     }
     return false;
