@@ -16,7 +16,7 @@
  ***************************************************************************/
 
 #include "config.h"
-#include <float.h>
+#include <float.h> // for FLT_MAX, DBL_MIN, DBL_MAX
 #include <math.h>
 
 #include <QtAlgorithms>
