@@ -27,13 +27,14 @@
 
 #include <kdemacros.h>
 
+#include "libkwave/Sample.h"
 #include "libkwave/TypesMap.h"
 #include "libkwave/LabelList.h"
 
 /**
  * @enum FileProperty
  * Enumeration type that lists all file properties. If you extend this list,
- * please don't forget to add a verbose name and a description it in
+ * please don't forget to add a verbose name and a description of it in
  * FileInfo.cpp
  */
 typedef enum {
@@ -45,6 +46,7 @@ typedef enum {
     INF_BITRATE_LOWER,       /**< lower bitrate limit */
     INF_BITRATE_NOMINAL,     /**< nominal bitrate */
     INF_BITRATE_UPPER,       /**< upper bitrate limit */
+    INF_BITS_PER_SAMPLE,     /**< number of bits per sample */
     INF_CD,                  /**< number of the CD in an album */
     INF_COMMISSIONED,        /**< commissioned */
     INF_COMMENTS,            /**< comments */
@@ -60,6 +62,7 @@ typedef enum {
     INF_ISRC,                /**< International Standard Recording Code */
     INF_KEYWORDS,            /**< keywords */
     INF_LABELS,              /**< labels/markers */
+    INF_LENGTH,              /**< length of the file in samples */
     INF_LICENSE,             /**< license information */
     INF_MEDIUM,              /**< medium */
     INF_MIMETYPE,            /**< mime type of the file format */
@@ -74,12 +77,14 @@ typedef enum {
     INF_PRIVATE,             /**< "private" bit */
     INF_PRODUCT,             /**< product */
     INF_SAMPLE_FORMAT,       /**< sample format (libaudiofile) */
+    INF_SAMPLE_RATE,         /**< sample rate (bits/sample) */
     INF_SOFTWARE,            /**< software */
     INF_SOURCE,              /**< source */
     INF_SOURCE_FORM,         /**< source form */
     INF_SUBJECT,             /**< subject */
     INF_TECHNICAN,           /**< technican */
     INF_TRACK,               /**< track of the CD */
+    INF_TRACKS,              /**< number of tracks of the signal */
     INF_VBR_QUALITY,         /**< base quality of an ogg file in VBR mode */
     INF_VERSION              /**< version/remix */
 } FileProperty;
@@ -98,7 +103,7 @@ class KDE_EXPORT FileInfo
 {
 public:
 
-    /** Constructor */
+    /** Default constructor, creates an empty file info object */
     FileInfo();
 
     /** Copy constructor */
@@ -117,7 +122,7 @@ public:
     inline FileInfo & operator = (const FileInfo &source) {
 	copy(source);
 	return *this;
-    };
+    }
 
     /** Compare operator */
     inline bool operator == (const FileInfo &other) {
@@ -125,39 +130,39 @@ public:
     }
 
     /** returns the number of samples */
-    inline unsigned int length() const { return m_length; };
+    inline sample_index_t length() const { return m_length; }
 
     /** Sets the length in samples */
-    inline void setLength(unsigned int length) { m_length = length; };
+    inline void setLength(unsigned int length) { m_length = length; }
 
     /** returns the sample rate [samples/second] */
-    inline double rate() const { return m_rate; };
+    inline double rate() const { return m_rate; }
 
     /** sets a new sample rate */
-    inline void setRate(double rate) { m_rate = rate; };
+    inline void setRate(double rate) { m_rate = rate; }
 
     /** returns the number of bits per sample */
-    inline unsigned int bits() const { return m_bits; };
+    inline unsigned int bits() const { return m_bits; }
 
     /** sets a new resolution in bits per sample */
-    inline void setBits(unsigned int bits) { m_bits = bits; };
+    inline void setBits(unsigned int bits) { m_bits = bits; }
 
     /** returns the number of tracks */
-    inline unsigned int tracks() const { return m_tracks; };
+    inline unsigned int tracks() const { return m_tracks; }
 
     /** Sets the number of tracks */
-    inline void setTracks(unsigned int tracks) { m_tracks = tracks; };
+    inline void setTracks(unsigned int tracks) { m_tracks = tracks; }
 
     /** Returs the list of labels. @note can be modified! */
-    inline LabelList &labels() { return m_labels; };
+    inline LabelList &labels() { return m_labels; }
 
     /** Returs the list of labels. @note cannot be modified! */
-    inline const LabelList &labels() const { return m_labels; };
+    inline const LabelList &labels() const { return m_labels; }
 
     /** Returns true if the given property exists */
     inline bool contains(const FileProperty property) const {
 	return m_properties.contains(property);
-    };
+    }
 
     /**
      * Sets a property to a new value. If the property does not already
@@ -179,32 +184,32 @@ public:
     /**
      * Returns true if a property is only internal.
      */
-    bool isInternal(FileProperty key);
+    bool isInternal(FileProperty key) const;
 
     /**
      * Returns true if a property is intended to be saved into or
      * loaded from a file.
      */
-    bool canLoadSave(FileProperty key);
+    bool canLoadSave(FileProperty key) const;
 
     /**
      * Returns the name of a property.
      */
-    inline QString name(FileProperty key) {
+    inline QString name(FileProperty key) const {
 	return m_property_map.name(key);
-    };
+    }
 
     /**
      * Returns the localized description of a property.
      */
-    inline QString description(FileProperty key) {
+    inline QString description(FileProperty key) const {
 	return m_property_map.description(key, false);
-    };
+    }
 
     /** Returns a reference to the list of non-standard properties */
     const QMap<FileProperty, QVariant> &properties() const {
 	return m_properties;
-    };
+    }
 
     /** Returns a list of all known non-standard properties */
     QList<FileProperty> allKnownProperties();
@@ -213,7 +218,7 @@ public:
     void clear();
 
     /** dumps all properties to stdout, useful for debugging */
-    void dump();
+    void dump() const;
 
 private:
 
@@ -241,11 +246,21 @@ private:
     class PropertyTypesMap: public TypesMap<FileProperty, int>
     {
     public:
+	/** constructor */
+	explicit PropertyTypesMap()
+	    :TypesMap<FileProperty, int>()
+	{
+	    fill();
+	}
+
+	/** Destructor */
+	virtual ~PropertyTypesMap() {}
+
 	/** fills the list with */
 	virtual void fill();
 
 	/** returns a list of all properties */
-	virtual QList<FileProperty> all();
+	virtual QList<FileProperty> all() const;
     };
 
     /** map with properties and their names and descriptions */
