@@ -27,16 +27,18 @@
 
 #include <kdemacros.h>
 
+#include "libkwave/Sample.h"
+
 class QMimeData;
 class QWidget;
 
 class FileInfo;
 class MultiTrackReader;
 class SignalManager;
+namespace Kwave { class MetaDataList; }
 
 /**
  * Simple class for drag & drop of wav data.
- * @todo support for several codecs
  * @todo the current storage mechanism is straight-forward and stupid, it
  *       should be extended to use virtual memory
  */
@@ -73,10 +75,11 @@ public:
      * @param widget the widget used for displaying error messages
      * @param e mime source
      * @param sig signal that receives the mime data
+     * @param pos position within the signal where to insert the data
      * @return number of decoded samples if successful, zero if failed
      */
-    static unsigned int decode(QWidget *widget, const QMimeData *e,
-                               SignalManager &sig, unsigned int pos);
+    static sample_index_t decode(QWidget *widget, const QMimeData *e,
+                                  SignalManager &sig, sample_index_t pos);
 
 };
 
