@@ -21,9 +21,9 @@
 #include <string.h>
 #include <stdio.h>
 #include <dlfcn.h>
-#include <sched.h>
 
 #include <QProgressDialog>
+#include <QThread>
 #include <QTime>
 #include <QVector>
 #include <QWidget>
@@ -35,7 +35,6 @@
 #include "libkwave/ConfirmCancelProxy.h"
 #include "libkwave/KwavePlugin.h"
 #include "libkwave/MultiTrackReader.h"
-#include "libkwave/MultiTrackWriter.h"
 #include "libkwave/Sample.h"
 #include "libkwave/SampleReader.h"
 #include "libkwave/Signal.h"
@@ -52,6 +51,9 @@
 /** number of updates of the progress bat per second */
 #define PROGRESS_UPDATES_PER_SECOND 4
 
+/** maximum value of the progress bar */
+#define PROGRESS_MAXIMUM 1000000
+
 /*
    A plugin can be unloaded through two different ways. The possible
    scenarios are:
@@ -77,16 +79,15 @@ Kwave::Plugin::Plugin(const PluginContext &c)
      m_stop(false),
      m_progress(0),
      m_confirm_cancel(0),
-     m_usage_count(0),
+     m_usage_count(1),
      m_usage_lock(),
      m_progress_timer(),
      m_current_progress(-1),
-     m_progress_lock()
+     m_progress_lock(QMutex::Recursive)
 {
-    use();
-
     connect(&m_progress_timer, SIGNAL(timeout()),
-            this, SLOT(updateProgressTick()));
+            this, SLOT(updateProgressTick()),
+            Qt::DirectConnection);
 }
 
 //***************************************************************************
@@ -96,6 +97,9 @@ Kwave::Plugin::~Plugin()
     // delete itself
     close();
 
+    // lock usage
+    QMutexLocker lock(&m_usage_lock);
+
     {
 	QMutexLocker lock(&m_thread_lock);
 	if (m_thread) {
@@ -111,8 +115,8 @@ Kwave::Plugin::~Plugin()
 	}
     }
 
-    if (m_confirm_cancel) delete m_confirm_cancel;
-    if (m_progress)       delete m_progress;
+    // finally get rid of the confirm/cancel proxy and the progress dialog
+    closeProgressDialog(this);
 }
 
 //***************************************************************************
@@ -164,15 +168,15 @@ int Kwave::Plugin::start(QStringList &)
 
     // set up the progress dialog when in processing (not pre-listen) mode
     if (m_progress_enabled && m_progress) {
-	unsigned int first, last;
-	unsigned int tracks = selectedTracks().count();
+	sample_index_t first, last;
+	QList<unsigned int> tracks;
 
-	selection(&first, &last, true);
+	selection(&tracks, &first, &last, true);
 	m_progress->setModal(true);
 	m_progress->setVisible(false);
 	m_progress->setMinimumDuration(2000);
 	m_progress->setAutoClose(false);
-	m_progress->setMaximum((last - first + 1) * tracks);
+	m_progress->setMaximum(PROGRESS_MAXIMUM);
 	m_progress->setValue(0);
 	m_progress->setLabelText(progressText());
 	int h = m_progress->sizeHint().height();
@@ -203,7 +207,7 @@ int Kwave::Plugin::start(QStringList &)
 //***************************************************************************
 QString Kwave::Plugin::progressText()
 {
-    return i18n("running plugin '%1' ...", name());
+    return i18n("Running plugin '%1'...", name());
 }
 
 //***************************************************************************
@@ -254,21 +258,21 @@ void Kwave::Plugin::setProgressDialogEnabled(bool enable)
 }
 
 //***************************************************************************
-void Kwave::Plugin::updateProgress(unsigned int progress)
+void Kwave::Plugin::updateProgress(qreal progress)
 {
     // check: this must be called from the GUI thread only!
     Q_ASSERT(this->thread() == QThread::currentThread());
     Q_ASSERT(this->thread() == qApp->thread());
 
-//     qDebug("Kwave::Plugin::updateProgress(%u)", progress);
+//     qDebug("Kwave::Plugin::updateProgress(%0.1f)", progress);
     QMutexLocker lock(&m_progress_lock);
 
     // take over the current progress
     // note: no lock needed, this is called in the GUI thread context!
-    m_current_progress = progress;
+    m_current_progress = qreal(PROGRESS_MAXIMUM / 100.0) * progress;
 
     // start the timer for updating the progress bar if it is not active
-    if (!m_progress_timer.isActive()) {
+    if (!m_progress_timer.isActive() && m_progress) {
 	m_progress_timer.setSingleShot(true);
 	m_progress_timer.start(1000 / PROGRESS_UPDATES_PER_SECOND);
     }
@@ -281,7 +285,6 @@ void Kwave::Plugin::updateProgressTick()
     Q_ASSERT(this->thread() == QThread::currentThread());
     Q_ASSERT(this->thread() == qApp->thread());
 
-//     qDebug("void Kwave::Plugin::updateProgressTick(%d)", m_current_progress);
     QMutexLocker lock(&m_progress_lock);
     if (m_progress) m_progress->setValue(m_current_progress);
 }
@@ -294,15 +297,27 @@ void Kwave::Plugin::closeProgressDialog(Kwave::Plugin *)
     Q_ASSERT(this->thread() == qApp->thread());
 
     QMutexLocker lock(&m_progress_lock);
+
+    // stop the progress timer
+    m_progress_timer.stop();
+    m_progress_timer.disconnect();
+
+    if (m_confirm_cancel) m_confirm_cancel->disconnect();
+    if (m_progress)       m_progress->disconnect();
+
+    // NOTE: as the progress dialog is *modal*, it is higly probable
+    //       that this function is called from the context of the event
+    //       loop that is provided by the progress dialog
+    //       => deleting this object should be done somewhere later...
     if (m_confirm_cancel) {
-	ConfirmCancelProxy *proxy = m_confirm_cancel;
+	m_confirm_cancel->deleteLater();
 	m_confirm_cancel = 0;
-	delete proxy;
     }
+
     if (m_progress) {
-	QProgressDialog *prog = m_progress;
+	m_progress->done(0);
+	m_progress->deleteLater();
 	m_progress = 0;
-	delete prog;
     }
 }
 
@@ -322,6 +337,10 @@ int Kwave::Plugin::execute(QStringList &params)
     Q_ASSERT(m_thread);
     if (!m_thread) return -ENOMEM;
 
+    // increment the use count, it is decremented at the end of
+    // the run_wrapper when the thread is finished
+    use();
+
     // start the thread, this executes run()
     m_thread->start();
 
@@ -375,12 +394,10 @@ void Kwave::Plugin::close()
 	Q_ASSERT(this->thread() == QThread::currentThread());
 	Q_ASSERT(this->thread() == qApp->thread());
 
-	m_progress_timer.stop();
-	qApp->sendPostedEvents();
-	qApp->processEvents();
-	qApp->syncX();
-
+	closeProgressDialog(this);
 	stop();
+    } else if ((QThread::currentThread() == m_thread)) {
+	qWarning("Kwave::Plugin::close -> called from worker thread?");
     }
 }
 
@@ -388,6 +405,7 @@ void Kwave::Plugin::close()
 void Kwave::Plugin::use()
 {
     QMutexLocker lock(&m_usage_lock);
+    Q_ASSERT(m_usage_count); // should be at least 1 (from constructor)
     m_usage_count++;
 }
 
@@ -439,7 +457,7 @@ QString Kwave::Plugin::signalName()
 }
 
 //***************************************************************************
-unsigned int Kwave::Plugin::signalLength()
+sample_index_t Kwave::Plugin::signalLength()
 {
     return manager().signalLength();
 }
@@ -457,25 +475,31 @@ const QList<unsigned int> Kwave::Plugin::selectedTracks()
 }
 
 //***************************************************************************
-unsigned int Kwave::Plugin::selection(unsigned int *left, unsigned int *right,
-                                    bool expand_if_empty)
+sample_index_t Kwave::Plugin::selection(QList<unsigned int> *tracks,
+                                        sample_index_t *left,
+                                        sample_index_t *right,
+                                        bool expand_if_empty)
 {
-    int l = manager().selectionStart();
-    int r = manager().selectionEnd();
+    sample_index_t l = manager().selectionStart();
+    sample_index_t r = manager().selectionEnd();
 
     // expand to the whole signal if left==right and expand_if_empty is set
     if ((l == r) && (expand_if_empty)) {
 	l = 0;
-	r = manager().signalLength()-1;
+	r = manager().signalLength();
+	if (r) r--;
     }
 
+    // get the list of selected tracks
+    if (tracks) *tracks = manager().selectedTracks();
+
     if (left)  *left  = l;
     if (right) *right = r;
-    return r-l+1;
+    return (r != l) ? (r - l + 1) : 0;
 }
 
 //***************************************************************************
-void Kwave::Plugin::selectRange(unsigned int offset, unsigned int length)
+void Kwave::Plugin::selectRange(sample_index_t offset, sample_index_t length)
 {
     manager().selectRange(offset, length);
 }
@@ -484,7 +508,7 @@ void Kwave::Plugin::selectRange(unsigned int offset, unsigned int length)
 void Kwave::Plugin::yield()
 {
     pthread_testcancel();
-    sched_yield();
+    QThread::yieldCurrentThread();
 }
 
 //***************************************************************************
