@@ -19,7 +19,13 @@
 #include "Label.h"
 
 //***************************************************************************
-Label::Label(unsigned int position, const QString &name)
+Label::Label()
+    :m_data(0)
+{
+}
+
+//***************************************************************************
+Label::Label(sample_index_t position, const QString &name)
     :m_data(new LabelData)
 {
     moveTo(position);
@@ -32,13 +38,13 @@ Label::~Label()
 }
 
 //***************************************************************************
-void Label::moveTo(unsigned int position)
+void Label::moveTo(sample_index_t position)
 {
     if (m_data) m_data->m_position = position;
 }
 
 //***************************************************************************
-unsigned int Label::pos() const
+sample_index_t Label::pos() const
 {
     return (m_data) ? m_data->m_position : -1;
 }
@@ -55,6 +61,12 @@ QString Label::name() const
     return (m_data) ? m_data->m_name : QString();
 }
 
+//***************************************************************************
+bool Label::isNull() const
+{
+    return (m_data == 0);
+}
+
 //***************************************************************************
 //***************************************************************************
 Label::LabelData::LabelData()
