@@ -35,14 +35,13 @@
 #include "libkwave/KwavePlugin.h"
 #include "libkwave/MessageBox.h"
 #include "libkwave/MultiTrackReader.h"
-#include "libkwave/MultiTrackWriter.h"
 #include "libkwave/Parser.h"
 #include "libkwave/PlayBackDevice.h"
 #include "libkwave/PlaybackDeviceFactory.h"
 #include "libkwave/PluginContext.h"
 #include "libkwave/SampleReader.h"
-#include "libkwave/SampleWriter.h"
 #include "libkwave/SignalManager.h"
+#include "libkwave/Writer.h"
 #include "libkwave/undo/UndoTransactionGuard.h"
 #include "libkwave/undo/UndoAction.h"
 #include "libkwave/undo/UndoModifyAction.h"
@@ -54,14 +53,29 @@ Kwave::PluginManager::PluginDeleter::PluginDeleter(Kwave::Plugin *plugin,
                                                    void *handle)
   :QObject(), m_plugin(plugin), m_handle(handle)
 {
+    Q_ASSERT(m_plugin);
 }
 
 //***************************************************************************
 Kwave::PluginManager::PluginDeleter::~PluginDeleter()
 {
+    // check: this must be called from the GUI thread only!
+    Q_ASSERT(this->thread() == QThread::currentThread());
+    Q_ASSERT(this->thread() == qApp->thread());
+
+    // empty the event queues before deleting
+    qApp->processEvents();
+    qApp->flush();
+
     // delete the plugin, this should also remove everything it has allocated
+    Q_ASSERT(m_plugin);
     if (m_plugin) delete m_plugin;
 
+    // empty the event queues before unmap, in case the destructors
+    // queued something
+    qApp->processEvents();
+    qApp->flush();
+
     // now the handle of the shared object can be cleared too
     dlclose(m_handle);
 }
@@ -71,7 +85,8 @@ Kwave::PluginManager::PluginDeleter::~PluginDeleter()
 
 // static initializers
 
-QMap<QString, QString> Kwave::PluginManager::m_plugin_files;
+QMap<QString, Kwave::PluginManager::PluginInfo> \
+    Kwave::PluginManager::m_found_plugins;
 
 Kwave::PluginManager::PluginList Kwave::PluginManager::m_unique_plugins;
 
@@ -80,15 +95,16 @@ static QList<PlaybackDeviceFactory *> m_playback_factories;
 //***************************************************************************
 Kwave::PluginManager::PluginManager(QWidget *parent,
                                     SignalManager &signal_manager)
-    :m_loaded_plugins(), m_running_plugins(),
-     m_parent_widget(parent), m_signal_manager(signal_manager)
+    :m_loaded_plugins(),
+     m_running_plugins(),
+     m_parent_widget(parent),
+     m_signal_manager(signal_manager)
 {
-    // use all unique plugins
-    // this does nothing on the first instance, all other instances
-    // will probably find a non-empty list
+    // connect all unique plugins
     foreach (KwavePluginPointer p, m_unique_plugins) {
 	Q_ASSERT(p && p->isUnique());
-	if (p && p->isUnique()) {
+	if (p) {
+	    // increase the use count of the existing unique plugin
 	    p->use();
 
 	    // maybe we will become responsible for releasing
@@ -111,25 +127,28 @@ Kwave::PluginManager::~PluginManager()
     this->sync();
 
     // release all unique plugins
-    while (!m_unique_plugins.isEmpty()) {
-	KwavePluginPointer p = m_unique_plugins.takeLast();
+    foreach (KwavePluginPointer p, m_unique_plugins) {
 	Q_ASSERT(p && p->isUnique());
-	if (p && p->isUnique()) p->release();
+	if (p) p->release();
+	this->sync();
     }
 
     // release all own persistent plugins
-    while (!m_loaded_plugins.isEmpty()) {
-	KwavePluginPointer p = m_loaded_plugins.takeLast();
+    foreach (KwavePluginPointer p, m_loaded_plugins) {
 	Q_ASSERT(p);
 	if (p && p->isPersistent()) p->release();
+	this->sync();
     }
 
-    // release all own plugins that are left
+    // release all own plugins that are left (should be an empty list!)
     while (!m_loaded_plugins.isEmpty()) {
 	KwavePluginPointer p = m_loaded_plugins.takeLast();
 	Q_ASSERT(p);
+	qWarning("RELEASING LOADED PLUGIN '%s'", p->name().toLocal8Bit().data());
 	if (p) p->release();
+	this->sync();
     }
+
 }
 
 //***************************************************************************
@@ -138,24 +157,25 @@ void Kwave::PluginManager::loadAllPlugins()
     // Try to load all plugins. This has to be called only once per
     // instance of the main window!
     // NOTE: this also implicitely makes it resident if it is persistent or unique
-    foreach (QString name, m_plugin_files.keys()) {
+    foreach (QString name, m_found_plugins.keys()) {
 	KwavePluginPointer plugin = loadPlugin(name);
 	if (plugin) {
 // 	    QString state = "";
 // 	    if (plugin->isPersistent()) state += "(persistent)";
 // 	    if (plugin->isUnique()) state += "(unique)";
 // 	    if (!state.length()) state = "(normal)";
-// 	    qDebug("PluginManager::loadAllPlugins(): plugin '"+
-// 		   plugin->name()+"' "+state);
-	    if (!plugin->isUnique() && !plugin->isPersistent()) {
-		// remove it again if it is neither unique nor persistent
-		plugin->release();
-	    }
+// 	    qDebug("PluginManager::loadAllPlugins(): plugin '%s' %s",
+// 		   plugin->name().toLocal8Bit().data(),
+// 		   state.toLocal8Bit().data());
+
+	    // reduce use count again, unique and persistent plugins
+	    // stay loaded with use count 1
+	    plugin->release();
 	} else {
 	    // loading failed => remove it from the list
 	    qWarning("PluginManager::loadAllPlugins(): removing '%s' "\
 	            "from list", name.toLocal8Bit().data());
-	    m_plugin_files.remove(name);
+	    m_found_plugins.remove(name);
 	}
     }
 }
@@ -163,14 +183,18 @@ void Kwave::PluginManager::loadAllPlugins()
 //***************************************************************************
 Kwave::Plugin *Kwave::PluginManager::loadPlugin(const QString &name)
 {
+    // check: this must be called from the GUI thread only!
+    Q_ASSERT(this->thread() == QThread::currentThread());
+    Q_ASSERT(this->thread() == qApp->thread());
 
     // first find out if the plugin is already loaded and persistent
     foreach (KwavePluginPointer p, m_loaded_plugins) {
+	Q_ASSERT(p);
 	if (p && p->isPersistent() && (p->name() == name)) {
-	    Q_ASSERT(p->isPersistent());
 	    qDebug("PluginManager::loadPlugin('%s')"\
 	           "-> returning pointer to persistent",
 	           name.toLocal8Bit().data());
+	    p->use();
 	    return p;
 	}
     }
@@ -183,28 +207,29 @@ Kwave::Plugin *Kwave::PluginManager::loadPlugin(const QString &name)
 	    qDebug("PluginManager::loadPlugin('%s')"\
 	           "-> returning pointer to unique+persistent",
 	           name.toLocal8Bit().data());
+	    p->use();
 	    return p;
 	}
     }
 
     // show a warning and abort if the plugin is unknown
-    if (!(m_plugin_files.contains(name))) {
+    if (!(m_found_plugins.contains(name))) {
 	QString message =
-	    i18n("oops, plugin '%1' is unknown or invalid!", name);
+	    i18n("The plugin '%1' is unknown or invalid.", name);
 	Kwave::MessageBox::error(m_parent_widget, message,
-	    i18n("error on loading plugin"));
+	    i18n("Error On Loading Plugin"));
 	return 0;
     }
-    QString &filename = m_plugin_files[name];
+    QString &filename = m_found_plugins[name].m_filename;
 
     // try to get the file handle of the plugin's binary
     void *handle = dlopen(filename.toLocal8Bit(), RTLD_NOW);
     if (!handle) {
-	QString message = i18n("unable to load the file \n'%1'\n"\
-	                       " that contains the plugin '%2' !",
+	QString message = i18n("Unable to load the file\n'%1'\n"\
+	                       "that contains the plugin '%2'.",
 	                       filename, name);
 	Kwave::MessageBox::error(m_parent_widget, message,
-	    i18n("error on loading plugin"));
+	    i18n("Error On Loading Plugin"));
 	return 0;
     }
 
@@ -223,7 +248,7 @@ Kwave::Plugin *Kwave::PluginManager::loadPlugin(const QString &name)
 	static_cast<const char **>(dlsym(handle, sym_author));
     Q_ASSERT(pauthor);
     if (pauthor) author=*pauthor;
-    if (!author) author = i18n("(unknown)").toLocal8Bit();
+    if (!author) author = i18n("(Unknown)").toLocal8Bit();
 
     // get the plugin's version string
     const char *version = "";
@@ -231,7 +256,7 @@ Kwave::Plugin *Kwave::PluginManager::loadPlugin(const QString &name)
 	static_cast<const char **>(dlsym(handle, sym_version));
     Q_ASSERT(pver);
     if (pver) version=*pver;
-    if (!version) version = i18n("(unknown)").toLocal8Bit();
+    if (!version) version = i18n("(Unknown)").toLocal8Bit();
 
     plugin_loader =
         reinterpret_cast<Kwave::Plugin *(*)(const PluginContext *)>(
@@ -267,18 +292,21 @@ Kwave::Plugin *Kwave::PluginManager::loadPlugin(const QString &name)
 
     if (plugin->isUnique()) {
 	// append unique plugins to the global list of unique plugins
-	m_unique_plugins.append(plugin);
 	plugin->use();
+	m_unique_plugins.append(plugin);
     } else {
 	// append the plugin into our list of loaded plugins
-	m_loaded_plugins.append(plugin);
 	if (plugin->isPersistent()) {
-	    // increment the usage if it is persistent, we will
-	    // only load it once in this instance
 	    plugin->use();
 	}
+	m_loaded_plugins.append(plugin);
     }
 
+    // now we have a newly created plugin, the use count is
+    // 1 for normal plugins
+    // 2 for persistent plugins
+    // 2 for unique plugins
+
     // connect all necessary signals/slots
     connectPlugin(plugin);
 
@@ -297,6 +325,10 @@ int Kwave::PluginManager::executePlugin(const QString &name,
     QString command;
     int result = 0;
 
+    // check: this must be called from the GUI thread only!
+    Q_ASSERT(this->thread() == QThread::currentThread());
+    Q_ASSERT(this->thread() == qApp->thread());
+
     // synchronize: wait until any currently running plugins are done
     sync();
 
@@ -310,14 +342,14 @@ int Kwave::PluginManager::executePlugin(const QString &name,
 	result = plugin->start(*params);
 
 	// maybe the start() function has called close() ?
-	if (!plugin->isPersistent() && !m_loaded_plugins.contains(plugin)) {
-	    qDebug("PluginManager: plugin closed itself in start()"); // ###
+	if (!m_unique_plugins.contains(plugin) &&
+	    !m_loaded_plugins.contains(plugin)) {
+	    qDebug("PluginManager: plugin closed itself in start()");
 	    result = -1;
 	    plugin = 0;
 	}
 
 	if (plugin && (result >= 0)) {
-	    plugin->use(); // plugin->release() will be called after run()
 	    plugin->execute(*params);
 	}
     } else {
@@ -341,10 +373,8 @@ int Kwave::PluginManager::executePlugin(const QString &name,
 	    // macro recorder
 	    command = "plugin:execute(";
 	    command += name;
-	    for (int i = 0; i < params->count(); i++) {
-		command += ", ";
-		command += params->at(i);
-	    }
+	    foreach (const QString &p, *params)
+		command += ", " + p;
 	    delete params;
 	    command += ")";
 //	    qDebug("PluginManager: command='%s'",command.data());
@@ -353,7 +383,7 @@ int Kwave::PluginManager::executePlugin(const QString &name,
 
     // now the plugin is no longer needed here, so delete it
     // if it has not already been detached
-    if (plugin && !plugin->isPersistent()) plugin->release();
+    if (plugin) plugin->release();
 
     // emit a command, let the toplevel window (and macro recorder) get
     // it and call us again later...
@@ -365,16 +395,33 @@ int Kwave::PluginManager::executePlugin(const QString &name,
 //***************************************************************************
 bool Kwave::PluginManager::onePluginRunning()
 {
-    foreach (KwavePluginPointer plugin, m_loaded_plugins)
-	if (plugin && plugin->isRunning()) return true;
+    // check: this must be called from the GUI thread only!
+    Q_ASSERT(this->thread() == QThread::currentThread());
+    Q_ASSERT(this->thread() == qApp->thread());
+
+    if (!m_loaded_plugins.isEmpty())
+	foreach (KwavePluginPointer plugin, m_loaded_plugins)
+	    if (plugin && plugin->isRunning()) return true;
+    if (!m_unique_plugins.isEmpty())
+	foreach (KwavePluginPointer plugin, m_unique_plugins)
+	    if (plugin && plugin->isRunning()) return true;
     return false;
 }
 
 //***************************************************************************
 void Kwave::PluginManager::sync()
 {
+    // check: this must be called from the GUI thread only!
+    Q_ASSERT(this->thread() == QThread::currentThread());
+    Q_ASSERT(this->thread() == qApp->thread());
+
+    qApp->processEvents();
+    qApp->flush();
+
     while (onePluginRunning()) {
-	pthread_yield();
+	QThread::yieldCurrentThread();
+	qApp->processEvents();
+	qApp->flush();
     }
 }
 
@@ -395,12 +442,12 @@ int Kwave::PluginManager::setupPlugin(const QString &name)
 	// the setup function has not been aborted.
 	savePluginDefaults(name, plugin->version(), *params);
 	delete params;
-    } else return -1;
-
-    // now the plugin is no longer needed here, so delete it
-    // if it has not already been detached and is not persistent
-    if (!plugin->isPersistent()) plugin->release();
+    } else {
+	plugin->release();
+	return -1;
+    }
 
+    plugin->release();
     return 0;
 }
 
@@ -460,7 +507,7 @@ void Kwave::PluginManager::savePluginDefaults(const QString &name,
 }
 
 //***************************************************************************
-unsigned int Kwave::PluginManager::signalLength()
+sample_index_t Kwave::PluginManager::signalLength()
 {
     return m_signal_manager.length();
 }
@@ -478,31 +525,24 @@ const QList<unsigned int> Kwave::PluginManager::selectedTracks()
 }
 
 //***************************************************************************
-unsigned int Kwave::PluginManager::selectionStart()
+sample_index_t Kwave::PluginManager::selectionStart()
 {
     return m_signal_manager.selection().first();
 }
 
 //***************************************************************************
-unsigned int Kwave::PluginManager::selectionEnd()
+sample_index_t Kwave::PluginManager::selectionEnd()
 {
     return m_signal_manager.selection().last();
 }
 
 //***************************************************************************
-void Kwave::PluginManager::selectRange(unsigned int offset,
-                                       unsigned int length)
+void Kwave::PluginManager::selectRange(sample_index_t offset,
+                                       sample_index_t length)
 {
     m_signal_manager.selectRange(offset, length);
 }
 
-//***************************************************************************
-SampleWriter *Kwave::PluginManager::openSampleWriter(unsigned int track,
-	InsertMode mode, unsigned int left, unsigned int right)
-{
-    return m_signal_manager.openSampleWriter(track, mode, left, right, true);
-}
-
 //***************************************************************************
 Kwave::SampleSink *Kwave::PluginManager::openMultiTrackPlayback(
     unsigned int tracks, const QString *name)
@@ -553,6 +593,10 @@ void Kwave::PluginManager::signalClosed()
 //***************************************************************************
 void Kwave::PluginManager::pluginClosed(Kwave::Plugin *p)
 {
+    // check: this must be called from the GUI thread only!
+    Q_ASSERT(this->thread() == QThread::currentThread());
+    Q_ASSERT(this->thread() == qApp->thread());
+
     Q_ASSERT(p);
     Q_ASSERT(!m_loaded_plugins.isEmpty() || !m_unique_plugins.isEmpty());
     if (!p) return;
@@ -590,6 +634,10 @@ void Kwave::PluginManager::pluginStarted(Kwave::Plugin *p)
 //***************************************************************************
 void Kwave::PluginManager::pluginDone(Kwave::Plugin *p)
 {
+    // check: this must be called from the GUI thread only!
+    Q_ASSERT(this->thread() == QThread::currentThread());
+    Q_ASSERT(this->thread() == qApp->thread());
+
     Q_ASSERT(p);
     if (!p) return;
 
@@ -613,10 +661,11 @@ void Kwave::PluginManager::connectPlugin(Kwave::Plugin *plugin)
 
     connect(plugin, SIGNAL(sigClosed(Kwave::Plugin *)),
 	    this, SLOT(pluginClosed(Kwave::Plugin *)),
-	     Qt::QueuedConnection);
+	    Qt::QueuedConnection);
 
     connect(plugin, SIGNAL(sigRunning(Kwave::Plugin *)),
-	    this, SLOT(pluginStarted(Kwave::Plugin *)));
+	    this, SLOT(pluginStarted(Kwave::Plugin *)),
+	    Qt::DirectConnection);
 
     connect(plugin, SIGNAL(sigDone(Kwave::Plugin *)),
 	    this, SLOT(pluginDone(Kwave::Plugin *)),
@@ -653,7 +702,7 @@ void Kwave::PluginManager::setSignalName(const QString &name)
 //***************************************************************************
 void Kwave::PluginManager::findPlugins()
 {
-    if (!m_plugin_files.isEmpty()) {
+    if (!m_found_plugins.isEmpty()) {
 	// this is not the first call -> bail out
 	return;
     }
@@ -673,17 +722,26 @@ void Kwave::PluginManager::findPlugins()
 		static_cast<const char **>(dlsym(handle, "name"));
 	    const char **version =
 		static_cast<const char **>(dlsym(handle, "version"));
+	    const char **description =
+		static_cast<const char **>(dlsym(handle, "description"));
 	    const char **author  =
 		static_cast<const char **>(dlsym(handle, "author"));
 
 	    // skip it if something is missing or null
-	    if (!name || !version || !author) continue;
-	    if (!*name || !*version || !*author) continue;
+	    if (!name || !version || !author || !description) continue;
+	    if (!*name || !*version || !*author || !*description) continue;
 
-	    emit sigProgress(i18n("loading plugin %1...", *name));
+	    emit sigProgress(i18n("Loading plugin %1...", *name));
 	    QApplication::processEvents();
 
-	    m_plugin_files.insert(*name, file);
+	    PluginInfo info;
+	    info.m_filename    = file;
+	    info.m_author      = QString::fromUtf8(*author);
+	    info.m_description = i18n(*description);
+	    info.m_version     = QString::fromUtf8(*version);
+
+	    m_found_plugins.insert(*name, info);
+
 	    qDebug("%16s %5s written by %s",
 		*name, *version, *author);
 
@@ -694,7 +752,14 @@ void Kwave::PluginManager::findPlugins()
 	}
     }
 
-    qDebug("--- \n found %d plugins\n", m_plugin_files.count());
+    qDebug("--- \n found %d plugins\n", m_found_plugins.count());
+}
+
+//***************************************************************************
+const QList<Kwave::PluginManager::PluginInfo>
+    Kwave::PluginManager::pluginInfoList() const
+{
+    return m_found_plugins.values();
 }
 
 //***************************************************************************
