@@ -31,6 +31,7 @@
 #include <kdemacros.h>
 
 #include "libkwave/InsertMode.h"
+#include "libkwave/Sample.h"
 
 class FileInfo;
 class PlaybackController;
@@ -39,16 +40,16 @@ class QString;
 class QStringList;
 class SignalManager;
 class QWidget;
-namespace Kwave { class SampleSink; }
 namespace Kwave { class Plugin; }
+namespace Kwave { class SampleSink; }
 namespace Kwave { class Writer; }
 
-/**
- * Manages the loading, initializing, starting, running and closing
- * of the plugins of kwave. Each instance of a TopWidget creates a
- * new instance of the PluginManager to be independent from other
- * toplevel widgets.
- */
+    /**
+     * Manages the loading, initializing, starting, running and closing
+     * of the plugins of kwave. Each instance of a TopWidget creates a
+     * new instance of the PluginManager to be independent from other
+     * toplevel widgets.
+     */
 namespace Kwave {
     class KDE_EXPORT PluginManager : public QObject
     {
@@ -108,7 +109,7 @@ namespace Kwave {
 	 * Returns the length of the current signal in samples.
 	 * If no signal is present the return value will be 0.
 	 */
-	unsigned int signalLength();
+	sample_index_t signalLength();
 
 	/**
 	 * Returns the current sample rate in samples per second.
@@ -125,34 +126,20 @@ namespace Kwave {
 	 * Returns the start of the selection. If nothing is currently
 	 * selected this will be the first sample (0).
 	 */
-	unsigned int selectionStart();
+	sample_index_t selectionStart();
 
 	/**
 	 * Returns the end of the selection. If nothing is currently
 	 * selected this will be the last sample (length-1).
 	 */
-	unsigned int selectionEnd();
+	sample_index_t selectionEnd();
 
 	/**
 	 * Sets the current start and length of the selection to new values.
 	 * @param offset index of the first sample
 	 * @param length number of samples
 	 */
-	void selectRange(unsigned int offset, unsigned int length);
-
-	/**
-	 * Opens an input stream for a track, starting at a specified sample
-	 * position. Also handles undo information.
-	 * @param track index of the track. If the track does not exist, this
-	 *        function will fail and return 0
-	 * @param mode specifies where and how to insert
-	 * @param left start of the input (only useful in insert and
-	 *             overwrite mode)
-	 * @param right end of the input (only useful with overwrite mode)
-	 * @see InsertMode
-	 */
-	Kwave::Writer *openWriter(unsigned int track, InsertMode mode,
-	    unsigned int left = 0, unsigned int right = 0);
+	void selectRange(sample_index_t offset, sample_index_t length);
 
 	/**
 	 * Opens a Kwave::MultiTrackSink for playback purposes.
@@ -206,6 +193,17 @@ namespace Kwave {
 	 */
 	void unregisterPlaybackDeviceFactory(PlaybackDeviceFactory *factory);
 
+	/** structure with information about a plugin */
+	typedef struct  {
+	    QString m_filename;    /**< name of the file           */
+	    QString m_author;      /**< name of the author         */
+	    QString m_description; /**< verbose name / description */
+	    QString m_version;     /**< plugin API version         */
+	} PluginInfo;
+
+	/** returns a list with info of all known plugins */
+	const QList<PluginInfo> pluginInfoList() const;
+
     signals:
 
 	/**
@@ -333,8 +331,13 @@ namespace Kwave {
 	/** connects all signals from and to a plugin */
 	void disconnectPlugin(Kwave::Plugin *plugin);
 
-	/** map for finding plugin files through their name */
-	static QMap<QString, QString> m_plugin_files;
+    private:
+
+	/**
+	 * map with plugin information: key = short name of the plugin,
+	 * data = plugin info (description, author, version etc...)
+	 */
+	static QMap<QString, PluginInfo> m_found_plugins;
 
 	/** list of own loaded plugins */
 	PluginList m_loaded_plugins;
