@@ -29,7 +29,6 @@
 
 #include "libkwave/MessageBox.h"
 #include "libkwave/MultiTrackReader.h"
-#include "libkwave/MultiTrackWriter.h"
 #include "libkwave/Track.h"
 #include "libkwave/WindowFunction.h"
 
@@ -46,7 +45,7 @@ Signal::Signal()
 }
 
 //***************************************************************************
-Signal::Signal(unsigned int tracks, unsigned int length)
+Signal::Signal(unsigned int tracks, sample_index_t length)
     :m_tracks(), m_lock_tracks()
 {
     while (tracks--) {
@@ -66,14 +65,12 @@ void Signal::close()
     QWriteLocker lock(&m_lock_tracks);
 
     while (m_tracks.count()) {
-	Track *t = m_tracks.last();
-	if (t) delete t;
-	m_tracks.removeAll(t);
+	deleteTrack(m_tracks.count() - 1);
     }
 }
 
 //***************************************************************************
-Track *Signal::insertTrack(unsigned int index, unsigned int length)
+Track *Signal::insertTrack(unsigned int index, sample_index_t length)
 {
     unsigned int track_nr = 0;
     Track *t = 0;
@@ -95,18 +92,18 @@ Track *Signal::insertTrack(unsigned int index, unsigned int length)
 	}
 
 	// connect to the track's signals
-	connect(t, SIGNAL(sigSamplesDeleted(Track *, unsigned int,
-	    unsigned int)),
-	    this, SLOT(slotSamplesDeleted(Track *, unsigned int,
-	    unsigned int)));
-	connect(t, SIGNAL(sigSamplesInserted(Track *, unsigned int,
-	    unsigned int)),
-	    this, SLOT(slotSamplesInserted(Track *, unsigned int,
-	    unsigned int)));
-	connect(t, SIGNAL(sigSamplesModified(Track *, unsigned int,
-	    unsigned int)),
-	    this, SLOT(slotSamplesModified(Track *, unsigned int,
-	    unsigned int)));
+	connect(t, SIGNAL(sigSamplesDeleted(Track *, sample_index_t,
+	    sample_index_t)),
+	    this, SLOT(slotSamplesDeleted(Track *, sample_index_t,
+	    sample_index_t)));
+	connect(t, SIGNAL(sigSamplesInserted(Track *, sample_index_t,
+	    sample_index_t)),
+	    this, SLOT(slotSamplesInserted(Track *, sample_index_t,
+	    sample_index_t)));
+	connect(t, SIGNAL(sigSamplesModified(Track *, sample_index_t,
+	    sample_index_t)),
+	    this, SLOT(slotSamplesModified(Track *, sample_index_t,
+	    sample_index_t)));
     }
 
     // track has been inserted at the end
@@ -116,7 +113,7 @@ Track *Signal::insertTrack(unsigned int index, unsigned int length)
 
 
 //***************************************************************************
-Track *Signal::appendTrack(unsigned int length)
+Track *Signal::appendTrack(sample_index_t length)
 {
     return insertTrack(tracks(), length);
 }
@@ -146,8 +143,8 @@ void Signal::deleteTrack(unsigned int index)
 }
 
 //***************************************************************************
-SampleWriter *Signal::openSampleWriter(unsigned int track,
-	InsertMode mode, unsigned int left, unsigned int right)
+Kwave::Writer *Signal::openWriter(unsigned int track,
+	InsertMode mode, sample_index_t left, sample_index_t right)
 {
     QReadLocker lock(&m_lock_tracks);
 
@@ -158,12 +155,12 @@ SampleWriter *Signal::openSampleWriter(unsigned int track,
 
     Track *t = m_tracks.at(track);
     Q_ASSERT(t);
-    return (t) ? t->openSampleWriter(mode, left, right) : 0;
+    return (t) ? t->openWriter(mode, left, right) : 0;
 }
 
 //***************************************************************************
 SampleReader *Signal::openSampleReader(Kwave::ReaderMode mode,
-	unsigned int track, unsigned int left, unsigned int right)
+	unsigned int track, sample_index_t left, sample_index_t right)
 {
     QReadLocker lock(&m_lock_tracks);
 
@@ -190,8 +187,8 @@ QList<unsigned int> Signal::allTracks()
 }
 
 //***************************************************************************
-void Signal::deleteRange(unsigned int track, unsigned int offset,
-                         unsigned int length)
+void Signal::deleteRange(unsigned int track, sample_index_t offset,
+                         sample_index_t length)
 {
     QReadLocker lock(&m_lock_tracks);
 
@@ -204,6 +201,21 @@ void Signal::deleteRange(unsigned int track, unsigned int offset,
     if (t) t->deleteRange(offset, length);
 }
 
+//***************************************************************************
+void Signal::insertSpace(unsigned int track, sample_index_t offset,
+                         sample_index_t length)
+{
+    QReadLocker lock(&m_lock_tracks);
+
+    Q_ASSERT(static_cast<int>(track) < m_tracks.count());
+    if (static_cast<int>(track) >= m_tracks.count())
+	return; // track does not exist !
+
+    Track *t = m_tracks.at(track);
+    Q_ASSERT(t);
+    if (t) t->insertSpace(offset, length);
+}
+
 //***************************************************************************
 unsigned int Signal::tracks()
 {
@@ -212,14 +224,14 @@ unsigned int Signal::tracks()
 }
 
 //***************************************************************************
-unsigned int Signal::length()
+sample_index_t Signal::length()
 {
     QReadLocker lock(&m_lock_tracks);
 
-    unsigned int max = 0;
+    sample_index_t max = 0;
     foreach (Track *track, m_tracks) {
 	if (!track) continue;
-	unsigned int len = track->length();
+	sample_index_t len = track->length();
 	if (len > max) max = len;
     }
     return max;
@@ -332,7 +344,7 @@ void Signal::selectTrack(unsigned int track, bool select)
 //    } else {
 //	if (data) delete data;
 //	Kwave::MessageBox::error
-//	(0, i18n("Info"), i18n("No Memory for FFT-buffers available !"), 2);
+//	(0, i18n("Info"), i18n("No Memory for FFT-buffers available."), 2);
 //    }
 //}
 //
@@ -403,7 +415,7 @@ void Signal::selectTrack(unsigned int track, bool select)
 ////    else {
 ////	if (data) delete data;
 ////	Kwave::MessageBox::error
-////	(0, i18n("Info"), i18n("No Memory for FFT-buffers available !"), 2);
+////	(0, i18n("Info"), i18n("No Memory for FFT-buffers available."), 2);
 ////    }
 //}
 ////*********************************************************
@@ -517,24 +529,24 @@ unsigned int Signal::trackIndex(const Track *track)
 }
 
 //***************************************************************************
-void Signal::slotSamplesInserted(Track *src, unsigned int offset,
-                                 unsigned int length)
+void Signal::slotSamplesInserted(Track *src, sample_index_t offset,
+                                 sample_index_t length)
 {
     unsigned int track = trackIndex(src);
     emit sigSamplesInserted(track, offset, length);
 }
 
 //***************************************************************************
-void Signal::slotSamplesDeleted(Track *src, unsigned int offset,
-                                unsigned int length)
+void Signal::slotSamplesDeleted(Track *src, sample_index_t offset,
+                                sample_index_t length)
 {
     unsigned int track = trackIndex(src);
     emit sigSamplesDeleted(track, offset, length);
 }
 
 //***************************************************************************
-void Signal::slotSamplesModified(Track *src, unsigned int offset,
-                                 unsigned int length)
+void Signal::slotSamplesModified(Track *src, sample_index_t offset,
+                                 sample_index_t length)
 {
     unsigned int track = trackIndex(src);
     emit sigSamplesModified(track, offset, length);
