@@ -49,10 +49,10 @@
 #include "libkwave/Parser.h"
 #include "libkwave/Sample.h"
 #include "libkwave/SampleReader.h"
-#include "libkwave/SampleWriter.h"
 #include "libkwave/Signal.h"
 #include "libkwave/SignalManager.h"
 #include "libkwave/Track.h"
+#include "libkwave/Writer.h"
 #include "libkwave/undo/UndoTransactionGuard.h"
 #include "libkwave/undo/UndoAction.h"
 #include "libkwave/undo/UndoAddLabelAction.h"
@@ -63,6 +63,7 @@
 #include "libkwave/undo/UndoInsertAction.h"
 #include "libkwave/undo/UndoInsertTrack.h"
 #include "libkwave/undo/UndoModifyAction.h"
+#include "libkwave/undo/UndoModifyLabelAction.h"
 #include "libkwave/undo/UndoSelection.h"
 #include "libkwave/undo/UndoTransaction.h"
 
@@ -96,18 +97,24 @@ SignalManager::SignalManager(QWidget *parent)
             this, SLOT(slotTrackInserted(unsigned int, Track *)));
     connect(sig, SIGNAL(sigTrackDeleted(unsigned int)),
             this, SLOT(slotTrackDeleted(unsigned int)));
-    connect(sig, SIGNAL(sigSamplesDeleted(unsigned int, unsigned int,
-	unsigned int)),
-	this, SLOT(slotSamplesDeleted(unsigned int, unsigned int,
-	unsigned int)));
-    connect(sig, SIGNAL(sigSamplesInserted(unsigned int, unsigned int,
-	unsigned int)),
-	this, SLOT(slotSamplesInserted(unsigned int, unsigned int,
-	unsigned int)));
-    connect(sig, SIGNAL(sigSamplesModified(unsigned int, unsigned int,
-	unsigned int)),
-	this, SLOT(slotSamplesModified(unsigned int, unsigned int,
-	unsigned int)));
+    connect(sig, SIGNAL(sigSamplesDeleted(unsigned int, sample_index_t,
+	sample_index_t)),
+	this, SLOT(slotSamplesDeleted(unsigned int, sample_index_t,
+	sample_index_t)));
+    connect(sig, SIGNAL(sigSamplesInserted(unsigned int, sample_index_t,
+	sample_index_t)),
+	this, SLOT(slotSamplesInserted(unsigned int, sample_index_t,
+	sample_index_t)));
+    connect(sig, SIGNAL(sigSamplesModified(unsigned int, sample_index_t,
+	sample_index_t)),
+	this, SLOT(slotSamplesModified(unsigned int, sample_index_t,
+	sample_index_t)));
+}
+
+//***************************************************************************
+SignalManager::~SignalManager()
+{
+    close();
 }
 
 //***************************************************************************
@@ -163,12 +170,12 @@ int SignalManager::loadFile(const KUrl &url)
 
 	// create all tracks (empty)
 	unsigned int track;
-	const unsigned int tracks = decoder->info().tracks();
-	const unsigned int length = decoder->info().length();
+	const unsigned int tracks   = decoder->info().tracks();
+	const sample_index_t length = decoder->info().length();
 	Q_ASSERT(tracks);
 	if (!tracks) break;
 
-	for (track=0; track < tracks; ++track) {
+	for (track = 0; track < tracks; ++track) {
 	    Track *t = m_signal.appendTrack(length);
 	    Q_ASSERT(t);
 	    if (!t || (t->length() != length)) {
@@ -182,7 +189,7 @@ int SignalManager::loadFile(const KUrl &url)
 	// create the multitrack writer as destination
 	// if length was zero -> append mode / decode a stream ?
 	InsertMode mode = (streaming) ? Append : Overwrite;
-	MultiTrackWriter writers(*this, allTracks(), mode, 0,
+	Kwave::MultiTrackWriter writers(*this, allTracks(), mode, 0,
 	    (length) ? length-1 : 0);
 
 	// try to calculate the resulting length, but if this is
@@ -198,18 +205,19 @@ int SignalManager::loadFile(const KUrl &url)
 	    filename, resulting_size,
 	    info.length(), info.rate(), info.bits(), info.tracks());
 	Q_ASSERT(dialog);
+
 	if (use_src_size) {
 	    // use source size for progress / stream mode
-	    QObject::connect(decoder, SIGNAL(sourceProcessed(unsigned int)),
-	                     dialog, SLOT(setBytePosition(unsigned int)));
-	    QObject::connect(&writers, SIGNAL(progress(unsigned int)),
-	                     dialog, SLOT(setLength(unsigned int)));
+	    QObject::connect(decoder, SIGNAL(sourceProcessed(quint64)),
+	                     dialog,  SLOT(setBytePosition(quint64)));
+	    QObject::connect(&writers, SIGNAL(written(quint64)),
+	                     dialog,   SLOT(setLength(quint64)));
 	} else {
-	    // use resulting size for progress
-	    QObject::connect(&writers, SIGNAL(progress(unsigned int)),
-	                     dialog, SLOT(setValue(unsigned int)));
+	    // use resulting size percentage for progress
+	    QObject::connect(&writers, SIGNAL(progress(qreal)),
+	                     dialog,   SLOT(setValue(qreal)));
 	}
-	QObject::connect(dialog, SIGNAL(canceled()),
+	QObject::connect(dialog,   SIGNAL(canceled()),
 	                 &writers, SLOT(cancel()));
 
 	// now decode
@@ -230,23 +238,30 @@ int SignalManager::loadFile(const KUrl &url)
 	if (!res && streaming) {
 	    // source was opened in stream mode -> now we have the length
 	    writers.flush();
-	    unsigned int new_length = writers.last();
+	    sample_index_t new_length = writers.last();
 	    if (new_length) new_length++;
-	    m_file_info.setLength(new_length);
+	    info.setLength(new_length);
 	} else {
-	    m_file_info.setLength(this->length());
-	    m_file_info.setTracks(tracks);
+	    info.setLength(this->length());
+	    info.setTracks(tracks);
 	}
 
+	// enter the filename/mimetype and size into the file info
+	m_file_info.set(INF_FILENAME, fi.absoluteFilePath());
+	m_file_info.set(INF_FILESIZE, static_cast<unsigned int>(src.size()));
+	m_file_info.set(INF_MIMETYPE, mimetype);
 	// update the length info in the progress dialog if needed
 	if (dialog && use_src_size) {
-	    dialog->setLength(m_file_info.length() * m_file_info.tracks());
+	    dialog->setLength(
+		quint64(info.length()) *
+		quint64(info.tracks()));
 	    dialog->setBytePosition(src.size());
 	}
 
 	emitStatusInfo();
 	break;
     }
+
     if (!decoder) {
 	qWarning("unknown file type");
 	res = -EINVAL;
@@ -274,10 +289,10 @@ int SignalManager::loadFile(const KUrl &url)
 int SignalManager::save(const KUrl &url, bool selection)
 {
     int res = 0;
-    unsigned int ofs = 0;
-    unsigned int len = length();
+    sample_index_t ofs  = 0;
+    sample_index_t len  = length();
     unsigned int tracks = this->tracks();
-    unsigned int bits = this->bits();
+    unsigned int bits   = this->bits();
 
     if (selection) {
 	// zero-length -> nothing to do
@@ -288,7 +303,7 @@ int SignalManager::save(const KUrl &url, bool selection)
 
     if (!tracks || !len) {
 	Kwave::MessageBox::error(m_parent_widget,
-	    i18n("Signal is empty, nothing to save !"));
+	    i18n("Signal is empty, nothing to save."));
 	return 0;
     }
 
@@ -301,6 +316,7 @@ int SignalManager::save(const KUrl &url, bool selection)
 
     Encoder *encoder = CodecManager::encoder(mimetype_name);
     if (encoder) {
+
 	// maybe we now have a new mime type
 	m_file_info.set(INF_MIMETYPE, mimetype_name);
 
@@ -311,7 +327,7 @@ int SignalManager::save(const KUrl &url, bool selection)
 	bool all_supported = true;
 	QMap<FileProperty, QVariant>::Iterator it;
 	QString lost_properties;
-	for (it=properties.begin(); it!=properties.end(); ++it) {
+	for (it = properties.begin(); it != properties.end(); ++it) {
 	    if ( (! supported.contains(it.key())) &&
                  (m_file_info.canLoadSave(it.key())) )
 	    {
@@ -387,8 +403,8 @@ int SignalManager::save(const KUrl &url, bool selection)
 	    m_file_info.length(), m_file_info.rate(), m_file_info.bits(),
 	    m_file_info.tracks());
 	Q_ASSERT(dialog);
-	QObject::connect(&src,   SIGNAL(progress(unsigned int)),
-	                 dialog, SLOT(setValue(unsigned int)),
+	QObject::connect(&src,   SIGNAL(progress(qreal)),
+	                 dialog, SLOT(setValue(qreal)),
 	                 Qt::QueuedConnection);
 	QObject::connect(dialog, SIGNAL(canceled()),
 	                 &src,   SLOT(cancel()));
@@ -405,7 +421,7 @@ int SignalManager::save(const KUrl &url, bool selection)
 	    QMutableListIterator<Label> it(labels);
 	    while (it.hasNext()) {
 		Label &label = it.next();
-		unsigned int pos = label.pos();
+		sample_index_t pos = label.pos();
 		if ((pos < ofs) || (pos >= ofs + len)) {
 		    // out of the selected area -> remove
 		    it.remove();
@@ -426,7 +442,7 @@ int SignalManager::save(const KUrl &url, bool selection)
 	}
 	if (!encoded) {
 	    Kwave::MessageBox::error(m_parent_widget,
-	        i18n("An error occurred while saving the file!"));
+	        i18n("An error occurred while saving the file."));
 	    res = -1;
 	}
 
@@ -437,14 +453,14 @@ int SignalManager::save(const KUrl &url, bool selection)
 		// user really pressed cancel !
 		Kwave::MessageBox::error(m_parent_widget,
 		    i18n("The file has been truncated and "\
-		         "might be corrupted!"));
+		         "might be corrupted."));
 	    }
 	    dialog->deleteLater();
 	    qApp->flush();
 	}
     } else {
 	Kwave::MessageBox::error(m_parent_widget,
-	    i18n("Sorry, the file type is not supported!"));
+	    i18n("Sorry, the file type is not supported."));
 	res = -EINVAL;
     }
 
@@ -459,10 +475,10 @@ int SignalManager::save(const KUrl &url, bool selection)
 }
 
 //***************************************************************************
-void SignalManager::newSignal(unsigned int samples, double rate,
+void SignalManager::newSignal(sample_index_t samples, double rate,
                               unsigned int bits, unsigned int tracks)
 {
-    // enter and stay in not modified state
+    // enter and stay in modified state
     enableModifiedChange(true);
     setModified(true);
     enableModifiedChange(false);
@@ -495,6 +511,10 @@ void SignalManager::newSignal(unsigned int samples, double rate,
 //***************************************************************************
 void SignalManager::close()
 {
+    // stop the playback
+    m_playback_controller.playbackStop();
+    m_playback_controller.reset();
+
     // fix the modified flag to false
     enableModifiedChange(true);
     setModified(false);
@@ -511,13 +531,17 @@ void SignalManager::close()
     flushUndoBuffers();
     flushRedoBuffer();
 
+    // reset the selection
+    m_selection.clear();
+
     m_empty = true;
     while (tracks()) deleteTrack(tracks()-1);
     m_signal.close();
+
+    // clear all meta data
     m_file_info.clear();
 
     m_closed = true;
-    m_selection.select(0,0);
     rememberCurrentSelection();
 
     emitStatusInfo();
@@ -560,85 +584,79 @@ const QList<unsigned int> SignalManager::allTracks()
 }
 
 //***************************************************************************
-SampleWriter *SignalManager::openSampleWriter(unsigned int track,
-	InsertMode mode, unsigned int left, unsigned int right,
-	bool with_undo)
+Kwave::Writer *SignalManager::openWriter(unsigned int track,
+	InsertMode mode, sample_index_t left, sample_index_t right)
 {
-    SampleWriter *writer = m_signal.openSampleWriter(
-	track, mode, left, right);
-
-    // skip all that undo stuff below if undo is not enabled
-    // or the writer creation has failed
-    if (!undoEnabled() || !writer || !with_undo) return writer;
-
-    // get the real/effective left and right sample
-    left  = writer->first();
-    right = writer->last();
-
-    // enter a new undo transaction and let it close when the writer closes
-    UndoTransactionGuard guard(*this, 0);
-    startUndoTransaction();
-    QObject::connect(writer, SIGNAL(destroyed()),
-                     this,   SLOT(closeUndoTransaction()),
-                     Qt::QueuedConnection);
-
-    // create an undo action for the modification of the samples
-    UndoAction *undo = 0;
-    switch (mode) {
-	case Append:
-	case Insert: {
-	    QList<unsigned int> track_list;
-	    track_list.append(track);
-	    undo = new UndoInsertAction(
-		m_parent_widget, track_list, left, right-left+1);
-	    if (undo) {
-		QObject::connect(
-		    writer,
-		    SIGNAL(sigSamplesWritten(unsigned int)),
-		    static_cast<UndoInsertAction *>(undo),
-		    SLOT(setLength(unsigned int)));
-	    }
-	    break;
-	}
-	case Overwrite:
-	    undo = new UndoModifyAction(track, left, right-left+1);
-	    break;
-    }
-
-    if (!registerUndoAction(undo)) {
-	// aborted, do not continue without undo
-	delete writer;
-	return 0;
-    }
-
-    // Everything was ok, the action now is owned by the current undo
-    // transaction. The transaction is owned by the SignalManager and
-    // will be closed when the writer gets closed.
-    return writer;
+    return m_signal.openWriter(track, mode, left, right);
 }
 
 //***************************************************************************
-bool SignalManager::executeCommand(const QString &command)
+int SignalManager::executeCommand(const QString &command)
 {
-    unsigned int offset = m_selection.offset();
-    unsigned int length = m_selection.length();
+    sample_index_t offset = m_selection.offset();
+    sample_index_t length = m_selection.length();
 
-    if (!command.length()) return true;
+    if (!command.length()) return -EINVAL;
     Parser parser(command);
 
     if (false) {
+    // --- undo / redo ---
     CASE_COMMAND("undo")
 	undo();
     CASE_COMMAND("redo")
 	redo();
+
+    // --- copy & paste + clipboard ---
+    CASE_COMMAND("copy")
+	qDebug("copy(%lu,%lu)", static_cast<unsigned long int>(offset),
+	       static_cast<unsigned long int>(length));
+	if (length) {
+	    ClipBoard &clip = ClipBoard::instance();
+	    clip.copy(
+		m_parent_widget,
+		*this,
+		selectedTracks(),
+		offset, length
+	    );
+	    // remember the last selection
+	    rememberCurrentSelection();
+	}
+    CASE_COMMAND("paste")
+	ClipBoard &clip = ClipBoard::instance();
+	if (clip.isEmpty()) return 0;
+	if (!selectedTracks().size()) return 0;
+
+	UndoTransactionGuard undo(*this, i18n("Paste"));
+	clip.paste(m_parent_widget, *this, offset, length);
+    CASE_COMMAND("cut")
+	if (length) {
+	    // remember the last selection
+	    rememberCurrentSelection();
+
+	    ClipBoard &clip = ClipBoard::instance();
+	    clip.copy(
+		m_parent_widget,
+		*this,
+		selectedTracks(),
+		offset, length
+	    );
+	    UndoTransactionGuard undo(*this, i18n("Cut"));
+	    deleteRange(offset, length);
+	    selectRange(m_selection.offset(), 0);
+	}
+    CASE_COMMAND("clipboard_flush")
+	ClipBoard::instance().clear();
     CASE_COMMAND("crop")
-	UndoTransactionGuard undo(*this, i18n("crop"));
-	unsigned int rest = this->length() - offset;
+	UndoTransactionGuard undo(*this, i18n("Crop"));
+	sample_index_t rest = this->length() - offset;
 	rest = (rest > length) ? (rest-length) : 0;
 	QList<unsigned int> tracks = selectedTracks();
 	if (saveUndoDelete(tracks, offset+length, rest) &&
 	    saveUndoDelete(tracks, 0, offset))
 	{
+	    // remember the last selection
+	    rememberCurrentSelection();
+
 	    unsigned int count = tracks.count();
 	    while (count--) {
 		m_signal.deleteRange(count, offset+length, rest);
@@ -647,7 +665,10 @@ bool SignalManager::executeCommand(const QString &command)
 	    selectRange(0, length);
 	}
     CASE_COMMAND("delete")
+	UndoTransactionGuard undo(*this, i18n("Delete"));
 	deleteRange(offset, length);
+	selectRange(m_selection.offset(), 0);
+
 //    CASE_COMMAND("mixpaste")
 //	if (globals.clipboard) {
 //	    SignalManager *toinsert = globals.clipboard->getSignal();
@@ -681,24 +702,28 @@ bool SignalManager::executeCommand(const QString &command)
 //    CASE_COMMAND("invertchannels")
 //	for (unsigned int i = 0; i < m_channels; i++)
 //	    toggleChannel(i);
+    CASE_COMMAND("toggle_track_selection")
+	int track = parser.toInt();
+	UndoTransactionGuard undo(*this, i18n("Toggle Track Selection"));
+	selectTrack(track, !(trackSelected(track)));
     } else {
-	return false;
+	return -ENOSYS;
     }
 
-    return true;
+    return 0;
 }
 
 //***************************************************************************
 void SignalManager::appendTrack()
 {
-    UndoTransactionGuard u(*this, i18n("append track"));
+    UndoTransactionGuard u(*this, i18n("Append Track"));
     insertTrack(tracks());
 }
 
 //***************************************************************************
 void SignalManager::insertTrack(unsigned int index)
 {
-    UndoTransactionGuard u(*this, i18n("insert track"));
+    UndoTransactionGuard u(*this, i18n("Insert Track"));
 
     if (m_undo_enabled && !registerUndoAction(
 	new UndoInsertTrack(m_signal, index))) return;
@@ -709,7 +734,7 @@ void SignalManager::insertTrack(unsigned int index)
 
     // if the signal is currently empty, use the last
     // known length instead of the current one
-    unsigned int len = (count) ? length() : m_last_length;
+    sample_index_t len = (count) ? length() : m_last_length;
 
     if (index >= count) {
 	// do an "append"
@@ -728,7 +753,7 @@ void SignalManager::insertTrack(unsigned int index)
 //***************************************************************************
 void SignalManager::deleteTrack(unsigned int index)
 {
-    UndoTransactionGuard u(*this, i18n("delete track"));
+    UndoTransactionGuard u(*this, i18n("Delete Track"));
 
     if (m_undo_enabled && !registerUndoAction(
 	new UndoDeleteTrack(m_signal, index))) return;
@@ -758,7 +783,7 @@ void SignalManager::slotTrackDeleted(unsigned int index)
 
 //***************************************************************************
 void SignalManager::slotSamplesInserted(unsigned int track,
-	unsigned int offset, unsigned int length)
+	sample_index_t offset, sample_index_t length)
 {
     // remember the last known length
     m_last_length = m_signal.length();
@@ -770,7 +795,7 @@ void SignalManager::slotSamplesInserted(unsigned int track,
 	QMutableListIterator<Label> it(labels());
 	while (it.hasNext()) {
 	    Label &label = it.next();
-	    unsigned int pos = label.pos();
+	    sample_index_t pos = label.pos();
 	    if (pos >= offset) {
 		label.moveTo(pos + length);
 	    }
@@ -783,7 +808,7 @@ void SignalManager::slotSamplesInserted(unsigned int track,
 
 //***************************************************************************
 void SignalManager::slotSamplesDeleted(unsigned int track,
-	unsigned int offset, unsigned int length)
+	sample_index_t offset, sample_index_t length)
 {
     // remember the last known length
     m_last_length = m_signal.length();
@@ -796,24 +821,87 @@ void SignalManager::slotSamplesDeleted(unsigned int track,
 
 //***************************************************************************
 void SignalManager::slotSamplesModified(unsigned int track,
-	unsigned int offset, unsigned int length)
+	sample_index_t offset, sample_index_t length)
 {
     setModified(true);
     emit sigSamplesModified(track, offset, length);
 }
 
 //***************************************************************************
-SignalManager::~SignalManager()
+bool SignalManager::deleteRange(sample_index_t offset, sample_index_t length,
+                                const QList<unsigned int> &track_list)
 {
-    close();
+    if (!length || track_list.isEmpty()) return true; // nothing to do
+
+    // delete all labels in the selected range
+    // after that the selected area should be free of labels
+    // and labels are stored in seperate undo actions
+    foreach (Label label, labels()) {
+	const sample_index_t pos = label.pos();
+	const int index          = labelIndex(label);
+
+	if ((pos >= offset) && (pos < offset + length)) {
+	    // delete the label
+	    deleteLabel(index, true);
+	}
+    }
+
+    // store undo data for all tracks (without labels)
+    if (m_undo_enabled) {
+	if (!registerUndoAction(new UndoDeleteAction(
+	    m_parent_widget, track_list, offset, length))) {
+	    abortUndoTransaction();
+	    return false;
+	}
+    }
+
+    // delete the ranges in all tracks
+    // (this makes all label positions after the selected range invalid)
+    foreach (unsigned int track, track_list) {
+	m_signal.deleteRange(track, offset, length);
+    }
+
+    // adjust the labels after the deleted range
+    // without undo!
+    bool old_undo_enabled = m_undo_enabled;
+    m_undo_enabled = false;
+    foreach (Label label, labels()) {
+	const sample_index_t pos = label.pos();
+	const int index          = labelIndex(label);
+
+	if (pos >= offset + length) {
+	    // move label left
+	    qDebug("modifyLabel(index=%u, pos=%lu, name=%s)", index, 
+		static_cast<long unsigned int>(pos), label.name().toLocal8Bit().data());
+	    if (!modifyLabel(index, pos - length, label.name())) {
+		// this should hopefully not happen:
+		// new position is already occupied ?
+		qWarning("SignalManager::deleteRange() "\
+		         "-> killing duplicate label @ %lu", 
+		         static_cast<unsigned long int>(pos));
+		m_undo_enabled = old_undo_enabled;
+		deleteLabel(index, true);
+		m_undo_enabled = false;
+	    }
+	}
+    }
+    m_undo_enabled = old_undo_enabled;
+
+    return true;
 }
 
 //***************************************************************************
-bool SignalManager::deleteRange(unsigned int offset, unsigned int length,
+bool SignalManager::deleteRange(sample_index_t offset, sample_index_t length)
+{
+    return deleteRange(offset, length, selectedTracks());
+}
+
+//***************************************************************************
+bool SignalManager::insertSpace(sample_index_t offset, sample_index_t length,
                                 const QList<unsigned int> &track_list)
 {
     if (!length) return true; // nothing to do
-    UndoTransactionGuard undo(*this, i18n("delete"));
+    UndoTransactionGuard undo(*this, i18n("Insert Space"));
 
     unsigned int count = track_list.count();
     if (!count) return true; // nothing to do
@@ -821,55 +909,26 @@ bool SignalManager::deleteRange(unsigned int offset, unsigned int length,
     // first store undo data for all tracks
     unsigned int track;
     if (m_undo_enabled) {
-	if (!registerUndoAction(new UndoDeleteAction(
+	if (!registerUndoAction(new UndoInsertAction(
 	    m_parent_widget, track_list, offset, length))) return false;
     }
 
-    // then delete the ranges in all tracks
+    // then insert space into all tracks
     foreach (track, track_list) {
-	m_signal.deleteRange(track, offset, length);
+	m_signal.insertSpace(track, offset, length);
     }
 
-    // delete all labels in the selected range
-    // and adjust the labels after the deleted range
-    foreach (Label label, labels()) {
-	const unsigned int pos = label.pos();
-	const int index        = labelIndex(label);
-
-	if (pos >= offset + length) {
-	    // move label left
-	    if (!modifyLabel(index, pos - length, label.name())) {
-		// this should never happen: position is already occupied?
-		qWarning("SignalManager::deleteRange() "\
-		         "-> killing duplicate label @ %u", pos);
-		deleteLabel(index, false);
-	    }
-	} else if ((pos >= offset) && (pos < offset + length)) {
-	    // delete the label
-	    deleteLabel(index, false);
-	}
-    }
-
-    // finally set the current selection to zero-length
-    selectRange(m_selection.offset(), 0);
-
     return true;
 }
 
 //***************************************************************************
-bool SignalManager::deleteRange(unsigned int offset, unsigned int length)
-{
-    return deleteRange(offset, length, selectedTracks());
-}
-
-//***************************************************************************
-void SignalManager::selectRange(unsigned int offset, unsigned int length)
+void SignalManager::selectRange(sample_index_t offset, sample_index_t length)
 {
     // first do some range checking
-    unsigned int len = this->length();
+    sample_index_t len = this->length();
 
-    if (offset >= len) offset = len ? (len-1) : 0;
-    if ((offset+length) > len) length = len - offset;
+    if (offset >= len) offset = len ? (len - 1) : 0;
+    if ((offset + length) > len) length = len - offset;
 
     m_selection.select(offset, length);
 }
@@ -979,8 +1038,7 @@ void SignalManager::closeUndoTransaction()
 			UndoAction *redo_action;
 
 			// unqueue the undo action
-			undo_action = m_undo_transaction->nextUndo();
-			m_undo_transaction->removeAll(undo_action);
+			undo_action = m_undo_transaction->takeLast();
 			Q_ASSERT(undo_action);
 			if (!undo_action) continue;
 
@@ -1004,6 +1062,10 @@ void SignalManager::closeUndoTransaction()
 	    }
 	}
 
+	// dump, for debugging
+// 	if (m_undo_transaction)
+// 	    m_undo_transaction->dump("closed undo transaction: ");
+
 	// declare the current transaction as "closed"
 	rememberCurrentSelection();
 	m_undo_transaction = 0;
@@ -1119,11 +1181,10 @@ bool SignalManager::registerUndoAction(UndoAction *action)
     Q_ASSERT(action);
     if (!action) return continueWithoutUndo();
 
-    // if undo is not enabled, this will fail -> no memory leak!
-    Q_ASSERT(m_undo_enabled);
+    // if undo is not enabled, this will fail -> discard the action
     if (!m_undo_enabled) {
 	delete action;
-	return continueWithoutUndo();
+	return true;
     }
 
     // check if the undo action is too large
@@ -1162,7 +1223,7 @@ bool SignalManager::registerUndoAction(UndoAction *action)
 
 //***************************************************************************
 bool SignalManager::saveUndoDelete(QList<unsigned int> &track_list,
-                                   unsigned int offset, unsigned int length)
+                                   sample_index_t offset, sample_index_t length)
 {
     if (!m_undo_enabled) return true;
     if (track_list.isEmpty()) return true;
@@ -1232,14 +1293,14 @@ void SignalManager::emitUndoRedoInfo()
 	if (!m_undo_buffer.isEmpty()) {
 	    transaction = m_undo_buffer.last();
 	    if (transaction) undo_name = transaction->description();
-	    if (!undo_name.length()) undo_name = i18n("last action");
+	    if (!undo_name.length()) undo_name = i18n("Last Action");
 	}
 
 	// get the description of the last redo action
 	if (!m_redo_buffer.isEmpty()) {
 	    transaction = m_redo_buffer.first();
 	    if (transaction) redo_name = transaction->description();
-	    if (!redo_name.length()) redo_name = i18n("last action");
+	    if (!redo_name.length()) redo_name = i18n("Last Action");
 	}
     }
 
@@ -1252,11 +1313,20 @@ void SignalManager::undo()
 {
     QMutexLocker lock(&m_undo_transaction_lock);
 
+    // check for modified selection
+    checkSelectionChange();
+
+    // remember the last selection
+    rememberCurrentSelection();
+
     // get the last undo transaction and abort if none present
     if (m_undo_buffer.isEmpty()) return;
     UndoTransaction *undo_transaction = m_undo_buffer.takeLast();
     if (!undo_transaction) return;
 
+    // dump, for debugging
+//     undo_transaction->dump("before undo: ");
+
     // temporarily disable undo while undo itself is running
     bool old_undo_enabled = m_undo_enabled;
     m_undo_enabled = false;
@@ -1297,8 +1367,7 @@ void SignalManager::undo()
 	UndoAction *redo_action;
 
 	// unqueue the undo action
-	undo_action = undo_transaction->nextUndo();
-	undo_transaction->removeAll(undo_action);
+	undo_action = undo_transaction->takeLast();
 	Q_ASSERT(undo_action);
 	if (!undo_action) continue;
 
@@ -1332,6 +1401,22 @@ void SignalManager::undo()
 	redo_transaction = 0;
     }
 
+    // check whether the selection has changed, if yes: put a undo action
+    // for this selection change at the end of the redo transaction
+    if (redo_transaction) {
+	bool range_modified = !(m_selection == m_last_selection);
+	QList<unsigned int> tracks = selectedTracks();
+	bool tracks_modified = !(tracks == m_last_track_selection);
+	if (range_modified || tracks_modified) {
+	    UndoAction *redo_action = new UndoSelection(*this,
+		m_last_track_selection,
+		m_last_selection.offset(),
+		m_last_selection.length());
+	    Q_ASSERT(redo_action);
+	    if (redo_action) redo_transaction->append(redo_action);
+	}
+    }
+
     // find out if there is still an action in the undo buffer
     // that has to do with modification of the signal
     if (m_modified) {
@@ -1372,16 +1457,13 @@ void SignalManager::redo()
 {
     QMutexLocker lock(&m_undo_transaction_lock);
 
-    // check for modified selection
-    checkSelectionChange();
-
     // get the last redo transaction and abort if none present
     if (m_redo_buffer.isEmpty()) return;
-    UndoTransaction *redo_transaction = m_redo_buffer.first();
+    UndoTransaction *redo_transaction = m_redo_buffer.takeFirst();
     if (!redo_transaction) return;
 
-    // remove the redo transaction from the list without deleting it
-    m_redo_buffer.takeFirst();
+    // check for modified selection
+    checkSelectionChange();
 
     // temporarily disable undo while redo is running
     bool old_undo_enabled = m_undo_enabled;
@@ -1425,8 +1507,7 @@ void SignalManager::redo()
 	UndoAction *redo_action;
 
 	// unqueue the undo action
-	redo_action = redo_transaction->nextRedo();
-	redo_transaction->removeAll(redo_action);
+	redo_action = redo_transaction->takeFirst();
 
 	// execute the redo operation
 	Q_ASSERT(redo_action);
@@ -1480,7 +1561,7 @@ void SignalManager::setModified(bool mod)
 
     if (m_modified != mod) {
 	m_modified = mod;
-	qDebug("SignalManager::setModified(%d)",mod);
+// 	qDebug("SignalManager::setModified(%d)",mod);
 	emit sigModified(m_modified);
     }
 }
@@ -1496,7 +1577,7 @@ void SignalManager::setFileInfo(FileInfo &new_info, bool with_undo)
 {
     if (m_undo_enabled && with_undo) {
 	/* save data for undo */
-	UndoTransactionGuard undo_transaction(*this, i18n("modify file info"));
+	UndoTransactionGuard undo_transaction(*this, i18n("Modify File Info"));
 	if (!registerUndoAction(new UndoFileInfo(*this))) return;
     }
 
@@ -1507,7 +1588,7 @@ void SignalManager::setFileInfo(FileInfo &new_info, bool with_undo)
 }
 
 //***************************************************************************
-Label SignalManager::findLabel(unsigned int pos)
+Label SignalManager::findLabel(sample_index_t pos)
 {
     QMutableListIterator<Label> it(labels());
     while (it.hasNext())
@@ -1538,7 +1619,7 @@ Label SignalManager::labelAtIndex(int index)
 }
 
 //***************************************************************************
-bool SignalManager::addLabel(unsigned int pos)
+bool SignalManager::addLabel(sample_index_t pos)
 {
     // if there already is a label at the given position, do nothing
     if (!findLabel(pos).isNull()) return false;
@@ -1551,19 +1632,22 @@ bool SignalManager::addLabel(unsigned int pos)
     labels().sort();
 
     // register the undo action
-    UndoTransactionGuard undo(*this, i18n("add label"));
+    UndoTransactionGuard undo(*this, i18n("Add Label"));
     if (!registerUndoAction(new UndoAddLabelAction(labelIndex(label)))) {
 	labels().removeAll(label);
 	return false;
     }
 
+    // register this as a modification
+    setModified(true);
+
     emit sigLabelCountChanged();
     emit labelsChanged(labels());
     return true;
 }
 
 //***************************************************************************
-Label SignalManager::addLabel(unsigned int pos, const QString &name)
+Label SignalManager::addLabel(sample_index_t pos, const QString &name)
 {
     // if there already is a label at the given position, do nothing
     if (!findLabel(pos).isNull()) return Label();
@@ -1574,6 +1658,10 @@ Label SignalManager::addLabel(unsigned int pos, const QString &name)
     // put the label into the list
     labels().append(label);
     labels().sort();
+
+    // register this as a modification
+    setModified(true);
+
     emit sigLabelCountChanged();
     emit labelsChanged(labels());
 
@@ -1591,18 +1679,22 @@ void SignalManager::deleteLabel(int index, bool with_undo)
 
     // register the undo action
     if (with_undo) {
-	UndoTransactionGuard undo(*this, i18n("delete label"));
+	UndoTransactionGuard undo(*this, i18n("Delete Label"));
 	if (!registerUndoAction(new UndoDeleteLabelAction(label)))
 	    return;
     }
 
     labels().removeAll(label);
+
+    // register this as a modification
+    setModified(true);
+
     emit sigLabelCountChanged();
     emit labelsChanged(labels());
 }
 
 //***************************************************************************
-bool SignalManager::modifyLabel(int index, unsigned int pos,
+bool SignalManager::modifyLabel(int index, sample_index_t pos,
                                 const QString &name)
 {
     Q_ASSERT(index >= 0);
@@ -1618,10 +1710,24 @@ bool SignalManager::modifyLabel(int index, unsigned int pos,
     if ((pos != label.pos()) && !findLabel(pos).isNull())
 	return false;
 
+    // add a undo action
+    if (m_undo_enabled) {
+	UndoModifyLabelAction *undo_modify = new UndoModifyLabelAction(label);
+	if (!registerUndoAction(undo_modify))
+	    return false;
+	// now store the label's current position,
+	// for finding it again later
+	undo_modify->setLastPosition(pos);
+    }
+
+    // now modify the label
     label.moveTo(pos);
     label.rename(name);
     labels().sort();
 
+    // register this as a modification
+    setModified(true);
+
     emit labelsChanged(labels());
     return true;
 }
@@ -1648,6 +1754,8 @@ void SignalManager::checkSelectionChange()
     if (range_modified || tracks_modified) {
 	// selection has changed since last undo/redo operation
 // 	qDebug("SignalManager::checkSelectionChange() => manually modified");
+// 	qDebug("    before: [%8u...%8u]", m_last_selection.first(), m_last_selection.last());
+// 	qDebug("    after : [%8u...%8u]", m_selection.first(),      m_selection.last());
 
 	// temporarily activate the previous selection (last stored)
 	Selection new_selection(m_selection);
@@ -1656,9 +1764,9 @@ void SignalManager::checkSelectionChange()
 
 	// save the last selection into a undo action
 	if (tracks_modified && !range_modified)
-	    UndoTransactionGuard undo(*this, i18n("manual track selection"));
+	    UndoTransactionGuard undo(*this, i18n("Manual Track Selection"));
 	else
-	    UndoTransactionGuard undo(*this, i18n("manual selection"));
+	    UndoTransactionGuard undo(*this, i18n("Manual Selection"));
 
 	// restore the current selection again
 	m_selection = new_selection;
