@@ -23,6 +23,7 @@
 #include <stdio.h>
 
 #include <QList>
+#include <QMap>
 #include <QMutex>
 #include <QObject>
 #include <QString>
@@ -47,6 +48,7 @@ class Track;
 class UndoAction;
 class UndoTransaction;
 class UndoTransactionGuard;
+namespace Kwave { class MultiTrackWriter; }
 namespace Kwave { class Writer; }
 
 #define NEW_FILENAME i18n("New File")
@@ -81,7 +83,7 @@ public:
      * @param bits number of bits per sample
      * @param tracks number of tracks
      */
-    void newSignal(unsigned int samples, double rate,
+    void newSignal(sample_index_t samples, double rate,
                    unsigned int bits, unsigned int tracks);
 
     /**
@@ -101,8 +103,13 @@ public:
     /** Returns a reference to the playback controller. */
     PlaybackController &playbackController();
 
-    /** Executes a Kwave text command */
-    bool executeCommand(const QString &command);
+    /**
+     * Execute a Kwave text command
+     * @param command a text command
+     * @return zero if succeeded or negative error code if failed
+     * @retval -ENOSYS is returned if the command is unknown in this component
+     */
+    int executeCommand(const QString &command);
 
     /**
      * Returns a reference to the FileInfo object associated with the
@@ -137,7 +144,7 @@ public:
      * Returns the number of samples in the current signal. Will be
      * zero if no signal is loaded.
      */
-    inline unsigned int length() { return m_signal.length(); }
+    inline sample_index_t length() { return m_signal.length(); }
 
     /** Returns a reference to the current selection */
     inline Selection &selection() { return m_selection; }
@@ -177,7 +184,7 @@ public:
      * @return true if successful or nothing to do, false if not enough
      *         memory for undo
      */
-    bool deleteRange(unsigned int offset, unsigned int length,
+    bool deleteRange(sample_index_t offset, sample_index_t length,
                      const QList<unsigned int> &track_list);
 
     /**
@@ -188,7 +195,7 @@ public:
      * @return true if successful or nothing to do, false if not enough
      *         memory for undo
      */
-    bool deleteRange(unsigned int offset, unsigned int length);
+    bool deleteRange(sample_index_t offset, sample_index_t length);
 
     /**
      * Inserts space at a given position and creates an undo action.
@@ -198,7 +205,7 @@ public:
      * @return true if successful or nothing to do, false if not enough
      *         memory for undo
      */
-    bool insertSpace(unsigned int offset, unsigned int length,
+    bool insertSpace(sample_index_t offset, sample_index_t length,
                      const QList<unsigned int> &track_list);
 
     /**
@@ -206,7 +213,7 @@ public:
      * @param offset index of the first sample
      * @param length number of samples
      */
-    void selectRange(unsigned int offset, unsigned int length);
+    void selectRange(sample_index_t offset, sample_index_t length);
 
     /**
      * Inserts a new track with the size of the current signal after
@@ -251,12 +258,10 @@ public:
      * @param left start of the input (only useful in insert and
      *             overwrite mode)
      * @param right end of the input (only useful with overwrite mode)
-     * @param with_undo if true, an undo action will be created
      * @see InsertMode
      */
     Kwave::Writer *openWriter(unsigned int track, InsertMode mode,
-	unsigned int left = 0, unsigned int right = 0,
-	bool with_undo = false);
+	sample_index_t left = 0, sample_index_t right = 0);
 
     /**
      * Opens a stream for reading samples. If the the last position
@@ -269,21 +274,21 @@ public:
      */
     inline SampleReader *openSampleReader(Kwave::ReaderMode mode,
 	unsigned int track,
-	unsigned int left = 0, unsigned int right = UINT_MAX)
+	sample_index_t left = 0, sample_index_t right = SAMPLE_INDEX_MAX)
     {
 	return m_signal.openSampleReader(mode, track, left, right);
     }
 
     /** Returns true if undo/redo is currently enabled */
-    inline bool undoEnabled() { return m_undo_enabled; }
+    inline bool undoEnabled() const { return m_undo_enabled; }
 
     /** Return true if undo is possible */
-    inline bool canUndo() {
+    inline bool canUndo() const {
 	return !m_undo_buffer.isEmpty() && undoEnabled();
     }
 
     /** Return true if redo is possible */
-    inline bool canRedo() {
+    inline bool canRedo() const {
 	return !m_redo_buffer.isEmpty() && undoEnabled();
     }
 
@@ -311,7 +316,7 @@ public:
      * @param pos position of the label [samples]
      * @return true if successful, false if failed
      */
-    bool addLabel(unsigned int pos);
+    bool addLabel(sample_index_t pos);
 
     /**
      * add a new label, without undo
@@ -319,7 +324,7 @@ public:
      * @param name the name of the label
      * @return pointer to the new created label
      */
-    Label addLabel(unsigned int pos, const QString &name);
+    Label addLabel(sample_index_t pos, const QString &name);
 
     /**
      * delete an existing label
@@ -337,7 +342,7 @@ public:
      * @return true if succeeded, false if the index is out of range or if
      *         the new position is already occupied by an existing label
      */
-    bool modifyLabel(int index, unsigned int pos, const QString &name);
+    bool modifyLabel(int index, sample_index_t pos, const QString &name);
 
     /**
      * find a label by it's index
@@ -358,7 +363,7 @@ public:
      * @param pos position of the label [samples]
      * @return valid label at the position or null label if not found
      */
-    Label findLabel(unsigned int pos);
+    Label findLabel(sample_index_t pos);
 
     /** Shortcut for accessing the label list @note can be modified */
     inline LabelList &labels() { return m_file_info.labels(); }
@@ -370,7 +375,7 @@ signals:
      * @param offset index of the first selected sample
      * @param length number of selected samples
      */
-    void sigSelectionChanged(unsigned int offset, unsigned int length);
+    void sigSelectionChanged(sample_index_t offset, sample_index_t length);
 
     /**
      * Signals that a track has been inserted.
@@ -393,7 +398,7 @@ signals:
      * @param rate sample rate [samples/second]
      * @param bits resolution in bits
      */
-    void sigStatusInfo(unsigned int length, unsigned int tracks,
+    void sigStatusInfo(sample_index_t length, unsigned int tracks,
                        double rate, unsigned int bits);
 
     /**
@@ -405,8 +410,8 @@ signals:
      * @param length number of samples inserted
      * @see sigSamplesModified
      */
-    void sigSamplesInserted(unsigned int track, unsigned int offset,
-                            unsigned int length);
+    void sigSamplesInserted(unsigned int track, sample_index_t offset,
+                            sample_index_t length);
 
     /**
      * Emitted if samples have been removed from a track.
@@ -414,8 +419,8 @@ signals:
      * @param offset position from which the data was removed
      * @param length number of samples deleted
      */
-    void sigSamplesDeleted(unsigned int track, unsigned int offset,
-                           unsigned int length);
+    void sigSamplesDeleted(unsigned int track, sample_index_t offset,
+                           sample_index_t length);
 
     /**
      * Emitted if samples within a track have been modified.
@@ -423,8 +428,15 @@ signals:
      * @param offset position from which the data was modified
      * @param length number of samples modified
      */
-    void sigSamplesModified(unsigned int track, unsigned int offset,
-                            unsigned int length);
+    void sigSamplesModified(unsigned int track, sample_index_t offset,
+                            sample_index_t length);
+
+
+    /** Emitted whenever the numer of labels has changed */
+    void sigLabelCountChanged();
+
+    /** Emitted whenever a label has been added, deleted or modified */
+    void labelsChanged(const LabelList &labels);
 
     /**
      * Emitted if the state or description of undo/redo has changed. If
@@ -440,12 +452,6 @@ signals:
      */
     void sigModified(bool modified);
 
-    /** Emitted whenever the numer of labels has changed */
-    void sigLabelCountChanged();
-
-    /** Emitted whenever a label has been added, deleted or modified */
-    void labelsChanged(const LabelList &labels);
-
 public slots:
 
     /**
@@ -485,8 +491,8 @@ private slots:
      * @see Signal::sigSamplesInserted
      * @internal
      */
-    void slotSamplesInserted(unsigned int track, unsigned int offset,
-                             unsigned int length);
+    void slotSamplesInserted(unsigned int track, sample_index_t offset,
+                             sample_index_t length);
 
     /**
      * Connected to the signal's sigSamplesDeleted.
@@ -496,8 +502,8 @@ private slots:
      * @see Signal::sigSamplesDeleted
      * @internal
      */
-    void slotSamplesDeleted(unsigned int track, unsigned int offset,
-                            unsigned int length);
+    void slotSamplesDeleted(unsigned int track, sample_index_t offset,
+                            sample_index_t length);
 
     /**
      * Connected to the signal's sigSamplesModified
@@ -507,8 +513,8 @@ private slots:
      * @see Signal::sigSamplesModified
      * @internal
      */
-    void slotSamplesModified(unsigned int track, unsigned int offset,
-                             unsigned int length);
+    void slotSamplesModified(unsigned int track, sample_index_t offset,
+                             sample_index_t length);
 
     /**
      * Closes an undo transaction or recurses the current recursion level
@@ -538,17 +544,18 @@ protected:
      * @param length number of samples
      * @see Signal::deleteRange
      */
-    inline void deleteRange(unsigned int track, unsigned int offset,
-                            unsigned int length)
+    inline void deleteRange(unsigned int track, sample_index_t offset,
+                            sample_index_t length)
     {
 	m_signal.deleteRange(track, offset, length);
     }
 
 protected:
 
-    friend class UndoTransactionGuard;
+    friend class Kwave::MultiTrackWriter;
     friend class PluginManager;
     friend class SignalWidget;
+    friend class UndoTransactionGuard;
 
     /**
      * Tries to free memory for a new undo action and stores all needed
@@ -574,7 +581,7 @@ protected:
      * @return true if successful, false if out of memory or aborted
      */
     bool saveUndoDelete(QList<unsigned int> &track_list,
-                        unsigned int offset, unsigned int length);
+                        sample_index_t offset, sample_index_t length);
 
     /**
      * Aborts an undo transaction by deleting all of it's undo actions.
@@ -606,6 +613,9 @@ protected:
      */
     void setModified(bool mod);
 
+    /** returns the associated parent widget, to be used for messages */
+    QWidget *parentWidget() const { return m_parent_widget; }
+
 private:
 
     /** Shortcut for emitting a sigStatusInfo */
@@ -689,7 +699,7 @@ private:
      * added to an empty signal and prevents from the creation of a
      * completely empty new signal.
      */
-    unsigned int m_last_length;
+    sample_index_t m_last_length;
 
     /** the controller for handling of playback */
     PlaybackController m_playback_controller;
