@@ -29,7 +29,7 @@
 //***************************************************************************
 UndoDeleteAction::UndoDeleteAction(QWidget *parent_widget,
                                    const QList<unsigned int> &track_list,
-                                   unsigned int offset, unsigned int length)
+                                   sample_index_t offset, sample_index_t length)
     :UndoAction(),
      m_parent_widget(parent_widget),
      m_track_list(track_list),
@@ -49,7 +49,7 @@ UndoDeleteAction::~UndoDeleteAction()
 //***************************************************************************
 QString UndoDeleteAction::description()
 {
-    return i18n("delete");
+    return i18n("Delete");
 }
 
 //***************************************************************************
@@ -67,6 +67,8 @@ int UndoDeleteAction::redoSize()
 //***************************************************************************
 bool UndoDeleteAction::store(SignalManager &manager)
 {
+    if (!m_length) return true; // shortcut: this is an empty action
+
     MultiTrackReader reader(Kwave::SinglePassForward, manager,
 	m_track_list, m_offset, m_offset + m_length - 1);
 
@@ -93,6 +95,8 @@ UndoAction *UndoDeleteAction::undo(SignalManager &manager, bool with_redo)
 	redo_action->store(manager);
     }
 
+    if (!m_length) return redo_action; // shortcut: this is an empty action
+
     // perform the undo operation
     if (!m_mime_data.decode(m_parent_widget, &m_mime_data,
                             manager, m_offset))
@@ -108,10 +112,12 @@ UndoAction *UndoDeleteAction::undo(SignalManager &manager, bool with_redo)
 //***************************************************************************
 void UndoDeleteAction::dump(const QString &indent)
 {
-    qDebug("%sundo delete from [%u ... %u] (%d)", indent.toLocal8Bit().data(),
-           m_offset,
-           m_offset + ((m_length) ? (m_length - 1) : m_length),
-           m_length);
+    qDebug("%sundo delete from [%lu ... %lu] (%lu)",
+           indent.toLocal8Bit().data(),
+           static_cast<unsigned long int>(m_offset),
+           static_cast<unsigned long int>(m_offset + ((m_length) ?
+               (m_length - 1) : m_length)),
+           static_cast<unsigned long int>(m_length));
 }
 
 //***************************************************************************
