@@ -39,7 +39,7 @@ UndoDeleteLabelAction::~UndoDeleteLabelAction()
 //***************************************************************************
 QString UndoDeleteLabelAction::description()
 {
-    return i18n("delete label");
+    return i18n("Delete Label");
 }
 
 //***************************************************************************
@@ -85,5 +85,15 @@ UndoAction *UndoDeleteLabelAction::undo(SignalManager &manager,
     return redo;
 }
 
+//***************************************************************************
+void UndoDeleteLabelAction::dump(const QString &indent)
+{
+    qDebug("%sundo delete label, %lu, '%s'",
+           indent.toLocal8Bit().data(),
+           static_cast<unsigned long int>(m_label.pos()),
+           m_label.name().toLocal8Bit().data()
+          );
+}
+
 //***************************************************************************
 //***************************************************************************
