@@ -28,7 +28,8 @@
 #include "BandPassPlugin.h"
 #include "BandPassDialog.h"
 
-KWAVE_PLUGIN(BandPassPlugin,"band_pass","Dave Flogeras");
+KWAVE_PLUGIN(BandPassPlugin, "band_pass", "2.1",
+             I18N_NOOP("Band Pass Filter"), "Dave Flogeras");
 
 //***************************************************************************
 BandPassPlugin::BandPassPlugin(const PluginContext &context)
@@ -75,10 +76,10 @@ KwavePluginSetupDialog *BandPassPlugin::createDialog(QWidget *parent)
     if (!dialog) return 0;
 
     // connect the signals for detecting value changes in pre-listen mode
-    connect(dialog, SIGNAL(freqChanged(qreal)),
-            this, SLOT(setFreqValue(qreal)));
-    connect(dialog, SIGNAL(bwChanged(qreal)),
-            this, SLOT(setBwValue(qreal)));
+    connect(dialog, SIGNAL(freqChanged(double)),
+            this, SLOT(setFreqValue(double)));
+    connect(dialog, SIGNAL(bwChanged(double)),
+            this, SLOT(setBwValue(double)));
     return dialog;
 }
 
@@ -98,7 +99,7 @@ bool BandPassPlugin::paramsChanged()
 void BandPassPlugin::updateFilter(Kwave::SampleSource *filter,
                                   bool force)
 {
-    qreal sr = signalRate();
+    double sr = signalRate();
 
     if (!filter) return;
 
@@ -117,17 +118,17 @@ void BandPassPlugin::updateFilter(Kwave::SampleSource *filter,
 //***************************************************************************
 QString BandPassPlugin::actionName()
 {
-    return i18n("band_pass");
+    return i18n("Band Pass");
 }
 
 //***************************************************************************
-void BandPassPlugin::setFreqValue(qreal frequency)
+void BandPassPlugin::setFreqValue(double frequency)
 {
     m_frequency = frequency;
 }
 
 //***************************************************************************
-void BandPassPlugin::setBwValue(qreal bw)
+void BandPassPlugin::setBwValue(double bw)
 {
     m_bw = bw;
 }
