@@ -24,10 +24,10 @@
 
 #include "libkwave/CompressionType.h"
 #include "libkwave/MessageBox.h"
-#include "libkwave/MultiTrackWriter.h"
+#include "libkwave/MultiWriter.h"
 #include "libkwave/Sample.h"
-#include "libkwave/SampleWriter.h"
 #include "libkwave/Signal.h"
+#include "libkwave/Writer.h"
 
 #include "AsciiCodecPlugin.h"
 #include "AsciiDecoder.h"
@@ -92,7 +92,7 @@ bool AsciiDecoder::open(QWidget *widget, QIODevice &src)
     qDebug("--- THE ASCII DECODER IS NOT FUNCTIONAL YET ---");
     qDebug("---           sorry :-(                     ---");
     Kwave::MessageBox::sorry(widget,
-	i18n("not implemented yet"),
+	i18n("This is not implemented yet."),
 	i18n("Sorry"));
     return false;
 
@@ -100,7 +100,7 @@ bool AsciiDecoder::open(QWidget *widget, QIODevice &src)
 }
 
 //***************************************************************************
-bool AsciiDecoder::decode(QWidget * /* widget */, MultiTrackWriter &dst)
+bool AsciiDecoder::decode(QWidget * /* widget */, Kwave::MultiWriter &dst)
 {
     Q_ASSERT(m_source);
     if (!m_source) return false;
