@@ -24,10 +24,10 @@
 
 #include "libkwave/CompressionType.h"
 #include "libkwave/MessageBox.h"
-#include "libkwave/MultiTrackWriter.h"
+#include "libkwave/MultiWriter.h"
 #include "libkwave/Sample.h"
-#include "libkwave/SampleWriter.h"
 #include "libkwave/Signal.h"
+#include "libkwave/Writer.h"
 
 #include "FlacCodecPlugin.h"
 #include "FlacDecoder.h"
@@ -109,9 +109,9 @@ Decoder *FlacDecoder::instance()
     unsigned int mul = (1 << shift);
 
     // decode the samples into a temporary buffer and
-    // flush it to the SampleWriter(s), track by track
+    // flush it to the Writer(s), track by track
     for (unsigned int track=0; track < tracks; track++) {
-	SampleWriter *writer = (*m_dest)[track];
+	Kwave::Writer *writer = (*m_dest)[track];
 	Q_ASSERT(writer);
 	if (!writer) continue;
 	register const FLAC__int32 *src = buffer[track];
@@ -276,14 +276,14 @@ bool FlacDecoder::open(QWidget *widget, QIODevice &src)
     FLAC__StreamDecoderInitStatus init_state = init();
     if (init_state > FLAC__STREAM_DECODER_INIT_STATUS_OK) {
         Kwave::MessageBox::error(widget, i18n(
-           "opening the FLAC bitstream failed."));
+           "Opening the FLAC bitstream failed."));
         return false;
     }
 #else /* API v1.1.2 and older */
     FLAC::Decoder::Stream::State init_state = init();
     if (init_state >= FLAC__STREAM_DECODER_END_OF_STREAM) {
         Kwave::MessageBox::error(widget, i18n(
-           "opening the FLAC bitstream failed."));
+           "Opening the FLAC bitstream failed."));
         return false;
     }
 #endif
@@ -294,7 +294,7 @@ bool FlacDecoder::open(QWidget *widget, QIODevice &src)
     FLAC::Decoder::Stream::State state = get_state();
     if (state >= FLAC__STREAM_DECODER_END_OF_STREAM) {
 	Kwave::MessageBox::error(widget, i18n(
-	   "error while parsing FLAC metadata. (%s)"),
+	   "Error while parsing the FLAC metadata. (%s)"),
 	   state.as_cstring());
 	return false;
     }
@@ -307,7 +307,7 @@ bool FlacDecoder::open(QWidget *widget, QIODevice &src)
 }
 
 //***************************************************************************
-bool FlacDecoder::decode(QWidget * /* widget */, MultiTrackWriter &dst)
+bool FlacDecoder::decode(QWidget * /* widget */, Kwave::MultiWriter &dst)
 {
     Q_ASSERT(m_source);
     if (!m_source) return false;
