@@ -102,7 +102,7 @@ void WavEncoder::fixAudiofileBrokenHeaderBug(QIODevice &dst, FileInfo &info,
     dst.read(reinterpret_cast<char *>(&data_size), 4);
     data_size = LE32_TO_CPU(data_size);
     if (data_size == length * frame_size) {
-	qDebug("(data size written by libaudiofile is correct)");
+// 	qDebug("(data size written by libaudiofile is correct)");
 	return;
     }
 
@@ -323,8 +323,6 @@ bool WavEncoder::encode(QWidget *widget, MultiTrackReader &src,
     if (!bits) bits = 16;
 
     // check for a valid source
-    Q_ASSERT(tracks);
-    Q_ASSERT(length);
     if ((!tracks) || (!length)) return false;
     Q_ASSERT(src.tracks() == tracks);
     if (src.tracks() != tracks) return false;
@@ -332,12 +330,12 @@ bool WavEncoder::encode(QWidget *widget, MultiTrackReader &src,
     // check if the choosen compression mode is supported for saving
     if ((compression != AF_COMPRESSION_NONE) &&
         (compression != AF_COMPRESSION_G711_ULAW) &&
-        (compression != AF_COMPRESSION_G711_ALAW) )
+        (compression != AF_COMPRESSION_G711_ALAW))
     {
 	qWarning("compression mode %d not supported!", compression);
 	int what_now = Kwave::MessageBox::warningYesNoCancel(widget,
-	    i18n("Sorry, the currently selected compression type can "
-	         "not be used for saving. Do you want to use "
+	    i18n("Sorry, the currently selected compression type cannot "
+	         "be used for saving. Do you want to use "
 	         "G711 ULAW compression instead?"), QString(),
 	    i18n("&Yes, use G711"),
 	    i18n("&No, store uncompressed")
@@ -345,8 +343,10 @@ bool WavEncoder::encode(QWidget *widget, MultiTrackReader &src,
 	switch (what_now) {
 	    case (KMessageBox::Yes):
 		compression = AF_COMPRESSION_G711_ULAW;
+		info.set(INF_COMPRESSION, AF_COMPRESSION_G711_ULAW);
 		break;
 	    case (KMessageBox::No):
+		info.set(INF_COMPRESSION, AF_COMPRESSION_NONE);
 		compression = AF_COMPRESSION_NONE;
 		break;
 	    default:
@@ -354,21 +354,33 @@ bool WavEncoder::encode(QWidget *widget, MultiTrackReader &src,
 	}
     }
 
-    // check for unsupported bits/sample format combinations
-    if (compression == AF_COMPRESSION_NONE) {
-	if ((bits <= 8) && (sample_format != AF_SAMPFMT_UNSIGNED)) {
-	    const SampleFormat format(SampleFormat::Unsigned);
-	    info.set(INF_SAMPLE_FORMAT, QVariant(format.toInt()));
-
-	    sample_format = AF_SAMPFMT_UNSIGNED;
-	    qDebug("auto-switching to unsigned format");
-	} else if ((bits > 8) && (sample_format != AF_SAMPFMT_TWOSCOMP)) {
+    // check for unsupported compression/bits/sample format combinations
+    // G.711 and MSADPCM support only 16 bit signed as input format!
+    if ((compression == AF_COMPRESSION_G711_ULAW) ||
+        (compression == AF_COMPRESSION_G711_ALAW))
+    {
+	if ((sample_format != AF_SAMPFMT_TWOSCOMP) &&
+	    (bits          != 16))
+	{
 	    const SampleFormat format(SampleFormat::Signed);
 	    info.set(INF_SAMPLE_FORMAT, QVariant(format.toInt()));
-
+	    info.setBits(16);
 	    sample_format = AF_SAMPFMT_TWOSCOMP;
-	    qDebug("auto-switching to signed format");
+	    bits          = 16;
+	    qDebug("auto-switching to 16 bit signed format");
 	}
+    } else if ((bits <= 8) && (sample_format != AF_SAMPFMT_UNSIGNED)) {
+	const SampleFormat format(SampleFormat::Unsigned);
+	info.set(INF_SAMPLE_FORMAT, QVariant(format.toInt()));
+
+	sample_format = AF_SAMPFMT_UNSIGNED;
+	qDebug("auto-switching to unsigned format");
+    } else if ((bits > 8) && (sample_format != AF_SAMPFMT_TWOSCOMP)) {
+	const SampleFormat format(SampleFormat::Signed);
+	info.set(INF_SAMPLE_FORMAT, QVariant(format.toInt()));
+
+	sample_format = AF_SAMPFMT_TWOSCOMP;
+	qDebug("auto-switching to signed format");
     }
 
     // open the output device
@@ -394,27 +406,27 @@ bool WavEncoder::encode(QWidget *widget, MultiTrackReader &src,
 
 	switch (outfile.lastError()) {
 	    case AF_BAD_NOT_IMPLEMENTED:
-	        reason = i18n("format or function is not implemented") /*+
+	        reason = i18n("Format or function is not implemented") /*+
 		         "\n("+format_name+")"*/;
 	        break;
 	    case AF_BAD_MALLOC:
-	        reason = i18n("out of memory");
+	        reason = i18n("Out of memory");
 	        break;
 	    case AF_BAD_HEADER:
-	        reason = i18n("file header is damaged");
+	        reason = i18n("File header is damaged");
 	        break;
 	    case AF_BAD_CODEC_TYPE:
-	        reason = i18n("invalid codec type")/* +
+	        reason = i18n("Invalid codec type")/* +
 		         "\n("+format_name+")"*/;
 	        break;
 	    case AF_BAD_OPEN:
-	        reason = i18n("opening the file failed");
+	        reason = i18n("Opening the file failed");
 	        break;
 	    case AF_BAD_READ:
-	        reason = i18n("read access failed");
+	        reason = i18n("Read access failed");
 	        break;
 	    case AF_BAD_SAMPFMT:
-	        reason = i18n("invalid sample format");
+	        reason = i18n("Invalid sample format");
 	        break;
 	    default:
 		reason = reason.number(outfile.lastError());
