@@ -25,13 +25,13 @@
 #include "FileInfoDialog.h"
 #include "FileInfoPlugin.h"
 
-KWAVE_PLUGIN(FileInfoPlugin,"fileinfo","2.1","Thomas Eschenbacher");
+KWAVE_PLUGIN(FileInfoPlugin, "fileinfo", "2.1",
+             I18N_NOOP("File Info"), "Thomas Eschenbacher");
 
 //***************************************************************************
 FileInfoPlugin::FileInfoPlugin(const PluginContext &context)
     :Kwave::Plugin(context)
 {
-    i18n("fileinfo");
 }
 
 //***************************************************************************
@@ -78,24 +78,36 @@ void FileInfoPlugin::apply(FileInfo &new_info)
 		 "the whole file to the new sample rate or do "
 		 "you only want to set the rate information in order "
 		 "to repair a damaged file? Note: changing only the sample "
-		 "rate can cause \"mickey mouse\" effects!"),
+		 "rate can cause \"mickey mouse\" effects."),
 	    0,
 	    i18n("&Convert"),
 	    i18n("&Set Rate"));
 	if (res == KMessageBox::Yes) {
-	    // resample
-	    emitCommand(QString("convert_rate(%1)").arg(new_rate));
+	    // Yes -> resample
+	    
+	    // take over all properties except the new sample rate, this will
+	    // be detected and changed in the sample rate plugin
+	    new_info.setRate(fileInfo().rate());
+	    signalManager().setFileInfo(new_info, true);
+
+	    // NOTE: this command could be executed asynchronously, thus
+	    //       we cannot change the sample rate afterwards
+	    emitCommand(QString(
+		"plugin:execute(samplerate,%1,all)").arg(new_rate)
+	    );
+
+	    return;
 	} else if (res == KMessageBox::No) {
-	    // change the rate only
-	    // fileInfo().setRate(new_rate);
+	    // No -> only change the rate in the file info
+	    new_info.setRate(new_rate);
 	} else {
-	    // canceled -> use old setting
+	    // canceled -> use old sample rate
 	    new_info.setRate(fileInfo().rate());
 	}
     }
 
     // just copy all other properties
-    signalManager().setFileInfo(new_info);
+    signalManager().setFileInfo(new_info, true);
 }
 
 //***************************************************************************
