@@ -29,13 +29,13 @@
 #include "GotoPlugin.h"
 #include "GotoDialog.h"
 
-KWAVE_PLUGIN(GotoPlugin,"goto","2.1","Thomas Eschenbacher");
+KWAVE_PLUGIN(GotoPlugin, "goto", "2.1",
+             I18N_NOOP("Goto Position"), "Thomas Eschenbacher");
 
 //***************************************************************************
 GotoPlugin::GotoPlugin(const PluginContext &c)
     :Kwave::Plugin(c), m_mode(SelectTimeWidget::bySamples), m_position(0)
 {
-     i18n("goto");
 }
 
 //***************************************************************************
@@ -51,9 +51,7 @@ QStringList *GotoPlugin::setup(QStringList &previous_params)
 
     // create the setup dialog
     double rate = signalRate();
-    unsigned int offset = 0;
-    selection(&offset, 0, false);
-    unsigned int length = signalLength();
+    sample_index_t length = signalLength();
 
     GotoDialog *dialog = new GotoDialog(parentWidget(),
         m_mode, m_position, rate, length);
@@ -90,7 +88,7 @@ int GotoPlugin::start(QStringList &params)
     if (result) return result;
 
     // get current offset of the signal
-    unsigned int offset = SelectTimeWidget::timeToSamples(
+    sample_index_t offset = SelectTimeWidget::timeToSamples(
 	m_mode, m_position, signalRate(), signalLength());
 
     // change the selection through the signal manager
