@@ -30,7 +30,8 @@
 #include "NoisePlugin.h"
 #include "NoiseGenerator.h"
 
-KWAVE_PLUGIN(NoisePlugin,"noise","2.1","Thomas Eschenbacher");
+KWAVE_PLUGIN(NoisePlugin, "noise", "2.1",
+             I18N_NOOP("Noise Generator"), "Thomas Eschenbacher");
 
 //***************************************************************************
 NoisePlugin::NoisePlugin(const PluginContext &context)
@@ -41,42 +42,41 @@ NoisePlugin::NoisePlugin(const PluginContext &context)
 //***************************************************************************
 void NoisePlugin::run(QStringList)
 {
-    unsigned int first, last;
+    sample_index_t first, last;
+    QList<unsigned int> tracks;
 
-    UndoTransactionGuard undo_guard(*this, i18n("noise"));
+    UndoTransactionGuard undo_guard(*this, i18n("Noise"));
 
-    selection(&first, &last, true);
+    selection(&tracks, &first, &last, true);
 
     // create all objects
-    unsigned int tracks = selectedTracks().count();
-    Kwave::MultiTrackSource<NoiseGenerator, true> source(tracks);
-    MultiTrackWriter sink(signalManager(), selectedTracks(), Overwrite,
+    Kwave::MultiTrackSource<NoiseGenerator, true> source(tracks.count());
+    Kwave::MultiTrackWriter sink(signalManager(), tracks, Overwrite,
         first, last);
 
     // break if aborted
     if (!sink.tracks()) return;
 
     // connect the progress dialog
-    connect(&sink, SIGNAL(progress(unsigned int)),
-	    this,  SLOT(updateProgress(unsigned int)),
+    connect(&sink, SIGNAL(progress(qreal)),
+	    this,  SLOT(updateProgress(qreal)),
 	     Qt::BlockingQueuedConnection);
 
     // connect them
     if (!Kwave::connect(source, SIGNAL(output(Kwave::SampleArray)),
                         sink,   SLOT(input(Kwave::SampleArray))))
     {
-	close();
 	return;
     }
 
     // transport the samples
-    qDebug("NoisePlugin: filter started [%u ... %u] ...", first, last);
+    qDebug("NoisePlugin: filter started [%lu ... %lu] ...",
+	   static_cast<unsigned long int>(first),
+	   static_cast<unsigned long int>(last));
     while (!shouldStop() && !sink.done()) {
 	source.goOn();
     }
     qDebug("NoisePlugin: filter done.");
-
-    close();
 }
 
 //***************************************************************************
