@@ -51,26 +51,26 @@ public:
 signals:
 
     /** emits a block with the filtered data */
-    void output(Kwave::SampleArray &data);
+    void output(Kwave::SampleArray data);
 
 public slots:
 
     /** receives input data */
-    void input(Kwave::SampleArray &data);
+    void input(Kwave::SampleArray data);
 
     /**
      * Sets the center frequency, normed to [0...2Pi]. The calculation is:
      * fc = frequency [Hz] * 2 * Pi / f_sample [Hz].
      * The default setting is 0.5.
      */
-    void setFrequency(const QVariant &fc);
+    void setFrequency(const QVariant fc);
 
     /**
      * Sets the bandwidth, normed to [0...2Pi]. The calculation is:
      * bw = bandwidth [Hz] * 2 * Pi / f_sample [Hz].
      * The default setting is 0.1.
      */
-    void setBandwidth(const QVariant &bw);
+    void setBandwidth(const QVariant bw);
 
 private:
 
