@@ -61,7 +61,8 @@
 #include "PlayBackDialog.h"
 #include "PlayBackPlugin.h"
 
-KWAVE_PLUGIN(PlayBackPlugin,"playback","2.2","Thomas Eschenbacher");
+KWAVE_PLUGIN(PlayBackPlugin, "playback", "2.2",
+             I18N_NOOP("Playback"), "Thomas Eschenbacher");
 
 /** Sets the number of screen refreshes per second when in playback mode */
 #define SCREEN_REFRESHES_PER_SECOND 16
@@ -82,15 +83,14 @@ PlayBackPlugin::PlayBackPlugin(const PluginContext &context)
 
     connect(this, SIGNAL(sigPlaybackDone()),
             &m_playback_controller, SLOT(playbackDone()));
-    connect(this, SIGNAL(sigPlaybackPos(unsigned int)),
-            &m_playback_controller, SLOT(updatePlaybackPos(unsigned int)));
+    connect(this, SIGNAL(sigPlaybackPos(sample_index_t)),
+            &m_playback_controller, SLOT(updatePlaybackPos(sample_index_t)));
     connect(this, SIGNAL(sigPlaybackDone()),
             this, SLOT(closeDevice()),
             Qt::QueuedConnection);
 
     // register as a factory for playback devices
     manager().registerPlaybackDeviceFactory(this);
-    i18n("playback");
 }
 
 //***************************************************************************
@@ -467,7 +467,7 @@ PlayBackDevice *PlayBackPlugin::openDevice(const QString &name,
 
 	// show an error message box
 	Kwave::MessageBox::error(parentWidget(), result,
-	    i18n("unable to open '%1'",
+	    i18n("Unable to open '%1'",
 	    params.device.section('|',0,0)));
     }
 
@@ -512,8 +512,8 @@ void PlayBackPlugin::startDevicePlayBack()
 	return;
     }
 
-    unsigned int first;
-    unsigned int last;
+    sample_index_t first;
+    sample_index_t last;
     selection(0, &first, &last, false);
 
     if (m_playback_controller.paused()) {
@@ -708,7 +708,7 @@ void PlayBackPlugin::testPlayBack()
     // check if we really have selected a playback device
     if (!playback_params.device.length()) {
 	Kwave::MessageBox::sorry(m_dialog, i18n(
-	    "please select a playback device first"));
+	    "Please select a playback device first"));
 	return;
     }
 
@@ -795,8 +795,8 @@ void PlayBackPlugin::testPlayBack()
 	    progress->setValue(1);
 	    progress->setLabelText(
 		"<html><p><br>" +
-		i18n("you should now hear a %1Hz test tone...<br><br>"\
-		     "(if you hear clicks or dropouts, please increase<br>"\
+		i18n("You should now hear a %1 Hz test tone.<br><br>"\
+		     "(If you hear clicks or dropouts, please increase<br>"\
 		     "the buffer size and try again)", static_cast<int>(freq)) +
 		"</p></html>"
 	    );
