@@ -136,9 +136,13 @@ RecordDialog::RecordDialog(QWidget *parent, QStringList &params,
     connect(sbRecordPre,  SIGNAL(valueChanged(int)),
             this,         SLOT(preRecordingTimeChanged(int)));
 
-    // record time
+    // record time (duration)
     STD_SETUP(record_time_limited, record_time, RecordTime);
 
+    // start time (date & time)
+    chkRecordStartTime->setChecked(m_params.start_time_enabled);
+    startTime->setDateTime(m_params.start_time);
+
     // record trigger
     STD_SETUP_SLIDER(record_trigger_enabled, record_trigger, RecordTrigger);
 
@@ -205,11 +209,17 @@ RecordDialog::RecordDialog(QWidget *parent, QStringList &params,
     connect(chkDisplayLevelMeter, SIGNAL(toggled(bool)),
             this, SLOT(displayLevelMeterChecked(bool)));
 
+    // setup controls
     connect(chkRecordTime, SIGNAL(toggled(bool)),
             this, SLOT(recordTimeChecked(bool)));
     connect(sbRecordTime, SIGNAL(valueChanged(int)),
             this, SLOT(recordTimeChanged(int)));
 
+    connect(chkRecordStartTime, SIGNAL(toggled(bool)),
+            this, SLOT(startTimeChecked(bool)));
+    connect(startTime, SIGNAL(valueChanged(const QDateTime &)),
+            this, SLOT(startTimeChanged(const QDateTime &)));
+
     connect(chkRecordTrigger, SIGNAL(toggled(bool)),
             this, SLOT(triggerChecked(bool)));
     connect(sbRecordTrigger, SIGNAL(valueChanged(int)),
@@ -242,6 +252,10 @@ RecordDialog::RecordDialog(QWidget *parent, QStringList &params,
     connect(btRecord, SIGNAL(clicked()),
             controller, SLOT(actionStart()));
 
+    // stop recording when the window gets closed
+    connect(this, SIGNAL(rejected()), controller, SLOT(actionStop()));
+    connect(this, SIGNAL(accepted()), controller, SLOT(actionStop()));
+
     // connect the notifications/commands of the record controller
     connect(controller, SIGNAL(stateChanged(RecordState)),
             this, SLOT(setState(RecordState )));
@@ -583,7 +597,7 @@ void RecordDialog::selectRecordDevice()
     filter += QString("\nadsp*|") + i18n("ALSA record device (adsp*)");
     filter += QString("\n*|") + i18n("Any device (*)");
 
-    KwaveFileDialog dlg(":<kwave_record_device>", filter, this,
+    KwaveFileDialog dlg("kfiledialog:///kwave_record_device", filter, this,
 	true, "file:/dev");
     dlg.setKeepLocation(true);
     dlg.setOperationMode(KFileDialog::Opening);
@@ -957,7 +971,7 @@ void RecordDialog::setState(RecordState state)
     m_state = state;
     switch (state) {
 	case REC_UNINITIALIZED:
-	    state_text = i18n("please check the source device settings...");
+	    state_text = i18n("Please check the source device settings...");
 	    enable_new      = true;
 	    enable_pause    = false;
 	    enable_stop     = false;
@@ -980,7 +994,7 @@ void RecordDialog::setState(RecordState state)
 	    lbl_state->changeItem("", ID_TIME);
 	    break;
 	case REC_BUFFERING:
-	    state_text = i18n("buffering...");
+	    state_text = i18n("Buffering...");
 	    enable_new      = true; /* throw away current FIFO content */
 	    enable_pause    = false;
 	    enable_stop     = true;
@@ -991,7 +1005,7 @@ void RecordDialog::setState(RecordState state)
 	    pixmaps.push_back(QPixmap(ledlightgreen_xpm));
 	    break;
 	case REC_PRERECORDING:
-	    state_text = i18n("prerecording...");
+	    state_text = i18n("Prerecording...");
 	    enable_new      = false;
 	    enable_pause    = false;
 	    enable_stop     = true;
@@ -1002,7 +1016,7 @@ void RecordDialog::setState(RecordState state)
 	    pixmaps.push_back(QPixmap(ledlightgreen_xpm));
 	    break;
 	case REC_WAITING_FOR_TRIGGER:
-	    state_text = i18n("waiting for trigger...");
+	    state_text = i18n("Waiting for trigger...");
 	    enable_new      = false;
 	    enable_pause    = false;
 	    enable_stop     = true;
@@ -1013,7 +1027,7 @@ void RecordDialog::setState(RecordState state)
 	    pixmaps.push_back(QPixmap(ledlightgreen_xpm));
 	    break;
 	case REC_RECORDING:
-	    state_text = i18n("recording...");
+	    state_text = i18n("Recording...");
 	    enable_new      = false;
 	    enable_pause    = true;
 	    enable_stop     = true;
@@ -1031,7 +1045,7 @@ void RecordDialog::setState(RecordState state)
 	    animation_time = 100;
 	    break;
 	case REC_PAUSED:
-	    state_text = i18n("paused");
+	    state_text = i18n("Paused");
 	    enable_new      = true; /* start again */
 	    enable_pause    = true; /* used for "continue" */
 	    enable_stop     = true;
@@ -1042,7 +1056,7 @@ void RecordDialog::setState(RecordState state)
 	    pixmaps.push_back(QPixmap(ledyellow_xpm));
 	    break;
 	case REC_DONE:
-	    state_text = i18n("done");
+	    state_text = i18n("Done");
 	    enable_new      = true;
 	    enable_pause    = false;
 	    enable_stop     = false;
@@ -1069,6 +1083,7 @@ void RecordDialog::setState(RecordState state)
     slRecordPre->setEnabled(enable_settings &&
                             chkRecordPre->isChecked());
 
+    chkRecordStartTime->setEnabled(enable_settings);
     chkRecordTime->setEnabled(enable_settings);
     sbRecordTime->setEnabled(enable_settings &&
                              chkRecordTime->isChecked());
@@ -1079,6 +1094,8 @@ void RecordDialog::setState(RecordState state)
                                 chkRecordTrigger->isChecked());
     slRecordTrigger->setEnabled(enable_trigger &&
                                 chkRecordTrigger->isChecked());
+    startTime->setEnabled(enable_settings &&
+                          chkRecordStartTime->isChecked());
 
     grpFormat->setEnabled(enable_settings);
     grpSource->setEnabled(enable_settings);
@@ -1116,9 +1133,48 @@ void RecordDialog::updateBufferState(unsigned int count, unsigned int total)
 	case REC_EMPTY:
 	case REC_BUFFERING:
 	case REC_PRERECORDING:
-	case REC_WAITING_FOR_TRIGGER:
 	    txt = "";
 	    break;
+	case REC_WAITING_FOR_TRIGGER: {
+	    txt = "";
+	    QString state_text;
+	    QDateTime now     = QDateTime::currentDateTime();
+	    QDateTime t_start = m_params.start_time;
+
+	    if (m_params.start_time_enabled && (now < t_start)) {
+		// waiting for start time to come...
+
+		int s = now.secsTo(t_start);
+		int m = s / 60;
+		s %= 60;
+		int h = m / 60;
+		m %= 60;
+		int d = h / 24;
+		h %= 24;
+
+		QString days    =
+		    (d) ? i18np("one day ",    "%1 days ",    d) : "";
+		QString hours   =
+		    (h) ? i18np("one hour ",   "%1 hours ",   h) : "";
+		QString minutes =
+		    (m) ? i18np("one minute ", "%1 minutes ", m) : "";
+		QString seconds =
+		    (d | h | m) ?
+		    i18np("and %1 second", "and %1 seconds", s) :
+		    i18np("%1 second", "%1 seconds", s);
+
+		state_text = i18nc(
+		    "%1=days; %2=hours; %3=minutes; %4=seconds",
+		    "Waiting for start in %1%2%3%4...",
+		    days, hours, minutes, seconds);
+	    } else {
+		// waiting for trigger...
+		state_text = i18n("Waiting for trigger...");
+	    }
+	    lbl_state->changeItem(state_text, ID_STATE);
+
+	    break;
+	}
 	case REC_RECORDING:
 	case REC_PAUSED:
 	case REC_DONE: {
@@ -1166,11 +1222,29 @@ void RecordDialog::recordTimeChanged(int limit)
     updateRecordButton();
 }
 
+//***************************************************************************
+void RecordDialog::startTimeChecked(bool enabled)
+{
+    m_params.start_time_enabled = enabled;
+    emit sigTriggerChanged(enabled || m_params.record_trigger_enabled);
+}
+
+//***************************************************************************
+void RecordDialog::startTimeChanged(const QDateTime &datetime)
+{
+    m_params.start_time = datetime;
+
+    // force seconds to zero
+    QTime t = m_params.start_time.time();
+    t.setHMS(t.hour(), t.minute(), 0, 0);
+    m_params.start_time.setTime(t);
+}
+
 //***************************************************************************
 void RecordDialog::triggerChecked(bool enabled)
 {
     m_params.record_trigger_enabled = enabled;
-    emit sigTriggerChanged(enabled);
+    emit sigTriggerChanged(enabled || m_params.start_time_enabled);
 }
 
 //***************************************************************************
