@@ -38,6 +38,7 @@
 
 #include "ui_RecordDlg.h"
 
+class QDateTime;
 class QTreeWidgetItem;
 class QWidget;
 
@@ -112,7 +113,7 @@ public:
 
     /**
      * sets the list of supported sample formats
-     * @param comps list of supported sample formats, must not be empty
+     * @param formats list of supported sample formats, must not be empty
      */
     void setSupportedSampleFormats(const QList<SampleFormat> &formats);
 
@@ -125,7 +126,7 @@ public:
 
     /**
      * updates the progress bar with the buffer fill state.
-     * @param filled number of filled/available buffers
+     * @param count number of filled/available buffers
      * @param total maximum amount of buffers
      */
     void updateBufferState(unsigned int count, unsigned int total);
@@ -256,6 +257,12 @@ private slots:
     /** record time has been changed */
     void recordTimeChanged(int record_time);
 
+    /** start time has been enabled/disabled */
+    void startTimeChecked(bool enabled);
+
+    /** start time has been changed */
+    void startTimeChanged(const QDateTime &datetime);
+
     /** record trigger has been enabled/disabled */
     void triggerChecked(bool enabled);
 
