@@ -41,8 +41,8 @@
 #include "libkwave/Sample.h"
 #include "libkwave/SampleFIFO.h"
 #include "libkwave/SampleFormat.h"
-#include "libkwave/SampleWriter.h"
 #include "libkwave/SignalManager.h"
+#include "libkwave/Writer.h"
 
 #include "RecordDevice.h"
 #include "RecordDialog.h"
@@ -53,7 +53,8 @@
 #include "Record-ALSA.h"
 #include "Record-OSS.h"
 
-KWAVE_PLUGIN(RecordPlugin,"record","2.2","Thomas Eschenbacher");
+KWAVE_PLUGIN(RecordPlugin, "record", "2.2",
+             I18N_NOOP("Record"), "Thomas Eschenbacher");
 
 //***************************************************************************
 RecordPlugin::RecordPlugin(const PluginContext &context)
@@ -63,7 +64,6 @@ RecordPlugin::RecordPlugin(const PluginContext &context)
      m_writers(0), m_buffers_recorded(0), m_inhibit_count(0),
      m_trigger_value()
 {
-    i18n("record");
 }
 
 //***************************************************************************
@@ -127,9 +127,14 @@ QStringList *RecordPlugin::setup(QStringList &previous_params)
             this,     SLOT(buffersChanged()));
     connect(this,     SIGNAL(sigRecordedSamples(unsigned int)),
             m_dialog, SLOT(setRecordedSamples(unsigned int)));
+
     connect(m_dialog,      SIGNAL(sigTriggerChanged(bool)),
             &m_controller, SLOT(enableTrigger(bool)));
-    m_controller.enableTrigger(m_dialog->params().record_trigger_enabled);
+    m_controller.enableTrigger(
+	m_dialog->params().record_trigger_enabled ||
+	m_dialog->params().start_time_enabled
+    );
+
     connect(m_dialog, SIGNAL(sigPreRecordingChanged(bool)),
             &m_controller, SLOT(enablePrerecording(bool)));
     connect(m_dialog, SIGNAL(sigPreRecordingChanged(bool)),
@@ -375,7 +380,7 @@ void RecordPlugin::setDevice(const QString &device)
 		    reason = i18n(
 			"Kwave was unable to open the device '%1'.\n"\
 			"Maybe your system lacks support for the corresponding "\
-			" hardware or the hardware is not connected.",
+			"hardware or the hardware is not connected.",
 			short_device_name);
 		    break;
 		case -EBUSY:
@@ -388,7 +393,7 @@ void RecordPlugin::setDevice(const QString &device)
 	    }
 
 	    if (reason.length()) Kwave::MessageBox::sorry(parentWidget(),
-		reason, i18n("unable to open the recording device"));
+		reason, i18n("Unable to open the recording device"));
 	}
 
 	m_device_name = QString::null;
@@ -498,8 +503,8 @@ void RecordPlugin::changeSampleRate(double new_rate)
 	if ((static_cast<int>(new_rate) > 0) &&
 	    (static_cast<int>(rate) > 0) &&
 	    (static_cast<int>(new_rate) != static_cast<int>(rate)))
-	    notice(i18n("%1Hz is not supported, "\
-		        "using %2Hz", sr1, sr2));
+	    notice(i18n("%1 Hz is not supported, "\
+		        "using %2 Hz", sr1, sr2));
     }
     m_dialog->setSupportedSampleRates(supported_rates);
 
@@ -514,7 +519,7 @@ void RecordPlugin::changeSampleRate(double new_rate)
 	if ((static_cast<int>(new_rate) > 0) &&
 	    (static_cast<int>(rate) > 0) &&
 	    (static_cast<int>(new_rate) != static_cast<int>(rate)))
-	    notice(i18n("%1Hz failed, using %2Hz", sr1, sr2));
+	    notice(i18n("%1 Hz failed, using %2 Hz", sr1, sr2));
     }
     m_dialog->setSampleRate(rate);
 
@@ -556,7 +561,7 @@ void RecordPlugin::changeCompression(int new_compression)
 	if (compression != new_compression) {
 	    const QString c1(types.name(types.findFromData(new_compression)));
 	    const QString c2(types.name(types.findFromData(compression)));
-	    notice(i18n("compression '%1' not supported, using '%2'",
+	    notice(i18n("Compression '%1' not supported, using '%2'",
                         c1, c2));
 	}
     }
@@ -573,7 +578,7 @@ void RecordPlugin::changeCompression(int new_compression)
 	    const QString c1(types.name(types.findFromData(compression)));
 	    const QString c2(types.name(types.findFromData(
 	                 m_device->compression())));
-	    notice(i18n("compression %1 failed, using %2.", c1 ,c2));
+	    notice(i18n("Compression '%1' failed, using '%2'.", c1 ,c2));
 	}
     }
     m_dialog->setCompression(compression);
@@ -664,7 +669,7 @@ void RecordPlugin::changeSampleFormat(SampleFormat new_format)
 	const QString s1 = sf.name(sf.findFromData(new_format));
 	const QString s2 = sf.name(sf.findFromData(format));
 	if (!(new_format == -1) && !(new_format == format)) {
-	    notice(i18n("sample format '%1' is not supported, "\
+	    notice(i18n("Sample format '%1' is not supported, "\
 		        "using '%2'", s1, s2));
 	}
     }
@@ -680,7 +685,7 @@ void RecordPlugin::changeSampleFormat(SampleFormat new_format)
 	const QString s1 = sf.name(sf.findFromData(new_format));
 	const QString s2 = sf.name(sf.findFromData(format));
 	if (format > 0) notice(
-	    i18n("sample format '%1' failed, using '%2'", s1, s2));
+	    i18n("Sample format '%1' failed, using '%2'", s1, s2));
     }
     m_dialog->setSampleFormat(format);
 }
@@ -898,7 +903,7 @@ void RecordPlugin::startRecording()
 
 	    // create a sink for our audio data
 	    if (m_writers) delete m_writers;
-	    m_writers = new MultiTrackWriter(signalManager(), Append);
+	    m_writers = new Kwave::MultiTrackWriter(signalManager(), Append);
 	    Q_ASSERT(m_writers);
 	    Q_ASSERT((m_writers) && (m_writers->tracks() == tracks));
 	    if ((!m_writers) || (m_writers->tracks() != tracks)) {
@@ -956,18 +961,19 @@ void RecordPlugin::recordStopped(int reason)
 	                       "number and/or size of the record buffers.");
 	    break;
 	case -EBUSY:
-	    description = i18n("The recording device seems to be busy!");
+	    description = i18n("The recording device seems to be busy.");
 	    break;
 	default:
-	    description = i18n("Reading from the recording device failed, "\
-	                       "error number = %1 (%2)", -reason,
+	    description = i18n("Reading from the recording device failed. "\
+	                       "Error number = %1 (%2)", -reason,
 			       QString::fromLocal8Bit(strerror(-reason)));
     }
     Kwave::MessageBox::error(m_dialog, description);
 
     if (m_writers) m_writers->flush();
-    qDebug("RecordPlugin::recordStopped(): last=%u",
-           (m_writers) ? m_writers->last() : 0);
+    qDebug("RecordPlugin::recordStopped(): last=%lu",
+	   static_cast<unsigned long int>(
+           (m_writers) ? m_writers->last() : 0));
 
     // flush away all prerecording buffers
     m_prerecording_queue.clear();
@@ -1038,33 +1044,121 @@ void RecordPlugin::split(QByteArray &raw_data, QByteArray &dest,
                          unsigned int tracks)
 {
     unsigned int samples = raw_data.size() / bytes_per_sample / tracks;
-    const unsigned int increment = (tracks-1) * bytes_per_sample;
-    char *src = raw_data.data();
-    char *dst = dest.data();
-    src += (track * bytes_per_sample);
 
 #if 0
-    static int saw[2] = {0, 0};
+    // simple sawtooth generator, based on raw data
+    // works for up to 16 channels
+    static int saw[16] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+    raw_data.fill(0x05);
+    for (unsigned int s = 0; s < samples; s++) {
+	int v = saw[track];
+	for (unsigned int byte = 0; byte < bytes_per_sample; byte++) {
+	    u_int8_t x = (u_int8_t)v;
+	    raw_data[(((s * tracks) + track) * bytes_per_sample) + byte] = x;
+	    v >>= 8;
+	}
 
-    // simple sawtooth generator, nice for debugging
-    while (samples--) {
-	for (unsigned int byte=0; byte < bytes_per_sample; byte++) {
-	    *dst = (saw[track] / 128);
-	    dst++;
-	}
-	saw[track]++;
-	if (saw[track] > (SAMPLE_MAX/1024)) saw[track] = (SAMPLE_MIN/1024);
-    }
-#else
-    while (samples--) {
-	for (unsigned int byte=0; byte < bytes_per_sample; byte++) {
-	    *dst = *src;
-	    dst++;
-	    src++;
-	}
-	src += increment;
+	const int max = (1 << ((bytes_per_sample * 8) - 1)) - 1;
+	saw[track] += max / 64;
+	if (saw[track] >= max) saw[track] = 0;
     }
 #endif
+
+    if (tracks == 1) {
+	// this would give a 1:1 memcpy
+	dest = raw_data;
+    } else {
+	switch (bytes_per_sample) {
+	    case 1: {
+		// 1...8 bits per sample, use 8 bit pointers
+		const u_int8_t *src =
+		    reinterpret_cast<u_int8_t *>(raw_data.data());
+		u_int8_t *dst =
+		    reinterpret_cast<u_int8_t *>(dest.data());
+		src += track;
+		while (samples--) {
+		    *dst = *src;
+		    dst++;
+		    src += tracks;
+		}
+		break;
+	    }
+	    case 2: {
+		// 9...16 bits per sample, use 16 bit pointers
+		const u_int16_t *src =
+		    reinterpret_cast<const u_int16_t *>(raw_data.data());
+		u_int16_t *dst =
+		    reinterpret_cast<u_int16_t *>(dest.data());
+		src += track;
+		while (samples--) {
+		    *dst = *src;
+		    dst++;
+		    src += tracks;
+		}
+		break;
+	    }
+	    case 3: {
+		// 17...24 bits per sample, use 8 bit pointers, three times
+		const u_int8_t *src =
+		    reinterpret_cast<u_int8_t *>(raw_data.data());
+		u_int8_t *dst =
+		    reinterpret_cast<u_int8_t *>(dest.data());
+		src += track * 3;
+		while (samples--) {
+		    *(dst++) = *(src++);
+		    *(dst++) = *(src++);
+		    *(dst++) = *(src++);
+		    src += (tracks - 1) * 3;
+		}
+		break;
+	    }
+	    case 4: {
+		// 24...32 bits per sample, use 32 bit pointers
+		const u_int32_t *src =
+		    reinterpret_cast<u_int32_t *>(raw_data.data());
+		u_int32_t *dst =
+		    reinterpret_cast<u_int32_t *>(dest.data());
+		src += track;
+		while (samples--) {
+		    *dst = *src;
+		    dst++;
+		    src += tracks;
+		}
+		break;
+	    }
+	    case 8: {
+		// 64 bits per sample, use 64 bit pointers
+		const u_int64_t *src =
+		    reinterpret_cast<const u_int64_t *>(raw_data.data());
+		u_int64_t *dst =
+		    reinterpret_cast<u_int64_t *>(dest.data());
+		src += track;
+		while (samples--) {
+		    *dst = *src;
+		    dst++;
+		    src += tracks;
+		}
+		break;
+	    }
+	    default: {
+		// default: bytewise operation
+		const u_int8_t *src =
+		    reinterpret_cast<const u_int8_t *>(raw_data.data());
+		u_int8_t *dst =
+		    reinterpret_cast<u_int8_t *>(dest.data());
+		src += (track * bytes_per_sample);
+		unsigned int increment = (tracks - 1) * bytes_per_sample;
+		while (samples--) {
+		    for (unsigned int b = 0; b < bytes_per_sample; b++) {
+			*dst = *src;
+			dst++;
+			src++;
+		    }
+		    src += increment;
+		}
+	    }
+	}
+    }
 }
 
 //***************************************************************************
@@ -1073,14 +1167,22 @@ bool RecordPlugin::checkTrigger(unsigned int track,
 {
     Q_ASSERT(m_dialog);
     if (!m_dialog) return false;
+
+    // check if the recording start time has been reached
+    if (m_dialog->params().start_time_enabled) {
+	if (QDateTime::currentDateTime() < m_dialog->params().start_time)
+	    return false;
+    }
+
+    // shortcut if no trigger has been set
+    if (!m_dialog->params().record_trigger_enabled) return true;
+
+    // check the input parameters
     if (!buffer.size()) return false;
     if (!m_writers) return false;
     if (m_trigger_value.size() != static_cast<int>(m_writers->tracks()))
 	return false;
 
-    // shortcut if no trigger has been set
-    if (!m_dialog->params().record_trigger_enabled) return true;
-
     // pass the buffer through a rectifier and a lowpass with
     // center frequency about 2Hz to get the amplitude
     float trigger = static_cast<float>(
@@ -1186,7 +1288,7 @@ void RecordPlugin::flushPrerecordingQueue()
 	fifo.crop(); // enforce the correct size
 
 	// push all buffers to the writer, starting at the tail
-	SampleWriter *writer = (*m_writers)[track];
+	Kwave::Writer *writer = (*m_writers)[track];
 	Q_ASSERT(writer);
 	if (writer) {
 	    Kwave::SampleArray buffer(writer->blockSize());
@@ -1194,7 +1296,7 @@ void RecordPlugin::flushPrerecordingQueue()
 	    while (rest) {
 		unsigned int read = fifo.get(buffer);
 		if (read < 1) break;
-		writer->flush(buffer, read);
+		writer->write(buffer, read);
 		rest -= read;
 	    }
 	} else {
@@ -1270,7 +1372,8 @@ void RecordPlugin::processBuffer()
     // note: this might change the state, which affects the
     //       processing of all tracks !
     if ((m_state == REC_WAITING_FOR_TRIGGER) ||
-        ((m_state == REC_PRERECORDING) && (params.record_trigger_enabled)))
+        ((m_state == REC_PRERECORDING) && (params.record_trigger_enabled)) ||
+        ((m_state == REC_PRERECORDING) && (params.start_time_enabled)))
     {
 	for (unsigned int track=0; track < tracks; ++track) {
 	    // split off and decode buffer with current track
@@ -1329,7 +1432,7 @@ void RecordPlugin::processBuffer()
 		Q_ASSERT(tracks == m_writers->tracks());
 		if (!tracks || (tracks != m_writers->tracks())) break;
 
-		SampleWriter *writer = (*m_writers)[track];
+		Kwave::Writer *writer = (*m_writers)[track];
 		Q_ASSERT(writer);
 		if (writer) (*writer) << decoded;
 		m_controller.setEmpty(false);
