@@ -44,7 +44,7 @@
 //***************************************************************************
 SonagramDialog::SonagramDialog(Kwave::Plugin &p)
     :QDialog(p.parentWidget()), Ui::SonagramDlg(),
-    m_length(p.selection()), m_rate(p.signalRate())
+    m_length(p.selection(0, 0, 0,true)), m_rate(p.signalRate())
 {
     setupUi(this);
     setModal(true);
@@ -56,9 +56,6 @@ SonagramDialog::SonagramDialog(Kwave::Plugin &p)
     if (!pointslider) return;
     if (!windowtypebox) return;
 
-    // if nothing selected, select all
-    if (m_length <= 1) m_length = p.signalLength();
-
     pointslider->setMaximum(m_length / 16);
 
     window_function_t wf = WINDOW_FUNC_NONE;
@@ -108,7 +105,7 @@ void SonagramDialog::parameters(QStringList &list)
     list.clear();
 
     // parameter #0: number of fft points
-    param = pointbox ? pointbox->currentText() : QString(0);
+    param = pointbox ? pointbox->currentText() : QString();
     list.append(param);
 
     // parameter #1: index of the window function
@@ -143,10 +140,10 @@ void SonagramDialog::setPoints(int points)
     text.setNum(points);
     pointbox->setEditText(text);
 
-    windowlabel->setText(i18n("( resulting window size: %1 )",
+    windowlabel->setText(i18n("(resulting window size: %1)",
 	Kwave::Plugin::ms2string(points * 1.0E3 / m_rate)));
 
-    bitmaplabel->setText(i18n("size of bitmap: %1x%2",
+    bitmaplabel->setText(i18n("Size of bitmap: %1x%2",
 	(m_length / points) + 1,
 	points/2));
 }
