@@ -44,7 +44,8 @@
 #include "SonagramDialog.h"
 #include "SonagramWindow.h"
 
-KWAVE_PLUGIN(SonagramPlugin,"sonagram","2.1","Thomas Eschenbacher");
+KWAVE_PLUGIN(SonagramPlugin, "sonagram", "2.1",
+             I18N_NOOP("Sonagram"), "Thomas Eschenbacher");
 
 #define MAX_QUEUE_USAGE 256
 
@@ -78,7 +79,7 @@ SonagramPlugin::SonagramPlugin(const PluginContext &c)
     connect(this, SIGNAL(stripeAvailable(StripeInfoPrivate *)),
             this, SLOT(insertStripe(StripeInfoPrivate *)),
             Qt::BlockingQueuedConnection);
-    i18n("sonagram");
+    i18n("Sonagram");
 }
 
 //***************************************************************************
