@@ -110,11 +110,11 @@ SonagramWindow::SonagramWindow(const QString &name)
     if (!file) return ;
 
 //    bar->addAction(i18n("&Spectral Data"), spectral);
-//    file->addAction(i18n("&Import from Bitmap ..."), this, SLOT(load()));
+//    file->addAction(i18n("&Import from Bitmap..."), this, SLOT(load()));
 
     file->addAction(
 	icon_loader.loadIcon("document-export", KIconLoader::Small),
-	i18n("&Export to Bitmap ..."),
+	i18n("&Export to Bitmap..."),
 	this, SLOT(save())
     );
     file->addAction(
@@ -124,7 +124,7 @@ SonagramWindow::SonagramWindow(const QString &name)
 	QKeySequence::Close
     );
 
-//    spectral->addAction (i18n("&reTransform to signal"), this, SLOT(toSignal()));
+//    spectral->addAction (i18n("&Retransform to Signal"), this, SLOT(toSignal()));
 
     KStatusBar *status = statusBar();
     Q_ASSERT(status);
@@ -200,7 +200,7 @@ void SonagramWindow::save()
 {
     if (m_image.isNull()) return;
 
-    KwaveFileDialog dlg(":<kwave_sonagram>", QString(),
+    KwaveFileDialog dlg("kfiledialog:///kwave_sonagram", QString(),
         this, true, QString(), "*.bmp");
     dlg.setOperationMode(KFileDialog::Saving);
     dlg.setCaption(i18n("Save Sonagram"));
@@ -274,9 +274,9 @@ void SonagramWindow::setImage(QImage image)
 }
 
 //****************************************************************************
-void SonagramWindow::setOverView(QBitmap overview)
+void SonagramWindow::setOverView(const QImage &overview)
 {
-    m_overview->setImage(overview.toImage());
+    m_overview->setImage(overview);
 }
 
 //****************************************************************************
@@ -304,7 +304,7 @@ void SonagramWindow::insertStripe(const unsigned int stripe_nr,
 	m_histogram[p]--;
 
 	// set the new pixel value
-	p = (int)stripe[(size - 1) - y];
+	p = static_cast<int>(stripe[(size - 1) - y]);
 	m_image.setPixel(stripe_nr, y, p);
 
 	// insert the new pixel into the histogram
@@ -332,7 +332,7 @@ void SonagramWindow::adjustBrightness()
 	sum += m_histogram[i];
     }
     // cut off all parts below the cutoff ratio (e.g. 0.1%)
-    unsigned int cutoff = (int)(sum * COLOR_CUTOFF_RATIO);
+    unsigned int cutoff = static_cast<int>(sum * COLOR_CUTOFF_RATIO);
 
     // get the first used color
     unsigned int first=0;
@@ -382,6 +382,7 @@ void SonagramWindow::refresh_view()
 //****************************************************************************
 void SonagramWindow::toSignal()
 {
+/** @todo needs to be ported to fftw and re-activated */
 //    gsl_fft_complex_wavetable table;
 //
 //    gsl_fft_complex_wavetable_alloc (points, &table);
@@ -450,7 +451,8 @@ void SonagramWindow::translatePixels2TF(const QPoint p, double *ms, double *f)
     if (ms) {
 	// get the time coordinate [0...(N_samples-1)* (1/f_sample) ]
 	if (m_rate != 0) {
-	    *ms = (double)p.x() * (double)m_points * 1000.0 / m_rate;
+	    *ms = static_cast<double>(p.x()) *
+	          static_cast<double>(m_points) * 1000.0 / m_rate;
 	} else {
 	    *ms = 0;
 	}
@@ -473,8 +475,8 @@ void SonagramWindow::updateScaleWidgets()
 
     translatePixels2TF(QPoint(m_image.width()-1, 0), &ms, &f);
 
-    m_xscale->setMinMax(0, (int)rint(ms));
-    m_yscale->setMinMax(0, (int)rint(f));
+    m_xscale->setMinMax(0, static_cast<int>(rint(ms)));
+    m_yscale->setMinMax(0, static_cast<int>(rint(f)));
 }
 
 //***************************************************************************
@@ -497,11 +499,10 @@ void SonagramWindow::setColorMode(int mode)
 //***************************************************************************
 void SonagramWindow::setName(const QString &name)
 {
-    QString windowname;
-    windowname += i18n("Sonagram of ");
-    windowname += name.length() ? name : QString(i18n("<nothing>"));
-
-    setCaption(windowname);
+    setCaption((name.length()) ?
+	i18n("Sonagram of %1", name) :
+	i18n("Sonagram")
+    );
 }
 
 //****************************************************************************
@@ -522,10 +523,10 @@ void SonagramWindow::cursorPosChanged(const QPoint pos)
     translatePixels2TF(pos, &ms, &f);
 
     // item 1: time in milliseconds
-    status->changeItem(i18n("Time: %1", KwavePlugin::ms2string(ms)), 1);
+    status->changeItem(i18n("Time: %1", Kwave::Plugin::ms2string(ms)), 1);
 
     // item 2: frequency in Hz
-    QString text = i18n("Frequency: %1 Hz", (int)f);
+    QString text = i18n("Frequency: %1 Hz", static_cast<int>(f));
     status->changeItem(text, 2);
 
     // item 3: amplitude in %
@@ -535,7 +536,7 @@ void SonagramWindow::cursorPosChanged(const QPoint pos)
     } else {
 	a = 0.0;
     }
-    text = i18n("Amplitude: %1 %", (int)a);
+    text = i18n("Amplitude: %1%", static_cast<int>(a));
     status->changeItem(text, 3);
 }
 
